(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[    254562,       8450]
NotebookOptionsPosition[    205242,       7120]
NotebookOutlinePosition[    225108,       7537]
CellTagsIndexPosition[    224456,       7513]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["Advanced Counting Techniques", "BookChapterTitle",
 CounterAssignments->{{"ChapterNumber", 8}}],

Cell[CellGroupData[{

Cell["Introduction", "Section"],

Cell[TextData[{
 "In this chapter, we will describe how to apply ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to three important topics in counting: recurrence relations, generating \
functions, and inclusion-exclusion. We begin by describing how ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " can be used to solve recurrence relations, including the recurrence \
relations that describe the complexity of divide-and-conquer algorithms. \
After studying recurrence relations, we show how to use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to manipulate generating functions and how these capabilities can help \
solve counting problems. We conclude the chapter with a discussion of the \
principle of inclusion and exclusion."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["8.1 Recurrence Relations", "Section",
 CellTags->"Section1"],

Cell[TextData[{
 "A recurrence relation describes a relationship between the members of a \
sequence and their predecessors. For example, the famous Fibonacci sequence \
",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    SubscriptBox["f", "n"], "}"}], TraditionalForm]]],
 " satisfies the recurrence relation"
}], "Text"],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["f", "n"], "=", 
   RowBox[{
    SubscriptBox["f", 
     RowBox[{"n", "-", "1"}]], "+", 
    SubscriptBox["f", 
     RowBox[{"n", "-", "2"}]]}]}], TraditionalForm]], "Equation"],

Cell[TextData[{
 "Together with the initial conditions ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["f", "1"], "=", "1"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["f", "2"], "=", "1"}], TraditionalForm]]],
 ", this relation is sufficient to define the entire sequence ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    SubscriptBox["f", "n"], "}"}], TraditionalForm]]],
 "."
}], "Text"],

Cell[TextData[{
 "To understand how we can work with recurrence relations in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", we have to remember that a sequence ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    SubscriptBox["a", "n"], "}"}], TraditionalForm]]],
 " is a function whose domain is a subset of the integers (usually the \
positive integers or nonnegative integers, depending on the context) and \
whose codomain contains the terms of the sequence (which can be numbers, \
matrices, circles, functions, etc.). (See the definition of sequence given in \
Section 2.4 of the textbook.)"
}], "Text"],

Cell[TextData[{
 "With this point of view, the sequence ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    SubscriptBox["a", "n"], "}"}], TraditionalForm]]],
 " is a function ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " and the ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 "th term of the sequence is the value of the function evaluated at the \
integer ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ", that is, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["a", "n"], "=", 
    RowBox[{"a", "(", "n", ")"}]}], TraditionalForm]]],
 ". This is only a change in notation, but it makes it easier to see that a \
recurrence relation can be represented in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " as a function taking integer arguments."
}], "Text"],

Cell["\<\
We can represent the Fibonacci sequence by the indexed variable and function \
below, which we use to compute the first twenty terms of the Fibonacci \
sequence. The initial conditions are given as specific assignments to an \
indexed variable, and the function that implements the recurrence relation \
stores computed values by assigning them to the indexed variable.\
\>", "Text"],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"fibonacci", "[", "1", "]"}], "=", "1"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"fibonacci", "[", "2", "]"}], "=", "1"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"fibonacci", "[", "n_Integer", "]"}], ":=", 
   RowBox[{
    RowBox[{"fibonacci", "[", "n", "]"}], "=", 
    RowBox[{
     RowBox[{"fibonacci", "[", 
      RowBox[{"n", "-", "1"}], "]"}], "+", 
     RowBox[{"fibonacci", "[", 
      RowBox[{"n", "-", "2"}], "]"}]}]}]}], ";"}]}], "Input",
 InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Table", "[", 
  RowBox[{
   RowBox[{"fibonacci", "[", "n", "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"n", ",", "1", ",", "20"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "1", ",", "1", ",", "2", ",", "3", ",", "5", ",", "8", ",", "13", ",", "21",
    ",", "34", ",", "55", ",", "89", ",", "144", ",", "233", ",", "377", ",", 
   "610", ",", "987", ",", "1597", ",", "2584", ",", "4181", ",", "6765"}], 
  "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Using indexed variables and functions like the one above is the approach we \
developed in Section 5.3 of this manual to implement functions with recursive \
definitions. This is a useful general approach, but ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " has more specific, and more efficient, methods for working with recurrence \
relations."
}], "Text"],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "\[CloseCurlyQuote]s ",
 StyleBox[ButtonBox["RecurrenceTable",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/RecurrenceTable"], "FunctionLink"],
 " function is used to generate lists of the values of a sequence defined by \
a recurrence relation. ",
 StyleBox[ButtonBox["RecurrenceTable",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/RecurrenceTable"], "FunctionLink"],
 " requires three arguments. The first argument must be a list of equations \
specifying the recurrence relation and all initial conditions. Note that \
these must be given as equations, using ",
 StyleBox[ButtonBox["Equal",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Equal"], "FunctionLink"],
 " (",
 StyleBox["==", "Input"],
 "). Also, the equations should be in terms of indexed variables such as ",
 StyleBox["fibonacci[n]", "Input"],
 " or ",
 StyleBox["a[k-1]", "Input"],
 ", or ",
 StyleBox["b[0]", "Input"],
 ". The second argument is the symbol used to name the sequence in the \
equations, for example ",
 StyleBox["fibonacci", "Input"],
 " or ",
 StyleBox["a", "Input"],
 ". The final argument is a range specification of the same form as used in a \
",
 StyleBox[ButtonBox["Table",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Table"], "FunctionLink"],
 ". For example, ",
 StyleBox["{n,10,20}", "Input"],
 " would be used to generate the list of the values with ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "=", "10"}], TraditionalForm]]],
 " through ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "=", "20"}], TraditionalForm]]],
 " and ",
 StyleBox["{k,10}", "Input"],
 " would generate the values up to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", "=", "10"}], TraditionalForm]]],
 ". "
}], "Text"],

Cell["\<\
To illustrate, the following generates the first 20 terms of the Fibonacci \
sequence. \
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"RecurrenceTable", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"a", "[", "n", "]"}], "\[Equal]", 
      RowBox[{
       RowBox[{"a", "[", 
        RowBox[{"n", "-", "1"}], "]"}], "+", 
       RowBox[{"a", "[", 
        RowBox[{"n", "-", "2"}], "]"}]}]}], ",", 
     RowBox[{
      RowBox[{"a", "[", "1", "]"}], "\[Equal]", "1"}], ",", 
     RowBox[{
      RowBox[{"a", "[", "2", "]"}], "\[Equal]", "1"}]}], "}"}], ",", "a", ",", 
   RowBox[{"{", 
    RowBox[{"n", ",", "20"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "1", ",", "1", ",", "2", ",", "3", ",", "5", ",", "8", ",", "13", ",", "21",
    ",", "34", ",", "55", ",", "89", ",", "144", ",", "233", ",", "377", ",", 
   "610", ",", "987", ",", "1597", ",", "2584", ",", "4181", ",", "6765"}], 
  "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "For some common sequences like the Fibonacci sequence, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " includes special functions to compute these values. For example, the 15th \
Fibonacci number can be computed using the ",
 StyleBox[ButtonBox["Fibonacci",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Fibonacci"], "FunctionLink"],
 " function."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Fibonacci", "[", "15", "]"}]], "Input"],

Cell[BoxData["610"], "Output"]
}, Open  ]],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["RecurrenceTable",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/RecurrenceTable"], "FunctionLink"],
 " function is very flexible, able to handle a wide variety of recurrence \
relations. For example, consider the recurrence relation defined by ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["a", 
     RowBox[{"n", "+", "1"}]], "=", 
    RowBox[{
     SuperscriptBox[
      SubscriptBox["a", "n"], "2"], "-", 
     RowBox[{"n", "\[CenterDot]", 
      SubscriptBox["a", 
       RowBox[{"n", "-", "1"}]]}]}]}], TraditionalForm]]],
 ", with initial conditions ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["a", "0"], "=", "1"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["a", "1"], "=", "1"}], TraditionalForm]]],
 ". The following expression computes ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["a", "10"], TraditionalForm]]],
 ". Note the use of the ",
 StyleBox[ButtonBox["Last",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Last"], "FunctionLink"],
 " function to obtain the last element of the sequence. This is equivalent \
to, but perhaps more expressive than, the ",
 StyleBox[ButtonBox["Part",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Part"], "FunctionLink"],
 " specification ",
 StyleBox["[[-1]]", "Input"],
 ". "
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Last", "[", 
  RowBox[{"RecurrenceTable", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       RowBox[{"a", "[", 
        RowBox[{"n", "+", "1"}], "]"}], "\[Equal]", 
       RowBox[{
        RowBox[{
         RowBox[{"a", "[", "n", "]"}], "^", "2"}], "-", 
        RowBox[{"n", "*", 
         RowBox[{"a", "[", 
          RowBox[{"n", "-", "1"}], "]"}]}]}]}], ",", 
      RowBox[{
       RowBox[{"a", "[", "0", "]"}], "\[Equal]", "1"}], ",", 
      RowBox[{
       RowBox[{"a", "[", "1", "]"}], "\[Equal]", "1"}]}], "}"}], ",", "a", 
    ",", 
    RowBox[{"{", 
     RowBox[{"n", ",", "10"}], "}"}]}], "]"}], "]"}]], "Input"],

Cell[BoxData["83095558751833088261963048239982609365670356"], "Output"]
}, Open  ]],

Cell[TextData[{
 "While ",
 StyleBox[ButtonBox["RecurrenceTable",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/RecurrenceTable"], "FunctionLink"],
 " is very general, sometimes a less flexible function can be useful. The ",
 StyleBox[ButtonBox["LinearRecurrence",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/LinearRecurrence"], "FunctionLink"],
 " function produces output similar to that of ",
 StyleBox[ButtonBox["RecurrenceTable",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/RecurrenceTable"], "FunctionLink"],
 " but applies only to linear homogeneous recurrence relations (defined in \
Section 8.2 of the textbook). ",
 StyleBox[ButtonBox["LinearRecurrence",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/LinearRecurrence"], "FunctionLink"],
 " applies for recurrence relations of the form"
}], "Text"],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["a", "n"], "=", 
   RowBox[{
    RowBox[{
     SubscriptBox["c", "1"], 
     SubscriptBox["a", 
      RowBox[{"n", "-", "1"}]]}], "+", 
    RowBox[{
     SubscriptBox["c", "2"], 
     SubscriptBox["a", 
      RowBox[{"n", "-", "2"}]]}], "+", "\[CenterEllipsis]", "+", 
    RowBox[{
     SubscriptBox["c", "k"], 
     SubscriptBox["a", 
      RowBox[{"n", "-", "k"}]]}]}]}], TraditionalForm]], "Equation"],

Cell[TextData[{
 "with initial conditions ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["a", "1"], ",", 
    SubscriptBox["a", "2"], ",", "\[Ellipsis]", ",", 
    SubscriptBox["a", "k"]}], TraditionalForm]]],
 ". ",
 StyleBox[ButtonBox["LinearRecurrence",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/LinearRecurrence"], "FunctionLink"],
 " requires three arguments. The first argument is the list of coefficients \
in the recurrence relation, i.e., ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{
     SubscriptBox["c", "1"], ",", 
     SubscriptBox["c", "2"], ",", "\[Ellipsis]", ",", 
     SubscriptBox["c", "k"]}], "}"}], TraditionalForm]]],
 ". The second argument is the list of initial conditions ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{
     SubscriptBox["a", "1"], ",", 
     SubscriptBox["a", "2"], ",", "\[Ellipsis]", ",", 
     SubscriptBox["a", "k"]}], "}"}], TraditionalForm]]],
 ". Note that the two lists should generally be the same length. The final \
argument is either a single integer to indicate the number of terms of the \
sequence to produce or a pair of integers representing a range of indices."
}], "Text"],

Cell[TextData[{
 "By way of comparison, the following two expressions use ",
 StyleBox[ButtonBox["RecurrenceTable",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/RecurrenceTable"], "FunctionLink"],
 " and ",
 StyleBox[ButtonBox["LinearRecurrence",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/LinearRecurrence"], "FunctionLink"],
 " to compute the first twenty terms of the sequence defined by the \
recurrence relation ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["a", "n"], "=", 
    RowBox[{
     RowBox[{"2", 
      SubscriptBox["a", 
       RowBox[{"n", "-", "1"}]]}], "-", 
     RowBox[{"3", 
      SubscriptBox["a", 
       RowBox[{"n", "-", "2"}]]}], "+", 
     SubscriptBox["a", 
      RowBox[{"n", "-", "3"}]]}]}], TraditionalForm]]],
 ", with initial values ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["a", "1"], "=", "1"}], ",", 
    RowBox[{
     SubscriptBox["a", "2"], "=", "3"}], ",", 
    RowBox[{
     SubscriptBox["a", "3"], "=", "5"}]}], TraditionalForm]]],
 ". "
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"RecurrenceTable", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"a", "[", "n", "]"}], "==", 
      RowBox[{
       RowBox[{"2", "*", 
        RowBox[{"a", "[", 
         RowBox[{"n", "-", "1"}], "]"}]}], "-", 
       RowBox[{"3", "*", 
        RowBox[{"a", "[", 
         RowBox[{"n", "-", "2"}], "]"}]}], "+", 
       RowBox[{"a", "[", 
        RowBox[{"n", "-", "3"}], "]"}]}]}], ",", 
     RowBox[{
      RowBox[{"a", "[", "1", "]"}], "\[Equal]", "1"}], ",", 
     RowBox[{
      RowBox[{"a", "[", "2", "]"}], "\[Equal]", "3"}], ",", 
     RowBox[{
      RowBox[{"a", "[", "3", "]"}], "\[Equal]", "5"}]}], "}"}], ",", "a", ",", 
   RowBox[{"{", 
    RowBox[{"n", ",", "20"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "3", ",", "5", ",", "2", ",", 
   RowBox[{"-", "8"}], ",", 
   RowBox[{"-", "17"}], ",", 
   RowBox[{"-", "8"}], ",", "27", ",", "61", ",", "33", ",", 
   RowBox[{"-", "90"}], ",", 
   RowBox[{"-", "218"}], ",", 
   RowBox[{"-", "133"}], ",", "298", ",", "777", ",", "527", ",", 
   RowBox[{"-", "979"}], ",", 
   RowBox[{"-", "2762"}], ",", 
   RowBox[{"-", "2060"}], ",", "3187"}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"LinearRecurrence", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"2", ",", 
     RowBox[{"-", "3"}], ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "3", ",", "5"}], "}"}], ",", "20"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "3", ",", "5", ",", "2", ",", 
   RowBox[{"-", "8"}], ",", 
   RowBox[{"-", "17"}], ",", 
   RowBox[{"-", "8"}], ",", "27", ",", "61", ",", "33", ",", 
   RowBox[{"-", "90"}], ",", 
   RowBox[{"-", "218"}], ",", 
   RowBox[{"-", "133"}], ",", "298", ",", "777", ",", "527", ",", 
   RowBox[{"-", "979"}], ",", 
   RowBox[{"-", "2762"}], ",", 
   RowBox[{"-", "2060"}], ",", "3187"}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Tower of Hanoi Problem", "Subsection",
 CellTags->"Section1a"],

Cell["\<\
In Example 2 of Section 8.1 of the textbook, the author describes the famous \
\[OpenCurlyDoubleQuote]Tower of Hanoi\[CloseCurlyDoubleQuote] puzzle and \
derives the recurrence relation\
\>", "Text"],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["H", "n"], "=", 
    RowBox[{
     RowBox[{"2", 
      SubscriptBox["H", 
       RowBox[{"n", "-", "1"}]]}], "+", "1"}]}], ",", "     ", 
   RowBox[{
    SubscriptBox["H", "1"], "=", "1"}], ","}], TraditionalForm]], "Equation"],

Cell[TextData[{
 "where ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["H", "n"], TraditionalForm]]],
 " represents the number of moves required to solve the puzzle for ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " disks. As discussed in the text, this has the solution"
}], "Text"],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["H", "n"], "=", 
   RowBox[{
    SuperscriptBox["2", "n"], "-", "1"}]}], TraditionalForm]], "Equation"],

Cell[TextData[{
 "Later, we will see how to use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to derive this result."
}], "Text"],

Cell[TextData[{
 "Rather than just computing the values, we can illustrate the solution to \
the Tower of Hanoi puzzle by writing a ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " program to compute the moves needed and to describe them to us. We will \
write a small program consisting of three ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " functions: the main program ",
 StyleBox["hanoi", "Input"],
 ", a utility function ",
 StyleBox["printMove", "Input"],
 ", and ",
 StyleBox["transferDisk", "Input"],
 ", which does most of the work."
}], "Text"],

Cell[TextData[{
 "The easiest part to write is the function ",
 StyleBox["printMove", "Input"],
 ", which merely displays the move to make at a given step."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"printMove", "[", 
   RowBox[{"src_String", ",", "dest_String"}], "]"}], ":=", 
  RowBox[{"Print", "[", 
   RowBox[{
   "\"\<Move disk from peg \>\"", ",", "src", ",", "\"\< to peg \>\"", ",", 
    "dest", ",", "\"\<.\>\""}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "In the above, the function ",
 StyleBox[ButtonBox["Print",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Print"], "FunctionLink"],
 " will combine all of its arguments and display them on a single line."
}], "Text"],

Cell[TextData[{
 "Next we write the recursive function ",
 StyleBox["transferDisk", "Input"],
 ", which does most of the work. This function models the idea of \
transferring a stack of ",
 StyleBox["ndisks", "Input"],
 " disks from the source peg, which is given as the argument ",
 StyleBox["src", "Input"],
 ", to the destination peg, ",
 StyleBox["dest", "Input"],
 ", via the intermediate peg, ",
 StyleBox["via", "Input"],
 ". As is described in the text, in order to move a stack of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " disks, you first move the top ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "-", "1"}], TraditionalForm]]],
 " pegs to the intermediate peg (using the destination as the intermediary), \
then move the bottom disk to the destination, and then move the smaller stack \
from the intermediate peg to the destination. Unless, of course, there is \
only 1 disk, in which you just move that disk to the destination. This is \
coded as follows:"
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"transferDisk", "[", 
   RowBox[{
   "src_String", ",", "via_String", ",", "dest_String", ",", 
    "ndisks_Integer"}], "]"}], ":=", "\[IndentingNewLine]", 
  RowBox[{"If", "[", 
   RowBox[{
    RowBox[{"ndisks", "\[Equal]", "1"}], ",", "\[IndentingNewLine]", 
    RowBox[{"printMove", "[", 
     RowBox[{"src", ",", "dest"}], "]"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"transferDisk", "[", 
      RowBox[{"src", ",", "dest", ",", "via", ",", 
       RowBox[{"ndisks", "-", "1"}]}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"printMove", "[", 
      RowBox[{"src", ",", "dest"}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"transferDisk", "[", 
      RowBox[{"via", ",", "src", ",", "dest", ",", 
       RowBox[{"ndisks", "-", "1"}]}], "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Finally, we package the recursive procedure in a top level function, ",
 StyleBox["hanoi", "Input"],
 ", providing an interface to the recursive engine."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"hanoi", "[", "ndisks_Integer", "]"}], "/;", 
   RowBox[{"ndisks", ">", "1"}]}], ":=", 
  RowBox[{"transferDisk", "[", 
   RowBox[{"\"\<A\>\"", ",", "\"\<B\>\"", ",", "\"\<C\>\"", ",", "ndisks"}], 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Our ",
 StyleBox["hanoi", "Input"],
 " program can exhibit a specific solution to the Tower of Hanoi puzzle for \
any number of disks:"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"hanoi", "[", "2", "]"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Move disk from peg \"\>", "\[InvisibleSpace]", "\<\"A\"\>", 
   "\[InvisibleSpace]", "\<\" to peg \"\>", "\[InvisibleSpace]", "\<\"B\"\>", 
   "\[InvisibleSpace]", "\<\".\"\>"}],
  SequenceForm["Move disk from peg ", "A", " to peg ", "B", "."],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Move disk from peg \"\>", "\[InvisibleSpace]", "\<\"A\"\>", 
   "\[InvisibleSpace]", "\<\" to peg \"\>", "\[InvisibleSpace]", "\<\"C\"\>", 
   "\[InvisibleSpace]", "\<\".\"\>"}],
  SequenceForm["Move disk from peg ", "A", " to peg ", "C", "."],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Move disk from peg \"\>", "\[InvisibleSpace]", "\<\"B\"\>", 
   "\[InvisibleSpace]", "\<\" to peg \"\>", "\[InvisibleSpace]", "\<\"C\"\>", 
   "\[InvisibleSpace]", "\<\".\"\>"}],
  SequenceForm["Move disk from peg ", "B", " to peg ", "C", "."],
  Editable->False]], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"hanoi", "[", "3", "]"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Move disk from peg \"\>", "\[InvisibleSpace]", "\<\"A\"\>", 
   "\[InvisibleSpace]", "\<\" to peg \"\>", "\[InvisibleSpace]", "\<\"C\"\>", 
   "\[InvisibleSpace]", "\<\".\"\>"}],
  SequenceForm["Move disk from peg ", "A", " to peg ", "C", "."],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Move disk from peg \"\>", "\[InvisibleSpace]", "\<\"A\"\>", 
   "\[InvisibleSpace]", "\<\" to peg \"\>", "\[InvisibleSpace]", "\<\"B\"\>", 
   "\[InvisibleSpace]", "\<\".\"\>"}],
  SequenceForm["Move disk from peg ", "A", " to peg ", "B", "."],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Move disk from peg \"\>", "\[InvisibleSpace]", "\<\"C\"\>", 
   "\[InvisibleSpace]", "\<\" to peg \"\>", "\[InvisibleSpace]", "\<\"B\"\>", 
   "\[InvisibleSpace]", "\<\".\"\>"}],
  SequenceForm["Move disk from peg ", "C", " to peg ", "B", "."],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Move disk from peg \"\>", "\[InvisibleSpace]", "\<\"A\"\>", 
   "\[InvisibleSpace]", "\<\" to peg \"\>", "\[InvisibleSpace]", "\<\"C\"\>", 
   "\[InvisibleSpace]", "\<\".\"\>"}],
  SequenceForm["Move disk from peg ", "A", " to peg ", "C", "."],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Move disk from peg \"\>", "\[InvisibleSpace]", "\<\"B\"\>", 
   "\[InvisibleSpace]", "\<\" to peg \"\>", "\[InvisibleSpace]", "\<\"A\"\>", 
   "\[InvisibleSpace]", "\<\".\"\>"}],
  SequenceForm["Move disk from peg ", "B", " to peg ", "A", "."],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Move disk from peg \"\>", "\[InvisibleSpace]", "\<\"B\"\>", 
   "\[InvisibleSpace]", "\<\" to peg \"\>", "\[InvisibleSpace]", "\<\"C\"\>", 
   "\[InvisibleSpace]", "\<\".\"\>"}],
  SequenceForm["Move disk from peg ", "B", " to peg ", "C", "."],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Move disk from peg \"\>", "\[InvisibleSpace]", "\<\"A\"\>", 
   "\[InvisibleSpace]", "\<\" to peg \"\>", "\[InvisibleSpace]", "\<\"C\"\>", 
   "\[InvisibleSpace]", "\<\".\"\>"}],
  SequenceForm["Move disk from peg ", "A", " to peg ", "C", "."],
  Editable->False]], "Print"]
}, Open  ]]
}, Open  ]],

Cell[TextData[{
 "Try experimenting with different values of ",
 StyleBox["ndisk", "Input"],
 " to get a feel for how large the problem becomes for even moderately large \
numbers of disks."
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Dynamic Programming", "Subsection",
 CellTags->"Section1b"],

Cell["\<\
We conclude this section with an implementation of Algorithm 1 from Section \
8.1 of the text. Recall that the goal of this algorithm is to find the \
maximum number of attendees that can be achieved by a schedule of talks. \
\>", "Text"],

Cell["\<\
We will represent each talk as a list of three elements, with the start time \
being the first element, the end time in the second position, and the weight, \
or attendance, will be last. Time of day will be represented by a single \
number with whole part equal to the hour in the 24-hour system and with \
fractional part equal to the part of an hour that corresponds to the number \
of minutes. For instance, 2:30 P.M. would be represented as 14.5. \
\>", "Text"],

Cell["As an example, consider the following eight talks. ", "Text"],

Cell[TextData[Cell[BoxData[GridBox[{
   {
    RowBox[{"Start", " ", "Time"}], 
    RowBox[{"End", " ", "Time"}], "Attendance"},
   {
    RowBox[{"9", ":", 
     RowBox[{"00", " ", "AM"}]}], 
    RowBox[{"11", ":", 
     RowBox[{"00", " ", "AM"}]}], "17"},
   {
    RowBox[{"9", ":", 
     RowBox[{"00", " ", "AM"}]}], 
    RowBox[{"10", ":", 
     RowBox[{"30", " ", "AM"}]}], "15"},
   {
    RowBox[{"10", ":", 
     RowBox[{"00", " ", "AM"}]}], 
    RowBox[{"11", ":", 
     RowBox[{"30", " ", "AM"}]}], "22"},
   {
    RowBox[{"10", ":", 
     RowBox[{"30", " ", "AM"}]}], 
    RowBox[{"12", ":", 
     RowBox[{"00", " ", "PM"}]}], "11"},
   {
    RowBox[{"11", ":", 
     RowBox[{"30", " ", "AM"}]}], 
    RowBox[{"1", ":", 
     RowBox[{"30", " ", "PM"}]}], "18"},
   {
    RowBox[{"12", ":", 
     RowBox[{"00", " ", "PM"}]}], 
    RowBox[{"1", ":", 
     RowBox[{"00", " ", "PM"}]}], "12"},
   {
    RowBox[{"1", ":", 
     RowBox[{"30", " ", "PM"}]}], 
    RowBox[{"3", ":", 
     RowBox[{"00", " ", "PM"}]}], "21"},
   {
    RowBox[{"2", ":", 
     RowBox[{"00", " ", "PM"}]}], 
    RowBox[{"4", ":", 
     RowBox[{"00", " ", "PM"}]}], "17"}
  },
  BaseStyle->{FontFamily -> "Times"},
  GridBoxDividers->{
   "Columns" -> {{True}}, "ColumnsIndexed" -> {}, "Rows" -> {{True}}, 
    "RowsIndexed" -> {}}]]]], "Text"],

Cell["\<\
We create the following list of lists to represent the talks. \
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"talks", "=", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"9", ",", "11", ",", "17"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"9", ",", "10.5", ",", "15"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"10", ",", "11.5", ",", "22"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"10.5", ",", "12", ",", "11"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"11.5", ",", "13.5", ",", "18"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"12", ",", "13", ",", "12"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"13.5", ",", "15", ",", "21"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"14", ",", "16", ",", "17"}], "}"}]}], "}"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"9", ",", "11", ",", "17"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"9", ",", "10.5`", ",", "15"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"10", ",", "11.5`", ",", "22"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"10.5`", ",", "12", ",", "11"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"11.5`", ",", "13.5`", ",", "18"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"12", ",", "13", ",", "12"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"13.5`", ",", "15", ",", "21"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"14", ",", "16", ",", "17"}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell["\<\
Recall the description of Algorithm 1 from the text. We summarize the general \
outline of the algorithm below.\
\>", "Text"],

Cell[CellGroupData[{

Cell["Sort the talks in order of increasing end time. ", "Item1Numbered"],

Cell[TextData[{
 "For each index ",
 Cell[BoxData[
  FormBox["j", TraditionalForm]]],
 ", compute ",
 Cell[BoxData[
  FormBox[
   RowBox[{"p", "(", "j", ")"}], TraditionalForm]]],
 " \[LongDash] the maximum index ",
 Cell[BoxData[
  FormBox["i", TraditionalForm]]],
 " such that talk ",
 Cell[BoxData[
  FormBox["i", TraditionalForm]]],
 " is compatible with talk ",
 Cell[BoxData[
  FormBox["j", TraditionalForm]]],
 "."
}], "Item1Numbered"],

Cell[TextData[{
 "For each index ",
 Cell[BoxData[
  FormBox["j", TraditionalForm]]],
 ", compute ",
 Cell[BoxData[
  FormBox[
   RowBox[{"T", "(", "j", ")"}], TraditionalForm]]],
 ", which is computed by the recurrence relation ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"T", "(", "j", ")"}], "=", 
    RowBox[{"max", "(", 
     RowBox[{
      RowBox[{
       SubscriptBox["w", "j"], "+", 
       RowBox[{"T", "(", 
        RowBox[{"p", "(", "j", ")"}], ")"}]}], ",", 
      RowBox[{"T", "(", 
       RowBox[{"j", "-", "1"}], ")"}]}], ")"}]}], TraditionalForm]]],
 " ",
 "and with initial condition ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"T", "(", "0", ")"}], "=", "0"}], TraditionalForm]]],
 "."
}], "Item1Numbered"],

Cell[TextData[{
 "The maximum total number of attendees is ",
 Cell[BoxData[
  FormBox[
   RowBox[{"T", "(", "n", ")"}], TraditionalForm]]],
 ", where ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " is the number of talks."
}], "Item1Numbered"]
}, Open  ]],

Cell[TextData[{
 "For step 1, we will make use of ",
 StyleBox[ButtonBox["Sort",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Sort"], "FunctionLink"],
 " with a custom ordering function. ",
 StyleBox[ButtonBox["Sort",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Sort"], "FunctionLink"],
 " can accept an optional argument in the form of a pure function of two \
arguments. This function should return true if the first argument precedes \
the second and false otherwise. Since we must sort the talks in increasing \
order of end time (which is stored in position 2 in the lists representing \
the talks), we will use the following function as the second argument to ",
 StyleBox[ButtonBox["Sort",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Sort"], "FunctionLink"],
 ". Recall that ",
 StyleBox["#1", "Input"],
 " and ",
 StyleBox["#2", "Input"],
 " indicate the inputs to the function and ",
 StyleBox["&", "Input"],
 " is used to terminate a pure ",
 StyleBox[ButtonBox["Function",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Function"], "FunctionLink"],
 "."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"#1", "[", 
    RowBox[{"[", "2", "]"}], "]"}], "<", 
   RowBox[{"#2", "[", 
    RowBox[{"[", "2", "]"}], "]"}]}], "&"}]], "myCode"],

Cell[TextData[{
 "For step 2, we must compute ",
 Cell[BoxData[
  FormBox[
   RowBox[{"p", "(", "j", ")"}], TraditionalForm]]],
 ". For this, we will create a function that accepts the sorted list of talks \
and returns an indexed variable that represents the function ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ". Recall that the value of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"p", "(", "j", ")"}], TraditionalForm]]],
 " is the largest index among talks compatible with the talk with index ",
 Cell[BoxData[
  FormBox["j", TraditionalForm]]],
 ", so we call this function ",
 StyleBox["compatible", "Input"],
 "."
}], "Text"],

Cell[TextData[{
 "After declaring local variables, we will loop through all the indices, ",
 Cell[BoxData[
  FormBox["j", TraditionalForm]]],
 ", from 1 to the number of talks in the list. We use a local variable, ",
 StyleBox["jstart", "Input"],
 ", to store the start time of the current talk being analyzed. We then \
consider all the talks earlier in the list beginning with the talk with index \
",
 Cell[BoxData[
  FormBox[
   RowBox[{"j", "-", "1"}], TraditionalForm]]],
 " and working backwards to talk 1. For each talk, we check to see if it ends \
before talk ",
 Cell[BoxData[
  FormBox["j", TraditionalForm]]],
 " starts. When we find such a talk, we set its index to the value of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"p", "(", "j", ")"}], TraditionalForm]]],
 " (since we're working backwards, the first one found is the talk with the \
largest index). If no compatible talk is found, then ",
 Cell[BoxData[
  FormBox[
   RowBox[{"p", "(", "j", ")"}], TraditionalForm]]],
 " is set to 0. Here is the function."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"compatible", "[", "talkList_", "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"p", ",", "j", ",", "jstart", ",", "i"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"For", "[", 
      RowBox[{
       RowBox[{"j", "=", "1"}], ",", 
       RowBox[{"j", "\[LessEqual]", 
        RowBox[{"Length", "[", "talkList", "]"}]}], ",", 
       RowBox[{"j", "++"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"jstart", "=", 
         RowBox[{
          RowBox[{"talkList", "[", 
           RowBox[{"[", "j", "]"}], "]"}], "[", 
          RowBox[{"[", "1", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"p", "[", "j", "]"}], "=", 
         RowBox[{"Catch", "[", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"For", "[", 
            RowBox[{
             RowBox[{"i", "=", 
              RowBox[{"j", "-", "1"}]}], ",", 
             RowBox[{"i", "\[GreaterEqual]", "1"}], ",", 
             RowBox[{"i", "--"}], ",", "\[IndentingNewLine]", 
             RowBox[{"If", "[", 
              RowBox[{
               RowBox[{
                RowBox[{
                 RowBox[{"talkList", "[", 
                  RowBox[{"[", "i", "]"}], "]"}], "[", 
                 RowBox[{"[", "2", "]"}], "]"}], "\[LessEqual]", "jstart"}], 
               ",", "\[IndentingNewLine]", 
               RowBox[{"Throw", "[", "i", "]"}]}], "\[IndentingNewLine]", 
              "]"}]}], "\[IndentingNewLine]", "]"}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"Throw", "[", "0", "]"}]}], "\[IndentingNewLine]", 
          "]"}]}]}]}], "\[IndentingNewLine]", "]"}], ";", 
     "\[IndentingNewLine]", "p"}]}], "\[IndentingNewLine]", "]"}]}]], "Input",\

 InitializationCell->True],

Cell[TextData[{
 "For step 3, we must compute ",
 Cell[BoxData[
  FormBox[
   RowBox[{"T", "(", "j", ")"}], TraditionalForm]]],
 ". To do this, we create a function that accepts as input the sorted list of \
talks and the indexed variable representing the function ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ". Initialize T by setting its value at 0 to 0. Then consider each integer \
",
 Cell[BoxData[
  FormBox["j", TraditionalForm]]],
 " from 1 to the number of talks and apply the formula: ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"T", "(", "j", ")"}], "=", 
    RowBox[{"max", "(", 
     RowBox[{
      RowBox[{
       SubscriptBox["w", "j"], "+", 
       RowBox[{"T", "(", 
        RowBox[{"p", "(", "j", ")"}], ")"}]}], ",", 
      RowBox[{"T", "(", 
       RowBox[{"j", "-", "1"}], ")"}]}], ")"}]}], TraditionalForm]]],
 "."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"totalAttendance", "[", 
   RowBox[{"talkList_", ",", "p_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"j", ",", "T"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{
      RowBox[{"T", "[", "0", "]"}], "=", "0"}], ";", "\[IndentingNewLine]", 
     RowBox[{"For", "[", 
      RowBox[{
       RowBox[{"j", "=", "1"}], ",", 
       RowBox[{"j", "\[LessEqual]", 
        RowBox[{"Length", "[", "talkList", "]"}]}], ",", 
       RowBox[{"j", "++"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"T", "[", "j", "]"}], "=", 
        RowBox[{"Max", "[", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"talkList", "[", 
             RowBox[{"[", "j", "]"}], "]"}], "[", 
            RowBox[{"[", "3", "]"}], "]"}], "+", 
           RowBox[{"T", "[", 
            RowBox[{"p", "[", "j", "]"}], "]"}]}], ",", 
          RowBox[{"T", "[", 
           RowBox[{"j", "-", "1"}], "]"}]}], "]"}]}]}], "\[IndentingNewLine]",
       "]"}], ";", "\[IndentingNewLine]", "T"}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell["\<\
We can now put the pieces together as outlined at the start of this \
subsection.\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"maximumAttendance", "[", "talkList_List", "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"L", ",", "p", ",", "T"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"L", "=", 
      RowBox[{"Sort", "[", 
       RowBox[{"talkList", ",", 
        RowBox[{
         RowBox[{
          RowBox[{"#1", "[", 
           RowBox[{"[", "2", "]"}], "]"}], "<", 
          RowBox[{"#2", "[", 
           RowBox[{"[", "2", "]"}], "]"}]}], "&"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"p", "=", 
      RowBox[{"compatible", "[", "L", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"T", "=", 
      RowBox[{"totalAttendance", "[", 
       RowBox[{"L", ",", "p"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"T", "[", 
      RowBox[{"Length", "[", "talkList", "]"}], "]"}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True],

Cell["\<\
And thus, the maximum attendance for the talks described above is: \
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"maximumAttendance", "[", "talks", "]"}]], "Input"],

Cell[BoxData["61"], "Output"]
}, Open  ]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["8.2 Solving Linear Recurrence Relations", "Section",
 CellTags->"Section2"],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " has a very powerful recurrence solver, ",
 StyleBox[ButtonBox["RSolve",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/RSolve"], "FunctionLink"],
 ". Its use, however, can obscure some of the important ideas that are \
involved. Therefore, we will first use some of ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "'s more primitive facilities to solve certain kinds of recurrence relations \
one step at a time."
}], "Text"],

Cell[TextData[{
 "Given a recursively defined sequence ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    SubscriptBox["a", "n"], "}"}], TraditionalForm]]],
 ", we would like to find a formula, involving only the index ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " (and, perhaps, other fixed constants and known functions) which does not \
depend on knowing the value of any prior elements of the sequence."
}], "Text"],

Cell[CellGroupData[{

Cell["\<\
Linear Homogeneous Recurrence Relations with Constant Coefficients\
\>", "Subsection",
 CellTags->"Section2a"],

Cell[TextData[{
 "We will begin by considering recurrence relations that are ",
 StyleBox["linear",
  FontSlant->"Italic"],
 ", ",
 StyleBox["homogeneous",
  FontSlant->"Italic"],
 ", and which have ",
 StyleBox["constant coefficients",
  FontSlant->"Italic"],
 "; that is, they have the form"
}], "Text"],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["a", "n"], "=", 
   RowBox[{
    RowBox[{
     SubscriptBox["c", "1"], 
     SubscriptBox["a", 
      RowBox[{"n", "-", "1"}]]}], "+", 
    RowBox[{
     SubscriptBox["c", "2"], 
     SubscriptBox["a", 
      RowBox[{"n", "-", "2"}]]}], "+", "\[CenterEllipsis]", "+", 
    RowBox[{
     SubscriptBox["c", "k"], 
     SubscriptBox["a", 
      RowBox[{"n", "-", "k"}]]}]}]}], TraditionalForm]], "Equation"],

Cell[TextData[{
 "where ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["c", "1"], ",", 
    SubscriptBox["c", "2"], ",", "\[Ellipsis]", ",", 
    SubscriptBox["c", "k"]}], TraditionalForm]]],
 " are real constants and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["c", "k"], TraditionalForm]]],
 " is nonzero. Recall that the integer ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " is called the degree of this recurrence relation. To have a unique \
solution, at least ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " initial conditions must be specified."
}], "Text"],

Cell["\<\
The general method for solving such a recurrence relation involves finding \
the roots of its characteristic polynomial \
\>", "Text"],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["r", "k"], "-", 
   RowBox[{
    SubscriptBox["c", "1"], 
    SuperscriptBox["r", 
     RowBox[{"k", "-", "1"}]]}], "-", 
   RowBox[{
    SubscriptBox["c", "2"], 
    SuperscriptBox["r", 
     RowBox[{"k", "-", "2"}]]}], "-", "\[CenterEllipsis]", "-", 
   RowBox[{
    SubscriptBox["c", 
     RowBox[{"k", "-", "1"}]], "r"}], "-", 
   SubscriptBox["c", "k"]}], TraditionalForm]], "Equation"],

Cell[TextData[{
 "When this polynomial has distinct roots, all solutions are linear \
combinations of the ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 "th powers of these roots. When there are repeated roots, the situation is a \
little more complicated, as we will see."
}], "Text"],

Cell[CellGroupData[{

Cell["A First Example", "Subsubsection"],

Cell["\<\
Consider the linear homogeneous recurrence relation with constant \
coefficients of degree two \
\>", "Text"],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["a", "n"], "=", 
   RowBox[{
    RowBox[{"2", 
     SubscriptBox["a", 
      RowBox[{"n", "-", "1"}]]}], "+", 
    RowBox[{"3", 
     SubscriptBox["a", 
      RowBox[{"n", "-", "2"}]]}]}]}], TraditionalForm]], "Equation"],

Cell[TextData[{
 "subject to the initial conditions ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["a", "1"], "=", "4"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["a", "2"], "=", "2"}], TraditionalForm]]],
 "."
}], "Text"],

Cell["Then its characteristic equation is", "Text"],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SuperscriptBox["r", "2"], "-", 
    RowBox[{"2", "r"}], "-", "3"}], "=", "0"}], TraditionalForm]], "Equation"],

Cell[TextData[{
 "To solve the recurrence relation, we must solve for the roots of this \
equation. Using ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " makes this very easy; we use the ",
 StyleBox[ButtonBox["Solve",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Solve"], "FunctionLink"],
 " function."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Solve", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"r", "^", "2"}], "-", 
     RowBox[{"2", "r"}], "-", "3"}], "\[Equal]", "0"}], ",", "r"}], 
  "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"r", "\[Rule]", 
     RowBox[{"-", "1"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", "\[Rule]", "3"}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["Solve",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Solve"], "FunctionLink"],
 " function computes the values of the variable ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 ", given as the second argument, that satisfy the equation in the first \
argument. Note that the output is a list of lists, within which are rules \
specifying the solutions."
}], "Text"],

Cell[TextData[{
 "Now that ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " has determined that the solutions are ",
 Cell[BoxData[
  FormBox[
   RowBox[{"r", "=", 
    RowBox[{"-", "1"}]}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"r", "=", "3"}], TraditionalForm]]],
 ", we can write down the form of the solution to the recurrence as"
}], "Text"],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["a", "n"], "=", 
   RowBox[{
    RowBox[{"\[Alpha]", "\[CenterDot]", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "n"]}], "+", 
    RowBox[{"\[Beta]", "\[CenterDot]", 
     SuperscriptBox["3", "n"]}]}]}], TraditionalForm]], "Equation"],

Cell["\<\
where \[Alpha] and \[Beta] are constants that we have yet to determine.\
\>", "Text"],

Cell[TextData[{
 "Since the initial conditions are ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["a", "1"], "=", "4"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["a", "2"], "=", "2"}], TraditionalForm]]],
 ", we know that our recurrence relation must satisfy the following pair of \
equations."
}], "Text"],

Cell[BoxData[
 FormBox[
  RowBox[{"\[Piecewise]", GridBox[{
     {
      RowBox[{
       RowBox[{
        RowBox[{"-", "\[Alpha]"}], "+", 
        RowBox[{"3", "\[Beta]"}]}], "=", "4"}]},
     {
      RowBox[{
       RowBox[{"\[Alpha]", "+", 
        RowBox[{"9", "\[Beta]"}]}], "=", "2"}]}
    }]}], TraditionalForm]], "Equation"],

Cell[TextData[{
 "To find the solution to this system of linear equations, we again use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "'s ",
 StyleBox[ButtonBox["Solve",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Solve"], "FunctionLink"],
 " facility:"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Solve", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"-", "alpha"}], "+", 
       RowBox[{"3", "*", "beta"}]}], "\[Equal]", "4"}], ",", 
     RowBox[{
      RowBox[{"alpha", "+", 
       RowBox[{"9", " ", "*", " ", "beta"}]}], "\[Equal]", "2"}]}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"alpha", ",", "beta"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"alpha", "\[Rule]", 
     RowBox[{"-", 
      FractionBox["5", "2"]}]}], ",", 
    RowBox[{"beta", "\[Rule]", 
     FractionBox["1", "2"]}]}], "}"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The braces tell ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to solve the list of equations as a simultaneous system of equations. \
Likewise, the variables to be solved for form a list."
}], "Text"],

Cell["\<\
Now that we have the values for \[Alpha] and \[Beta], we see that the \
complete solution to the recurrence relation is\
\>", "Text"],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["a", "n"], "=", 
   RowBox[{
    RowBox[{
     FractionBox[
      RowBox[{"-", "5"}], "2"], 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "n"]}], "+", 
    RowBox[{
     FractionBox["1", "2"], 
     SuperscriptBox["3", "n"]}]}]}], TraditionalForm]], "Equation"],

Cell[TextData[{
 "This formula allows us to write a ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " function for finding the terms of the sequence ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    SubscriptBox["a", "n"], "}"}], TraditionalForm]]],
 ", which can be more efficient than a recursive approach."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"aFormula", "[", "n_Integer", "]"}], ":=", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", "5"}], "/", "2"}], ")"}], "*", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}], "^", "n"}]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"1", "/", "2"}], ")"}], "*", 
    RowBox[{"3", "^", "n"}]}]}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Table", "[", 
  RowBox[{
   RowBox[{"aFormula", "[", "n", "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"n", ",", "1", ",", "10"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "4", ",", "2", ",", "16", ",", "38", ",", "124", ",", "362", ",", "1096", 
   ",", "3278", ",", "9844", ",", "29522"}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["A Second Example", "Subsubsection"],

Cell["\<\
Let's try another example. We will solve the recurrence relation\
\>", "Text"],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["a", "n"], "=", 
   RowBox[{
    RowBox[{
     FractionBox[
      RowBox[{"-", "5"}], "3"], 
     SubscriptBox["a", 
      RowBox[{"n", "-", "1"}]]}], "+", 
    RowBox[{
     FractionBox["2", "3"], 
     SubscriptBox["a", 
      RowBox[{"n", "-", "2"}]]}]}]}], TraditionalForm]], "Equation"],

Cell[TextData[{
 "with initial conditions ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["a", "1"], "=", 
    FractionBox["1", "2"]}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["a", "2"], "=", "4"}], TraditionalForm]]],
 "."
}], "Text"],

Cell[TextData[{
 "To do this, we ask ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to solve the characteristic equation of the recurrence relation, and then \
solve the system of linear equations obtained from the roots of the \
characteristic equation and the initial conditions. Note that this method \
works because this recurrence relation is linear, homogeneous, and has \
constant coefficients."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"charEqnRoots", "=", 
  RowBox[{"Solve", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"r", "^", "2"}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"5", "/", "3"}], ")"}], "*", "r"}], "-", 
      RowBox[{"(", 
       RowBox[{"2", "/", "3"}], ")"}]}], "\[Equal]", "0"}], ",", "r"}], 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"r", "\[Rule]", 
     RowBox[{"-", "2"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", "\[Rule]", 
     FractionBox["1", "3"]}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Note that we can transform the output from a list of lists of rules into a \
simple list of the solutions using the ",
 StyleBox[ButtonBox["ReplaceAll",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ReplaceAll"], "FunctionLink"],
 " (",
 StyleBox["/.", "Input"],
 ") operator as shown below."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"charRootsL", "=", 
  RowBox[{"r", "/.", "charEqnRoots"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"-", "2"}], ",", 
   FractionBox["1", "3"]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "When ",
 StyleBox[ButtonBox["ReplaceAll",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ReplaceAll"], "FunctionLink"],
 " is given a list of lists of rules as its right operand, its output is the \
list obtained by applying the rules in each sublist in turn. "
}], "Text"],

Cell["\<\
This allows us to solve for \[Alpha] and \[Beta] by referencing the solutions \
to the characteristic equation, rather than typing them. \
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Solve", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"alpha", "*", 
        RowBox[{"charRootsL", "[", 
         RowBox[{"[", "1", "]"}], "]"}]}], "+", 
       RowBox[{"beta", "*", 
        RowBox[{"charRootsL", "[", 
         RowBox[{"[", "2", "]"}], "]"}]}]}], "\[Equal]", 
      RowBox[{"1", "/", "2"}]}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{
       RowBox[{"alpha", "*", 
        RowBox[{
         RowBox[{"charRootsL", "[", 
          RowBox[{"[", "1", "]"}], "]"}], "^", "2"}]}], "+", 
       RowBox[{"beta", "*", 
        RowBox[{
         RowBox[{"charRootsL", "[", 
          RowBox[{"[", "2", "]"}], "]"}], "^", "2"}]}]}], "\[Equal]", "4"}]}],
     "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"alpha", ",", "beta"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"alpha", "\[Rule]", 
     FractionBox["23", "28"]}], ",", 
    RowBox[{"beta", "\[Rule]", 
     FractionBox["45", "7"]}]}], "}"}], "}"}]], "Output"]
}, Open  ]],

Cell["Thus we see that the solution to the recurrence relation is ", "Text"],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["a", "n"], "=", 
   RowBox[{
    RowBox[{
     FractionBox["23", "28"], 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "2"}], ")"}], "n"]}], "+", 
    RowBox[{
     FractionBox["45", "7"], 
     SuperscriptBox[
      RowBox[{"(", 
       FractionBox["1", "3"], ")"}], "n"]}]}]}], TraditionalForm]], "Equation"]
}, Open  ]],

Cell[CellGroupData[{

Cell["The Fibonacci Sequence", "Subsubsection"],

Cell["\<\
We can derive an explicit formula for the Fibonacci sequence this way as \
well. The characteristic polynomial for the Fibonacci sequence is \
\>", "Text"],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["r", "2"], "-", "r", "-", "1"}], 
  TraditionalForm]], "Equation"],

Cell["We find the roots of the characteristic equation.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"cEqnRoots", "=", 
  RowBox[{"r", "/.", 
   RowBox[{"Solve", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"r", "^", "2"}], "-", "r", "-", "1"}], "\[Equal]", "0"}], ",", 
     "r"}], "]"}]}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{"(", 
     RowBox[{"1", "-", 
      SqrtBox["5"]}], ")"}]}], ",", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{"(", 
     RowBox[{"1", "+", 
      SqrtBox["5"]}], ")"}]}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "So the formula for the ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 "th Fibonacci number is of the form"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Fn", "=", 
  RowBox[{
   RowBox[{"alpha", "*", 
    RowBox[{
     RowBox[{"cEqnRoots", "[", 
      RowBox[{"[", "1", "]"}], "]"}], "^", "n"}]}], "+", 
   RowBox[{"beta", "*", 
    RowBox[{
     RowBox[{"cEqnRoots", "[", 
      RowBox[{"[", "2", "]"}], "]"}], "^", "n"}]}]}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{
      FractionBox["1", "2"], " ", 
      RowBox[{"(", 
       RowBox[{"1", "-", 
        SqrtBox["5"]}], ")"}]}], ")"}], "n"], " ", "alpha"}], "+", 
  RowBox[{
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{
      FractionBox["1", "2"], " ", 
      RowBox[{"(", 
       RowBox[{"1", "+", 
        SqrtBox["5"]}], ")"}]}], ")"}], "n"], " ", "beta"}]}]], "Output"]
}, Open  ]],

Cell["\<\
We find the coefficients \[Alpha] and \[Beta] in the formula by using the \
initial conditions.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"alphas", "=", 
  RowBox[{"Solve", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"alpha", "*", 
         RowBox[{"cEqnRoots", "[", 
          RowBox[{"[", "1", "]"}], "]"}]}], "+", 
        RowBox[{"beta", "*", 
         RowBox[{"cEqnRoots", "[", 
          RowBox[{"[", "2", "]"}], "]"}]}]}], "\[Equal]", "1"}], ",", 
      RowBox[{
       RowBox[{
        RowBox[{"alpha", "*", 
         RowBox[{
          RowBox[{"cEqnRoots", "[", 
           RowBox[{"[", "1", "]"}], "]"}], "^", "2"}]}], "+", 
        RowBox[{"beta", "*", 
         RowBox[{
          RowBox[{"cEqnRoots", "[", 
           RowBox[{"[", "2", "]"}], "]"}], "^", "2"}]}]}], "\[Equal]", 
       "1"}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"alpha", ",", "beta"}], "}"}]}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"alpha", "\[Rule]", 
     RowBox[{"-", 
      FractionBox[
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", "1"}], "+", 
          SqrtBox["5"]}], ")"}], " ", 
        RowBox[{"(", 
         RowBox[{"1", "+", 
          SqrtBox["5"]}], ")"}]}], 
       RowBox[{"4", " ", 
        SqrtBox["5"]}]]}]}], ",", 
    RowBox[{"beta", "\[Rule]", 
     RowBox[{"-", 
      FractionBox[
       RowBox[{
        RowBox[{"-", "5"}], "-", 
        SqrtBox["5"]}], 
       RowBox[{"5", " ", 
        RowBox[{"(", 
         RowBox[{"1", "+", 
          SqrtBox["5"]}], ")"}]}]]}]}]}], "}"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "We can use ",
 StyleBox[ButtonBox["ReplaceAll",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ReplaceAll"], "FunctionLink"],
 " (",
 StyleBox["/.", "Input"],
 ") to substitute the values for \[Alpha] and \[Beta] into the formula ",
 StyleBox["Fn", "Input"],
 ". Note that we access the \[OpenCurlyDoubleQuote]first\
\[CloseCurlyDoubleQuote] element of alphas in order to not have a nested list."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Fn", "/.", 
  RowBox[{"alphas", "[", 
   RowBox[{"[", "1", "]"}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", 
   FractionBox[
    RowBox[{
     SuperscriptBox["2", 
      RowBox[{
       RowBox[{"-", "2"}], "-", "n"}]], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"1", "-", 
        SqrtBox["5"]}], ")"}], "n"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", "1"}], "+", 
       SqrtBox["5"]}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"1", "+", 
       SqrtBox["5"]}], ")"}]}], 
    SqrtBox["5"]]}], "-", 
  RowBox[{
   FractionBox["1", "5"], " ", 
   SuperscriptBox["2", 
    RowBox[{"-", "n"}]], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", "5"}], "-", 
     SqrtBox["5"]}], ")"}], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"1", "+", 
      SqrtBox["5"]}], ")"}], 
    RowBox[{
     RowBox[{"-", "1"}], "+", "n"}]]}]}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"Fn", "/.", 
   RowBox[{"alphas", "[", 
    RowBox[{"[", "1", "]"}], "]"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"-", 
  FractionBox[
   RowBox[{
    SuperscriptBox["2", 
     RowBox[{"-", "n"}]], " ", 
    RowBox[{"(", 
     RowBox[{"5", "+", 
      SqrtBox["5"]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"1", "-", 
         SqrtBox["5"]}], ")"}], "n"], "-", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"1", "+", 
         SqrtBox["5"]}], ")"}], "n"]}], ")"}]}], 
   RowBox[{"5", " ", 
    RowBox[{"(", 
     RowBox[{"1", "+", 
      SqrtBox["5"]}], ")"}]}]]}]], "Output"]
}, Open  ]],

Cell["\<\
If we are to use such a formula to repeatedly compute values, then we should \
define a function. You can do this by retyping the formula or copy and paste \
it into a function definition. Or you can use the expression as the function \
definition as shown below.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"fibonacci2", "[", "n_", "]"}], "=", 
  RowBox[{"Fn", "/.", 
   RowBox[{"alphas", "[", 
    RowBox[{"[", "1", "]"}], "]"}]}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", 
   FractionBox[
    RowBox[{
     SuperscriptBox["2", 
      RowBox[{
       RowBox[{"-", "2"}], "-", "n"}]], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"1", "-", 
        SqrtBox["5"]}], ")"}], "n"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", "1"}], "+", 
       SqrtBox["5"]}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"1", "+", 
       SqrtBox["5"]}], ")"}]}], 
    SqrtBox["5"]]}], "-", 
  RowBox[{
   FractionBox["1", "5"], " ", 
   SuperscriptBox["2", 
    RowBox[{"-", "n"}]], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", "5"}], "-", 
     SqrtBox["5"]}], ")"}], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"1", "+", 
      SqrtBox["5"]}], ")"}], 
    RowBox[{
     RowBox[{"-", "1"}], "+", "n"}]]}]}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Note that in this situation, it is very important to use ",
 StyleBox[ButtonBox["Set",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Set"], "FunctionLink"],
 " (",
 StyleBox["=", "Input"],
 "), not the usual ",
 StyleBox[ButtonBox["SetDelayed",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/SetDelayed"], "FunctionLink"],
 " (",
 StyleBox[":=", "Input"],
 "). This is so that the right hand side resolves into the expression before \
the function is defined."
}], "Text"],

Cell[TextData[{
 "Observe that ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " will not necessarily simplify it automatically."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"fibonacci2", "[", "1", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", 
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{"1", "-", 
       SqrtBox["5"]}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", "1"}], "+", 
       SqrtBox["5"]}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"1", "+", 
       SqrtBox["5"]}], ")"}]}], 
    RowBox[{"8", " ", 
     SqrtBox["5"]}]]}], "+", 
  RowBox[{
   FractionBox["1", "10"], " ", 
   RowBox[{"(", 
    RowBox[{"5", "+", 
     SqrtBox["5"]}], ")"}]}]}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "However, applying the ",
 StyleBox[ButtonBox["Simplify",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Simplify"], "FunctionLink"],
 " function will reduce the expression to the expected value."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"fibonacci2", "[", "1", "]"}], "//", "Simplify"}]], "Input"],

Cell[BoxData["1"], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["A Solver", "Subsubsection"],

Cell[TextData[{
 "Now let's generalize what we have been doing and write a ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " function to solve a degree two linear, homogeneous recurrence relation \
with constant coefficients, provided that the roots of the characteristic \
polynomial are distinct. We will write a function ",
 StyleBox["recSol2Distinct", "Input"],
 " which solves the recurrence"
}], "Text"],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["a", "n"], "=", 
   RowBox[{
    RowBox[{"c", "\[CenterDot]", 
     SubscriptBox["a", 
      RowBox[{"n", "-", "1"}]]}], "+", 
    RowBox[{"d", "\[CenterDot]", 
     SubscriptBox["a", 
      RowBox[{"n", "-", "2"}]]}]}]}], TraditionalForm]], "Equation"],

Cell[TextData[{
 "subject to the initial conditions ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["a", "1"], "=", "u"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["a", "2"], "=", "v"}], TraditionalForm]]],
 ", and then returns an expression that can be used to compute terms of the \
sequence."
}], "Text"],

Cell[TextData[{
 "For the moment, we assume that the characteristic polynomial ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["r", "2"], "-", 
    RowBox[{"c", "\[CenterDot]", "r"}], "-", "d"}], TraditionalForm]]],
 " has two distinct roots. Later, we will modify the function to relax that \
restriction. With the assumption that the roots of the characteristic \
polynomial are distinct, all our function needs to do is to repeat the steps \
we did manually in the examples above."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"recSol2Distinct", "[", 
    RowBox[{"c_", ",", "d_", ",", "u_", ",", "v_"}], "]"}], "/;", 
   RowBox[{
    RowBox[{"ValueQ", "[", "n", "]"}], "\[Equal]", "False"}]}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "CERoots", ",", "alphas", ",", "alpha", ",", "beta", ",", "f", ",", 
      "r"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"(*", " ", 
     RowBox[{
     "first", " ", "solve", " ", "the", " ", "characteristic", " ", 
      "equation"}], " ", "*)"}], "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"CERoots", "=", 
      RowBox[{"r", "/.", 
       RowBox[{"Solve", "[", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"r", "^", "2"}], "-", 
           RowBox[{"c", "*", "r"}], "-", "d"}], "\[Equal]", "0"}], ",", "r"}],
         "]"}]}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{
      "next", " ", "solve", " ", "using", " ", "the", " ", "initial", " ", 
       "conditions"}], " ", "*)"}], "\[IndentingNewLine]", 
     RowBox[{"alphas", "=", 
      RowBox[{
       RowBox[{"Solve", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"alpha", "*", 
              RowBox[{"CERoots", "[", 
               RowBox[{"[", "1", "]"}], "]"}]}], "+", 
             RowBox[{"beta", "*", 
              RowBox[{"CERoots", "[", 
               RowBox[{"[", "2", "]"}], "]"}]}]}], "\[Equal]", "u"}], ",", 
           RowBox[{
            RowBox[{
             RowBox[{"alpha", "*", 
              RowBox[{
               RowBox[{"CERoots", "[", 
                RowBox[{"[", "1", "]"}], "]"}], "^", "2"}]}], "+", 
             RowBox[{"beta", "*", 
              RowBox[{
               RowBox[{"CERoots", "[", 
                RowBox[{"[", "2", "]"}], "]"}], "^", "2"}]}]}], "\[Equal]", 
            "v"}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"alpha", ",", "beta"}], "}"}]}], "]"}], "[", 
       RowBox[{"[", "1", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{
      "finally", " ", "substitute", " ", "the", " ", "results", " ", "into", 
       " ", "the", " ", "general", " ", "form"}], " ", "*)"}], 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{
       RowBox[{"alpha", "*", 
        RowBox[{
         RowBox[{"CERoots", "[", 
          RowBox[{"[", "1", "]"}], "]"}], "^", "n"}]}], "+", 
       RowBox[{"beta", "*", 
        RowBox[{
         RowBox[{"CERoots", "[", 
          RowBox[{"[", "2", "]"}], "]"}], "^", "n"}]}]}], "/.", 
      "alphas"}]}]}], "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Observe that in the above, the symbol ",
 StyleBox["n", "Input"],
 " is used without declaring it as local to the ",
 StyleBox[ButtonBox["Module",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Module"], "FunctionLink"],
 ". This is necessary in order to have the resulting expression in terms of \
",
 StyleBox["n", "Input"],
 ". However, it cannot be used if ",
 StyleBox["n", "Input"],
 " has been assigned a value elsewhere. This is why we ",
 StyleBox[ButtonBox["Condition",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Condition"], "FunctionLink"],
 " (",
 StyleBox["/;", "Input"],
 ") the definition on ",
 Cell[BoxData[
  RowBox[{
   RowBox[{"ValueQ", "[", "n", "]"}], "==", "False"}]], "myCode"],
 ". The ",
 StyleBox[ButtonBox["ValueQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ValueQ"], "FunctionLink"],
 " function determines whether a symbol has been assigned a value or not."
}], "Text"],

Cell["\<\
To see how it works, we'll check that it gives the same result for the \
Fibonacci sequence that we obtained by hand. To construct a function for \
computing the Fibonacci sequence, invoke the new function as: \
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"f", "[", "n_", "]"}], "=", 
  RowBox[{"recSol2Distinct", "[", 
   RowBox[{"1", ",", "1", ",", "1", ",", "1"}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", 
   FractionBox[
    RowBox[{
     SuperscriptBox["2", 
      RowBox[{
       RowBox[{"-", "2"}], "-", "n"}]], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"1", "-", 
        SqrtBox["5"]}], ")"}], "n"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", "1"}], "+", 
       SqrtBox["5"]}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"1", "+", 
       SqrtBox["5"]}], ")"}]}], 
    SqrtBox["5"]]}], "-", 
  RowBox[{
   FractionBox["1", "5"], " ", 
   SuperscriptBox["2", 
    RowBox[{"-", "n"}]], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", "5"}], "-", 
     SqrtBox["5"]}], ")"}], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"1", "+", 
      SqrtBox["5"]}], ")"}], 
    RowBox[{
     RowBox[{"-", "1"}], "+", "n"}]]}]}]], "Output"]
}, Open  ]],

Cell["\<\
You can see that is the same formula that we derived above. And the first 10 \
Fibonacci numbers can be computed as follows. \
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Table", "[", 
  RowBox[{
   RowBox[{"Simplify", "[", 
    RowBox[{"f", "[", "n", "]"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"n", ",", "1", ",", "10"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "1", ",", "1", ",", "2", ",", "3", ",", "5", ",", "8", ",", "13", ",", "21",
    ",", "34", ",", "55"}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["A Recurrence with Repeated Roots", "Subsubsection"],

Cell["\<\
We will next create a function that can handle the case of repeated roots. \
But first, let's look at an example of a recurrence relation whose \
characteristic polynomial has a double root. The recurrence relation\
\>", "Text"],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["a", "n"], "=", 
   RowBox[{
    RowBox[{"4", 
     SubscriptBox["a", 
      RowBox[{"n", "-", "1"}]]}], "-", 
    RowBox[{"4", 
     SubscriptBox["a", 
      RowBox[{"n", "-", "2"}]]}]}]}], TraditionalForm]], "Equation"],

Cell["has characteristic equation", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"charEqn", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"r", "^", "2"}], "-", 
    RowBox[{"4", "*", "r"}], "+", "4"}], "\[Equal]", "0"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"4", "-", 
   RowBox[{"4", " ", "r"}], "+", 
   SuperscriptBox["r", "2"]}], "\[Equal]", "0"}]], "Output"]
}, Open  ]],

Cell["Its roots are:", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CERoots", "=", 
  RowBox[{"r", "/.", 
   RowBox[{"Solve", "[", 
    RowBox[{"charEqn", ",", "r"}], "]"}]}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"2", ",", "2"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "We can clearly see that in this case the root is repeated, but for ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to recognize it, we'll need to use the following test."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"CERoots", "[", 
   RowBox[{"[", "1", "]"}], "]"}], "\[Equal]", 
  RowBox[{"CERoots", "[", 
   RowBox[{"[", "2", "]"}], "]"}]}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]],

Cell[TextData[{
 "If we call the double root (2 in this case) ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["r", "0"], TraditionalForm]]],
 ", then the recurrence relation has the explicit solution"
}], "Text"],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["a", "n"], "=", 
   RowBox[{
    RowBox[{"\[Alpha]", "\[CenterDot]", 
     SubsuperscriptBox["r", "0", "n"]}], "+", 
    RowBox[{"n", "\[CenterDot]", "\[Beta]", "\[CenterDot]", 
     SubsuperscriptBox["r", "0", "n"]}]}]}], TraditionalForm]], "Equation"],

Cell[TextData[{
 "for all positive integers ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ", and for some constants \[Alpha] and \[Beta]. The initial conditions of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["a", "1"], "=", "1"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["a", "2"], "=", "4"}], TraditionalForm]]],
 " produce the system of equations"
}], "Text"],

Cell[BoxData[
 FormBox[
  RowBox[{"\[Piecewise]", GridBox[{
     {
      RowBox[{
       RowBox[{
        RowBox[{"\[Alpha]", "\[CenterDot]", 
         SuperscriptBox["2", "1"]}], "+", 
        RowBox[{"1", "\[CenterDot]", "\[Beta]", "\[CenterDot]", 
         SuperscriptBox["2", "1"]}]}], "=", "1"}]},
     {
      RowBox[{
       RowBox[{
        RowBox[{"\[Alpha]", "\[CenterDot]", 
         SuperscriptBox["2", "2"]}], "+", 
        RowBox[{"2", "\[CenterDot]", "\[Beta]", "\[CenterDot]", 
         SuperscriptBox["2", "2"]}]}], "=", "4"}]}
    }]}], TraditionalForm]], "Equation"],

Cell["\<\
As before, we solve this system for \[Alpha] and \[Beta].\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"alphas2", "=", 
  RowBox[{
   RowBox[{"Solve", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"alpha", "*", 
          RowBox[{"CERoots", "[", 
           RowBox[{"[", "1", "]"}], "]"}]}], "+", 
         RowBox[{"beta", "*", 
          RowBox[{"CERoots", "[", 
           RowBox[{"[", "1", "]"}], "]"}]}]}], "\[Equal]", "1"}], ",", 
       RowBox[{
        RowBox[{
         RowBox[{"alpha", "*", 
          RowBox[{
           RowBox[{"CERoots", "[", 
            RowBox[{"[", "1", "]"}], "]"}], "^", "2"}]}], "+", 
         RowBox[{"2", "*", "beta", "*", 
          RowBox[{
           RowBox[{"CERoots", "[", 
            RowBox[{"[", "1", "]"}], "]"}], "^", "2"}]}]}], "\[Equal]", 
        "4"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"alpha", ",", "beta"}], "}"}]}], "]"}], "[", 
   RowBox[{"[", "1", "]"}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"alpha", "\[Rule]", "0"}], ",", 
   RowBox[{"beta", "\[Rule]", 
    FractionBox["1", "2"]}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "And finally, substitute these values into the general form ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["a", "n"], "=", 
    RowBox[{
     RowBox[{"\[Alpha]", "\[CenterDot]", 
      SubsuperscriptBox["r", "0", "n"]}], "+", 
     RowBox[{"n", "\[CenterDot]", "\[Beta]", "\[CenterDot]", 
      SubsuperscriptBox["r", "0", "n"]}]}]}], TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"alpha", "*", 
    RowBox[{
     RowBox[{"CERoots", "[", 
      RowBox[{"[", "1", "]"}], "]"}], "^", "n"}]}], "+", 
   RowBox[{"n", "*", "beta", "*", 
    RowBox[{
     RowBox[{"CERoots", "[", 
      RowBox[{"[", "1", "]"}], "]"}], "^", "n"}]}]}], "/.", 
  "alphas2"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SuperscriptBox["2", 
   RowBox[{
    RowBox[{"-", "1"}], "+", "n"}]], " ", "n"}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["A More General Recurrence Solver", "Subsubsection"],

Cell[TextData[{
 "The steps carried out above are quite general and we can write a function, ",
 StyleBox["recSolver2", "Input"],
 ", which solves a recurrence relation (degree two, linear, homogeneous, with \
constant coefficients) regardless of whether the characteristic polynomial \
has distinct roots or not. The following function solves the recurrence"
}], "Text"],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["a", "n"], "=", 
   RowBox[{
    RowBox[{"c", "\[CenterDot]", 
     SubscriptBox["a", 
      RowBox[{"n", "-", "1"}]]}], "+", 
    RowBox[{"d", "\[CenterDot]", 
     SubscriptBox["a", 
      RowBox[{"n", "-", "2"}]]}]}]}], TraditionalForm]], "Equation"],

Cell[TextData[{
 "with initial conditions ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["a", "1"], "=", "u"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["a", "2"], "=", "v"}], TraditionalForm]]],
 "."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"recSolver2", "[", 
    RowBox[{"c_", ",", "d_", ",", "u_", ",", "v_"}], "]"}], "/;", 
   RowBox[{
    RowBox[{"ValueQ", "[", "n", "]"}], "\[Equal]", "False"}]}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"CERoots", ",", "alphas", ",", "alpha", ",", "beta", ",", "r"}], 
     "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"(*", " ", 
     RowBox[{
     "first", " ", "solve", " ", "the", " ", "characteristic", " ", 
      "equation"}], " ", "*)"}], "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"CERoots", "=", 
      RowBox[{"r", "/.", 
       RowBox[{"Solve", "[", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"r", "^", "2"}], "-", 
           RowBox[{"c", "*", "r"}], "-", "d"}], "\[Equal]", "0"}], ",", "r"}],
         "]"}]}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{
      "then", " ", "test", " ", "if", " ", "the", " ", "roots", " ", "are", 
       " ", "the", " ", "same"}], " ", "*)"}], "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"CERoots", "[", 
         RowBox[{"[", "1", "]"}], "]"}], "\[Equal]", 
        RowBox[{"CERoots", "[", 
         RowBox[{"[", "2", "]"}], "]"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{"(*", " ", 
        RowBox[{
         RowBox[{"the", " ", "roots", " ", "are", " ", "the", " ", "same"}], 
         ",", " ", 
         RowBox[{"follow", " ", "the", " ", "last", " ", "example"}]}], " ", 
        "*)"}], "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"alphas", "=", 
         RowBox[{
          RowBox[{"Solve", "[", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{
              RowBox[{
               RowBox[{
                RowBox[{"alpha", "*", 
                 RowBox[{"CERoots", "[", 
                  RowBox[{"[", "1", "]"}], "]"}]}], "+", 
                RowBox[{"beta", "*", 
                 RowBox[{"CERoots", "[", 
                  RowBox[{"[", "1", "]"}], "]"}]}]}], "\[Equal]", "u"}], ",", 
              
              RowBox[{
               RowBox[{
                RowBox[{"alpha", "*", 
                 RowBox[{
                  RowBox[{"CERoots", "[", 
                   RowBox[{"[", "1", "]"}], "]"}], "^", "2"}]}], "+", 
                RowBox[{"2", "*", "beta", "*", 
                 RowBox[{
                  RowBox[{"CERoots", "[", 
                   RowBox[{"[", "1", "]"}], "]"}], "^", "2"}]}]}], "\[Equal]",
                "v"}]}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"alpha", ",", "beta"}], "}"}]}], "]"}], "[", 
          RowBox[{"[", "1", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"Return", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"alpha", "*", 
            RowBox[{
             RowBox[{"CERoots", "[", 
              RowBox[{"[", "1", "]"}], "]"}], "^", "n"}]}], "+", 
           RowBox[{"n", "*", "beta", "*", 
            RowBox[{
             RowBox[{"CERoots", "[", 
              RowBox[{"[", "1", "]"}], "]"}], "^", "n"}]}]}], "/.", 
          "alphas"}], "]"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{"(*", " ", 
        RowBox[{"otherwise", ",", " ", 
         RowBox[{"use", " ", "the", " ", "recSol2Distinct", " ", "method"}]}],
         " ", "*)"}], "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"alphas", "=", 
         RowBox[{
          RowBox[{"Solve", "[", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{
              RowBox[{
               RowBox[{
                RowBox[{"alpha", "*", 
                 RowBox[{"CERoots", "[", 
                  RowBox[{"[", "1", "]"}], "]"}]}], "+", 
                RowBox[{"beta", "*", 
                 RowBox[{"CERoots", "[", 
                  RowBox[{"[", "2", "]"}], "]"}]}]}], "\[Equal]", "u"}], ",", 
              
              RowBox[{
               RowBox[{
                RowBox[{"alpha", "*", 
                 RowBox[{
                  RowBox[{"CERoots", "[", 
                   RowBox[{"[", "1", "]"}], "]"}], "^", "2"}]}], "+", 
                RowBox[{"beta", "*", 
                 RowBox[{
                  RowBox[{"CERoots", "[", 
                   RowBox[{"[", "2", "]"}], "]"}], "^", "2"}]}]}], "\[Equal]",
                "v"}]}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"alpha", ",", "beta"}], "}"}]}], "]"}], "[", 
          RowBox[{"[", "1", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"Return", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"alpha", "*", 
            RowBox[{
             RowBox[{"CERoots", "[", 
              RowBox[{"[", "1", "]"}], "]"}], "^", "n"}]}], "+", 
           RowBox[{"beta", "*", 
            RowBox[{
             RowBox[{"CERoots", "[", 
              RowBox[{"[", "2", "]"}], "]"}], "^", "n"}]}]}], "/.", 
          "alphas"}], "]"}]}]}], "\[IndentingNewLine]", "]"}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "The ",
 StyleBox["recSolver2", "Input"],
 " function first tests for a repeated root and then does the appropriate \
computation. We test this function on the examples we did by hand, such as \
the Fibonacci sequence:"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"recSolver2", "[", 
  RowBox[{"1", ",", "1", ",", "1", ",", "1"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", 
   FractionBox[
    RowBox[{
     SuperscriptBox["2", 
      RowBox[{
       RowBox[{"-", "2"}], "-", "n"}]], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"1", "-", 
        SqrtBox["5"]}], ")"}], "n"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", "1"}], "+", 
       SqrtBox["5"]}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"1", "+", 
       SqrtBox["5"]}], ")"}]}], 
    SqrtBox["5"]]}], "-", 
  RowBox[{
   FractionBox["1", "5"], " ", 
   SuperscriptBox["2", 
    RowBox[{"-", "n"}]], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", "5"}], "-", 
     SqrtBox["5"]}], ")"}], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"1", "+", 
      SqrtBox["5"]}], ")"}], 
    RowBox[{
     RowBox[{"-", "1"}], "+", "n"}]]}]}]], "Output"]
}, Open  ]],

Cell["And for the example with a double root:", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"recSolver2", "[", 
  RowBox[{"4", ",", 
   RowBox[{"-", "4"}], ",", "1", ",", "4"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SuperscriptBox["2", 
   RowBox[{
    RowBox[{"-", "1"}], "+", "n"}]], " ", "n"}]], "Output"]
}, Open  ]],

Cell["\<\
In both of those examples, the result is consistent with what we had obtained \
before. We will now use the solver to find the first ten terms of the \
sequence defined by the following recurrence relation and initial conditions. \
\>", "Text"],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["a", "n"], "=", 
    RowBox[{
     RowBox[{"4", 
      SubscriptBox["a", 
       RowBox[{"n", "-", "1"}]]}], "-", 
     RowBox[{"3", 
      SubscriptBox["a", 
       RowBox[{"n", "-", "2"}]]}]}]}], ",", "   ", 
   RowBox[{
    SubscriptBox["a", "1"], "=", "1"}], ",", "  ", 
   RowBox[{
    RowBox[{"and", " ", 
     SubscriptBox["a", "2"]}], "=", "2"}]}], TraditionalForm]], "Equation"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"g", "[", "n_", "]"}], "=", 
  RowBox[{"recSolver2", "[", 
   RowBox[{"4", ",", 
    RowBox[{"-", "3"}], ",", "1", ",", "2"}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  FractionBox["1", "2"], "+", 
  FractionBox[
   SuperscriptBox["3", 
    RowBox[{
     RowBox[{"-", "1"}], "+", "n"}]], "2"]}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Table", "[", 
  RowBox[{
   RowBox[{"Simplify", "[", 
    RowBox[{"g", "[", "n", "]"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"n", ",", "1", ",", "10"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "1", ",", "2", ",", "5", ",", "14", ",", "41", ",", "122", ",", "365", ",", 
   "1094", ",", "3281", ",", "9842"}], "}"}]], "Output"]
}, Open  ]],

Cell["\<\
As another example, consider the following recurrence relation\
\>", "Text"],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["a", "n"], "=", 
   RowBox[{
    RowBox[{"-", 
     SubscriptBox["a", 
      RowBox[{"n", "-", "1"}]]}], "-", 
    SubscriptBox["a", 
     RowBox[{"n", "-", "2"}]]}]}], TraditionalForm]], "Equation"],

Cell[TextData[{
 "with initial conditions ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["a", "1"], "=", "1"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["a", "2"], "=", "2"}], TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"h", "[", "n_", "]"}], "=", 
  RowBox[{"recSolver2", "[", 
   RowBox[{
    RowBox[{"-", "1"}], ",", 
    RowBox[{"-", "1"}], ",", "1", ",", "2"}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], 
      RowBox[{
       RowBox[{"2", " ", "n"}], "/", "3"}]], " ", 
     RowBox[{"(", 
      RowBox[{"2", "+", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"-", "1"}], ")"}], 
        RowBox[{"1", "/", "3"}]]}], ")"}]}], 
    RowBox[{"1", "+", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], 
      RowBox[{"1", "/", "3"}]]}]]}], "-", 
  FractionBox[
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"-", "1"}], ")"}], 
        RowBox[{"1", "/", "3"}]]}], ")"}], "n"], " ", 
    RowBox[{"(", 
     RowBox[{"1", "+", 
      RowBox[{"2", " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"-", "1"}], ")"}], 
        RowBox[{"1", "/", "3"}]]}]}], ")"}]}], 
   RowBox[{"1", "+", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}], 
     RowBox[{"1", "/", "3"}]]}]]}]], "Output"]
}, Open  ]],

Cell["\<\
Notice that the solution to this recurrence is very complicated and requires \
the use of cube roots of -1. But if we compute the first ten terms, we notice \
a very simple pattern emerges.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Table", "[", 
  RowBox[{
   RowBox[{"Simplify", "[", 
    RowBox[{"h", "[", "n", "]"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"n", ",", "1", ",", "10"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "2", ",", 
   RowBox[{"-", "3"}], ",", "1", ",", "2", ",", 
   RowBox[{"-", "3"}], ",", "1", ",", "2", ",", 
   RowBox[{"-", "3"}], ",", "1"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " can make this pattern explicit if we replace the numerical initial \
conditions with symbolic constants."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"k", "[", "n_", "]"}], "=", 
  RowBox[{"recSolver2", "[", 
   RowBox[{
    RowBox[{"-", "1"}], ",", 
    RowBox[{"-", "1"}], ",", "\[Lambda]", ",", "\[Mu]"}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], 
      RowBox[{
       RowBox[{"2", " ", "n"}], "/", "3"}]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"-", "1"}], ")"}], 
         RowBox[{"1", "/", "3"}]], " ", "\[Lambda]"}], "+", "\[Mu]"}], 
      ")"}]}], 
    RowBox[{"1", "+", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], 
      RowBox[{"1", "/", "3"}]]}]]}], "-", 
  FractionBox[
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"-", "1"}], ")"}], 
        RowBox[{"1", "/", "3"}]]}], ")"}], "n"], " ", 
    RowBox[{"(", 
     RowBox[{"\[Lambda]", "+", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"-", "1"}], ")"}], 
        RowBox[{"1", "/", "3"}]], " ", "\[Mu]"}]}], ")"}]}], 
   RowBox[{"1", "+", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}], 
     RowBox[{"1", "/", "3"}]]}]]}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Table", "[", 
  RowBox[{
   RowBox[{"Simplify", "[", 
    RowBox[{"k", "[", "n", "]"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"n", ",", "1", ",", "10"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\[Lambda]", ",", "\[Mu]", ",", 
   RowBox[{
    RowBox[{"-", "\[Lambda]"}], "-", "\[Mu]"}], ",", "\[Lambda]", ",", 
   "\[Mu]", ",", 
   RowBox[{
    RowBox[{"-", "\[Lambda]"}], "-", "\[Mu]"}], ",", "\[Lambda]", ",", 
   "\[Mu]", ",", 
   RowBox[{
    RowBox[{"-", "\[Lambda]"}], "-", "\[Mu]"}], ",", "\[Lambda]"}], 
  "}"}]], "Output"]
}, Open  ]],

Cell["\<\
Note that the Greek letters are entered by pressing \[EscapeKey], then typing \
the name of the letter, and pressing \[EscapeKey] again, e.g., \
\[EscapeKey]lambda\[EscapeKey] produces \[Lambda].\
\>", "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Nonhomogeneous Recurrence Relations", "Subsection",
 CellTags->"Section2b"],

Cell[TextData[{
 "So far, we have been restricted to homogeneous linear recurrence relations \
with constant coefficients. However, the techniques used in solving them may \
be extended to provide solutions to ",
 StyleBox["nonhomogeneous",
  FontSlant->"Italic"],
 " linear recurrence relations with constant coefficients. That is, \
recurrence relations of the form"
}], "Text"],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["a", "n"], "=", 
   RowBox[{
    RowBox[{
     SubscriptBox["c", "1"], 
     SubscriptBox["a", 
      RowBox[{"n", "-", "1"}]]}], "+", 
    RowBox[{
     SubscriptBox["c", "2"], 
     SubscriptBox["a", 
      RowBox[{"n", "-", "2"}]]}], "+", "\[CenterEllipsis]", "+", 
    RowBox[{
     SubscriptBox["c", "k"], 
     SubscriptBox["a", 
      RowBox[{"n", "-", "k"}]]}], "+", 
    RowBox[{"F", "(", "n", ")"}]}]}], TraditionalForm]], "Equation"],

Cell[TextData[{
 "with ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["c", "1"], ",", 
    SubscriptBox["c", "2"], ",", "\[Ellipsis]", ",", 
    SubscriptBox["c", "k"]}], TraditionalForm]]],
 " real numbers and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"F", "(", "n", ")"}], TraditionalForm]]],
 " a function depending only on ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ". To solve this more general form of a recurrence relation, we do two \
things: (1) find the solutions of the associated homogeneous recurrence \
relation (the relation obtained by removing ",
 Cell[BoxData[
  FormBox[
   RowBox[{"F", "(", "n", ")"}], TraditionalForm]]],
 "); and (2) find a particular solution for the nonhomogeneous equation."
}], "Text"],

Cell["Consider the following example: ", "Text"],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["a", "n"], "=", 
   RowBox[{
    RowBox[{"6", 
     SubscriptBox["a", 
      RowBox[{"n", "-", "1"}]]}], "-", 
    RowBox[{"9", 
     SubscriptBox["a", 
      RowBox[{"n", "-", "2"}]]}], "+", 
    RowBox[{"n", "\[CenterDot]", 
     SuperscriptBox["3", "n"]}]}]}], TraditionalForm]], "Equation"],

Cell["from Example 12 of Section 8.2 in the text.", "Text"],

Cell["\<\
The first step is to find the solutions to the associated homogeneous \
recurrence relation\
\>", "Text"],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["a", "n"], "=", 
   RowBox[{
    RowBox[{"6", 
     SubscriptBox["a", 
      RowBox[{"n", "-", "1"}]]}], "-", 
    RowBox[{"9", 
     SubscriptBox["a", 
      RowBox[{"n", "-", "2"}]]}]}]}], TraditionalForm]], "Equation"],

Cell[TextData[{
 "To do this, we can use our ",
 StyleBox["recSolver2", "Input"],
 ". We will use \[Alpha] and \[Beta] for the initial conditions so that we \
get all the solutions."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"hSolution", "[", "n_", "]"}], "=", 
  RowBox[{"recSolver2", "[", 
   RowBox[{"6", ",", 
    RowBox[{"-", "9"}], ",", "\[Alpha]", ",", "\[Beta]"}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SuperscriptBox["3", 
    RowBox[{
     RowBox[{"-", "2"}], "+", "n"}]], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"6", " ", "\[Alpha]"}], "-", "\[Beta]"}], ")"}]}], "+", 
  RowBox[{
   SuperscriptBox["3", "n"], " ", "n", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["\[Alpha]", "3"]}], "+", 
     FractionBox["\[Beta]", "9"]}], ")"}]}]}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The second step is to find a particular solution. Theorem 6 in section 8.2 \
of the text tells us how to find the form of the particular solution. Note \
that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"F", "(", "n", ")"}], "=", 
    RowBox[{"n", "\[CenterDot]", 
     SuperscriptBox["3", "n"]}]}], TraditionalForm]]],
 " and 3 is a root of the characteristic polynomial with multiplicity 2 (you \
can verify this by solving the characteristic equation of the associated \
homogeneous relation; it is also made apparent by the form of ",
 StyleBox["hSolution", "Input"],
 "). So the theorem tells us that there is a particular solution of the form"
}], "Text"],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SuperscriptBox["n", "2"], "(", 
    RowBox[{
     RowBox[{"p", "\[CenterDot]", "n"}], "+", "q"}], ")"}], 
   SuperscriptBox["3", "n"]}], TraditionalForm]], "Equation"],

Cell["\<\
We will define a function for the form of this particular solution.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"pForm", "[", "n_", "]"}], "=", 
  RowBox[{
   RowBox[{"n", "^", "2"}], "*", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"p", "*", "n"}], "+", "q"}], ")"}], "*", 
   RowBox[{"3", "^", "n"}]}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  SuperscriptBox["3", "n"], " ", 
  SuperscriptBox["n", "2"], " ", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"n", " ", "p"}], "+", "q"}], ")"}]}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "To find a particular solution, we need to find the values of ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["q", TraditionalForm]]],
 ". To find these values, we substitute the terms of ",
 StyleBox["pForm", "Input"],
 " into the recurrence relation. This gives us an equation in terms of ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["q", TraditionalForm]]],
 " (and ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ")."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"pEqn", "=", 
  RowBox[{
   RowBox[{"pForm", "[", "n", "]"}], "\[Equal]", 
   RowBox[{
    RowBox[{"6", "*", 
     RowBox[{"pForm", "[", 
      RowBox[{"n", "-", "1"}], "]"}]}], "-", 
    RowBox[{"9", "*", 
     RowBox[{"pForm", "[", 
      RowBox[{"n", "-", "2"}], "]"}]}], "+", 
    RowBox[{"n", "*", 
     RowBox[{"3", "^", "n"}]}]}]}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SuperscriptBox["3", "n"], " ", 
   SuperscriptBox["n", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"n", " ", "p"}], "+", "q"}], ")"}]}], "\[Equal]", 
  RowBox[{
   RowBox[{
    SuperscriptBox["3", "n"], " ", "n"}], "-", 
   RowBox[{
    SuperscriptBox["3", "n"], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", "2"}], "+", "n"}], ")"}], "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "2"}], "+", "n"}], ")"}], " ", "p"}], "+", "q"}], 
     ")"}]}], "+", 
   RowBox[{"2", " ", 
    SuperscriptBox["3", "n"], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", "1"}], "+", "n"}], ")"}], "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "1"}], "+", "n"}], ")"}], " ", "p"}], "+", "q"}], 
     ")"}]}]}]}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "We\[CloseCurlyQuote]ll simplify the equation with ",
 StyleBox[ButtonBox["Simplify",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Simplify"], "FunctionLink"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"pEqn", "=", 
  RowBox[{"Simplify", "[", "pEqn", "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SuperscriptBox["3", "n"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"-", "6"}], " ", "p"}], "+", 
     RowBox[{"n", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "1"}], "+", 
        RowBox[{"6", " ", "p"}]}], ")"}]}], "+", 
     RowBox[{"2", " ", "q"}]}], ")"}]}], "\[Equal]", "0"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Next we\[CloseCurlyQuote]ll have ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " solve that equation for ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["q", TraditionalForm]]],
 ". In order to indicate that we want ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to find values of ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["q", TraditionalForm]]],
 " that satisfy the equation for all values of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ", we will ask ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to solve"
}], "Text"],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"\[ForAll]", "n"}], ")"}], " ", 
    SuperscriptBox["3", "n"], 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", "6"}], "p"}], "+", 
      RowBox[{"n", "(", 
       RowBox[{
        RowBox[{"-", "1"}], "+", 
        RowBox[{"6", "p"}]}], ")"}], "+", 
      RowBox[{"2", "q"}]}], ")"}]}], "=", "0"}], TraditionalForm]], "Equation"],

Cell[TextData[{
 "We encode this using the ",
 StyleBox[ButtonBox["ForAll",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ForAll"], "FunctionLink"],
 " expression, with first argument ",
 StyleBox["n", "Input"],
 ", indicating that ",
 StyleBox["n", "Input"],
 " is the variable being quantified, and with second argument the equation. \
This expression will be the first argument to ",
 StyleBox[ButtonBox["Solve",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Solve"], "FunctionLink"],
 ". The second argument will be ",
 StyleBox["{p,q}", "Input"],
 ", indicating that those are the variables we wish to solve for."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"pVals", "=", 
  RowBox[{"Solve", "[", 
   RowBox[{
    RowBox[{"ForAll", "[", 
     RowBox[{"n", ",", "pEqn"}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"p", ",", "q"}], "}"}]}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"p", "\[Rule]", 
     FractionBox["1", "6"]}], ",", 
    RowBox[{"q", "\[Rule]", 
     FractionBox["1", "2"]}]}], "}"}], "}"}]], "Output"]
}, Open  ]],

Cell["Thus, the particular solution is:", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"pForm", "[", "n", "]"}], "/.", 
  RowBox[{"pVals", "[", 
   RowBox[{"[", "1", "]"}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SuperscriptBox["3", "n"], " ", 
  RowBox[{"(", 
   RowBox[{
    FractionBox["1", "2"], "+", 
    FractionBox["n", "6"]}], ")"}], " ", 
  SuperscriptBox["n", "2"]}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Putting it all together, we see that all solutions to the recurrence \
relation ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["a", "n"], "=", 
    RowBox[{
     RowBox[{"6", 
      SubscriptBox["a", 
       RowBox[{"n", "-", "1"}]]}], "-", 
     RowBox[{"9", 
      SubscriptBox["a", 
       RowBox[{"n", "-", "2"}]]}], "+", 
     RowBox[{"n", "\[CenterDot]", 
      SuperscriptBox["3", "n"]}]}]}], TraditionalForm]],
  CellChangeTimes->{{3.558282112814363*^9, 3.558282131726244*^9}}],
 " are of the form:"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"hSolution", "[", "n", "]"}], "+", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"pForm", "[", "n", "]"}], "/.", 
    RowBox[{"pVals", "[", 
     RowBox[{"[", "1", "]"}], "]"}]}], ")"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SuperscriptBox["3", "n"], " ", 
   RowBox[{"(", 
    RowBox[{
     FractionBox["1", "2"], "+", 
     FractionBox["n", "6"]}], ")"}], " ", 
   SuperscriptBox["n", "2"]}], "+", 
  RowBox[{
   SuperscriptBox["3", 
    RowBox[{
     RowBox[{"-", "2"}], "+", "n"}]], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"6", " ", "\[Alpha]"}], "-", "\[Beta]"}], ")"}]}], "+", 
  RowBox[{
   SuperscriptBox["3", "n"], " ", "n", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["\[Alpha]", "3"]}], "+", 
     FractionBox["\[Beta]", "9"]}], ")"}]}]}]], "Output"]
}, Open  ]],

Cell["\<\
where \[Alpha] and \[Beta] are the initial conditions.\
\>", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "\[CloseCurlyQuote]s Recurrence Solver"
}], "Subsection",
 CellTags->"Section2c"],

Cell[TextData[{
 "Now that we have seen how to use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to implement an algorithm to solve simple recurrence relations, it is time \
to introduce ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "'s built-in function for solving recurrence relations."
}], "Text"],

Cell[TextData[{
 "We have already seen the ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " function ",
 StyleBox[ButtonBox["Solve",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Solve"], "FunctionLink"],
 " for working with polynomial equations and systems of equations. Similarly, \
there is a function ",
 StyleBox[ButtonBox["RSolve",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/RSolve"], "FunctionLink"],
 ", which is engineered for dealing with recurrence relations. It is a much \
more sophisticated version of our ",
 StyleBox["recSolver2", "Input"],
 " function and can deal with recurrence relations of arbitrary degree, \
repeated roots, and nonlinear recurrence relations. The syntax of ",
 StyleBox[ButtonBox["RSolve",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/RSolve"], "FunctionLink"],
 " is very similar to that of ",
 StyleBox[ButtonBox["RecurrenceTable",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/RecurrenceTable"], "FunctionLink"],
 ". The first argument is an equation or list of equations representing the \
recurrence relation and any initial conditions in terms of symbols of the \
form ",
 StyleBox["a[n+1]", "Input"],
 ", ",
 StyleBox["a[n]", "Input"],
 ", ",
 StyleBox["a[n-1]", "Input"],
 ", etc. The second argument is of one of two forms: ",
 StyleBox["a", "Input"],
 " or ",
 StyleBox["a[n]", "Input"],
 ", where ",
 StyleBox["a", "Input"],
 " is the symbol used in the first argument to name the sequence. The form of \
the output depends on which of these is used. The final argument is the \
symbol used in the first argument to represent the index of the sequence, \
e.g., ",
 StyleBox["n", "Input"],
 "."
}], "Text"],

Cell[TextData[{
 "For example, to solve the recurrence relation ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["a", "n"], "=", 
    RowBox[{
     SubscriptBox["a", 
      RowBox[{"n", "-", "1"}]], "+", 
     RowBox[{"2", "\[CenterDot]", 
      SubscriptBox["a", 
       RowBox[{"n", "-", "2"}]]}]}]}], TraditionalForm]]],
 " with initial conditions ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["a", "1"], "=", 
    RowBox[{
     SubscriptBox["a", "2"], "=", "1"}]}], TraditionalForm]]],
 ", you enter the following."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"solveOutput", "=", 
  RowBox[{"RSolve", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       RowBox[{"a", "[", "n", "]"}], "\[Equal]", 
       RowBox[{
        RowBox[{"a", "[", 
         RowBox[{"n", "-", "1"}], "]"}], "+", 
        RowBox[{"2", "*", 
         RowBox[{"a", "[", 
          RowBox[{"n", "-", "2"}], "]"}]}]}]}], ",", 
      RowBox[{
       RowBox[{"a", "[", "1", "]"}], "\[Equal]", 
       RowBox[{"a", "[", "2", "]"}], "\[Equal]", "1"}]}], "}"}], ",", 
    RowBox[{"a", "[", "n", "]"}], ",", "n"}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"a", "[", "n", "]"}], "\[Rule]", 
    RowBox[{
     FractionBox["1", "3"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"-", "1"}], ")"}], "n"]}], "+", 
       SuperscriptBox["2", "n"]}], ")"}]}]}], "}"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Note that the structure of the solution is similar to that given by ",
 StyleBox[ButtonBox["Solve",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Solve"], "FunctionLink"],
 ": a list of lists of rules. If there were multiple solutions, each solution \
would appear as a separate sublist. "
}], "Text"],

Cell[TextData[{
 "In the example above, we gave the second argument in the form ",
 StyleBox["a[n]", "Input"],
 ". This resulted in a rule for ",
 StyleBox["a[n]", "Input"],
 ". The expression on the right side of the rule can be extracted by applying \
",
 StyleBox[ButtonBox["ReplaceAll",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ReplaceAll"], "FunctionLink"],
 " (",
 StyleBox["/.", "Input"],
 ") to the expression ",
 StyleBox["a[n]", "Input"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"a", "[", "n", "]"}], "/.", 
  RowBox[{"First", "[", "solveOutput", "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  FractionBox["1", "3"], " ", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"-", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "n"]}], "+", 
    SuperscriptBox["2", "n"]}], ")"}]}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Let\[CloseCurlyQuote]s break that down a bit. The ",
 StyleBox[ButtonBox["First",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/First"], "FunctionLink"],
 " function simply takes the first element of the ",
 StyleBox["solveOutput", "Input"],
 " list, which is the list containing the rule. So the above is the same as"
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"a", "[", "n", "]"}], "/.", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"a", "[", "n", "]"}], "\[Rule]", 
    RowBox[{
     FractionBox["1", "3"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"-", "1"}], ")"}], "n"]}], "+", 
       SuperscriptBox["2", "n"]}], ")"}]}]}], "}"}]}]], "myCode"],

Cell[TextData[{
 "This uses ",
 StyleBox[ButtonBox["ReplaceAll",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ReplaceAll"], "FunctionLink"],
 " (",
 StyleBox["/.", "Input"],
 ") on the expression ",
 StyleBox["a[n]", "Input"],
 " by applying the rule that substitutes the expression for the symbol ",
 StyleBox["a[n]", "Input"],
 ". In other words, the above evaluates to the expression ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    FractionBox["1", "3"], 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"-", "1"}], ")"}], "n"]}], "+", 
      SuperscriptBox["2", "n"]}], ")"}]}], TraditionalForm]]],
 "."
}], "Text"],

Cell[TextData[{
 "If we give the second argument in the form ",
 StyleBox["a", "Input"],
 " instead of ",
 StyleBox["a[n]", "Input"],
 ", the output will have the same structure, a list of lists of rules, but \
the rules will relate the symbol ",
 StyleBox["a", "Input"],
 " to a pure ",
 StyleBox[ButtonBox["Function",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Function"], "FunctionLink"],
 " (",
 StyleBox["&", "Input"],
 ") rather than an expression."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"solveOutput2", "=", 
  RowBox[{"RSolve", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       RowBox[{"a", "[", "n", "]"}], "\[Equal]", 
       RowBox[{
        RowBox[{"a", "[", 
         RowBox[{"n", "-", "1"}], "]"}], "+", 
        RowBox[{"2", "*", 
         RowBox[{"a", "[", 
          RowBox[{"n", "-", "2"}], "]"}]}]}]}], ",", 
      RowBox[{
       RowBox[{"a", "[", "1", "]"}], "\[Equal]", 
       RowBox[{"a", "[", "2", "]"}], "\[Equal]", "1"}]}], "}"}], ",", "a", 
    ",", "n"}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"{", 
   RowBox[{"a", "\[Rule]", 
    RowBox[{"Function", "[", 
     RowBox[{
      RowBox[{"{", "n", "}"}], ",", 
      RowBox[{
       FractionBox["1", "3"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"-", "1"}], ")"}], "n"]}], "+", 
         SuperscriptBox["2", "n"]}], ")"}]}]}], "]"}]}], "}"}], 
  "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "This output is more difficult to read, but it makes it easier to transform \
the output of ",
 StyleBox[ButtonBox["RSolve",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/RSolve"], "FunctionLink"],
 " into a function that we can use to compute values. To assign the symbol ",
 StyleBox["F", "Input"],
 " to the function that calculates values of this sequence, we just have to \
assign ",
 StyleBox["F", "Input"],
 " to the pure function, which we can extract just as we did above."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"F", "=", 
  RowBox[{"a", "/.", 
   RowBox[{"First", "[", "solveOutput2", "]"}]}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"Function", "[", 
  RowBox[{
   RowBox[{"{", "n", "}"}], ",", 
   RowBox[{
    FractionBox["1", "3"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"-", "1"}], ")"}], "n"]}], "+", 
      SuperscriptBox["2", "n"]}], ")"}]}]}], "]"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Now ",
 StyleBox["F", "Input"],
 " can be used to compute values."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"F", "[", "42", "]"}]], "Input"],

Cell[BoxData["1466015503701"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Table", "[", 
  RowBox[{
   RowBox[{"F", "[", "n", "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"n", ",", "1", ",", "10"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "1", ",", "1", ",", "3", ",", "5", ",", "11", ",", "21", ",", "43", ",", 
   "85", ",", "171", ",", "341"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Observe that these agree with the values obtained with ",
 StyleBox[ButtonBox["RecurrenceTable",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/RecurrenceTable"], "FunctionLink"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"RecurrenceTable", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"a", "[", "n", "]"}], "\[Equal]", 
      RowBox[{
       RowBox[{"a", "[", 
        RowBox[{"n", "-", "1"}], "]"}], "+", 
       RowBox[{"2", "*", 
        RowBox[{"a", "[", 
         RowBox[{"n", "-", "2"}], "]"}]}]}]}], ",", 
     RowBox[{
      RowBox[{"a", "[", "1", "]"}], "\[Equal]", 
      RowBox[{"a", "[", "2", "]"}], "\[Equal]", "1"}]}], "}"}], ",", "a", ",", 
   RowBox[{"{", 
    RowBox[{"n", ",", "1", ",", "10"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "1", ",", "1", ",", "3", ",", "5", ",", "11", ",", "21", ",", "43", ",", 
   "85", ",", "171", ",", "341"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["RSolve",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/RSolve"], "FunctionLink"],
 " function will let us solve nonhomogeneous recurrence relations like the \
Tower of Hanoi problem very easily. Recall that the Tower of Hanoi problem \
has the recurrence relation"
}], "Text"],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["H", "n"], "=", 
   RowBox[{
    RowBox[{"2", "\[CenterDot]", 
     SubscriptBox["H", 
      RowBox[{"n", "-", "1"}]]}], "+", "1"}]}], TraditionalForm]], "Equation"],

Cell[TextData[{
 "with initial condition ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["H", "1"], "=", "1"}], TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"Clear", "[", "H", "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"H", "=", 
  RowBox[{"H", "/.", 
   RowBox[{"First", "[", 
    RowBox[{"RSolve", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"H", "[", "n", "]"}], "\[Equal]", 
         RowBox[{
          RowBox[{"2", "*", 
           RowBox[{"H", "[", 
            RowBox[{"n", "-", "1"}], "]"}]}], "+", "1"}]}], ",", 
        RowBox[{
         RowBox[{"H", "[", "1", "]"}], "\[Equal]", "1"}]}], "}"}], ",", "H", 
      ",", "n"}], "]"}], "]"}]}]}]}], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"Function", "[", 
  RowBox[{
   RowBox[{"{", "n", "}"}], ",", 
   RowBox[{
    RowBox[{"-", "1"}], "+", 
    SuperscriptBox["2", "n"]}]}], "]"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Observe that in the above we used ",
 StyleBox["H", "Input"],
 " for both the name of the sequence and the symbol representing the \
resulting ",
 StyleBox[ButtonBox["Function",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Function"], "FunctionLink"],
 ". While this is natural, it carries the caveat that it can only be executed \
once, because ",
 StyleBox["H", "Input"],
 " now has a value and thus cannot be used within ",
 StyleBox[ButtonBox["RSolve",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/RSolve"], "FunctionLink"],
 ". Hence, it is a good idea to ",
 StyleBox[ButtonBox["Clear",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Clear"], "FunctionLink"],
 " the symbol before making the assignment."
}], "Text"],

Cell[TextData[{
 "It is not necessary to specify the initial conditions for a recurrence \
relation when applying ",
 StyleBox[ButtonBox["RSolve",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/RSolve"], "FunctionLink"],
 ". If they are not present, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " will still solve the equation, inserting symbolic constants (e.g., ",
 StyleBox["C[1]", "Input"],
 " and ",
 StyleBox["C[2]", "Input"],
 ") in place of numeric values, as the following example illustrates."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"Clear", "[", "G", "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"G", "=", 
  RowBox[{"G", "/.", 
   RowBox[{"First", "[", 
    RowBox[{"RSolve", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"G", "[", "n", "]"}], "==", 
       RowBox[{
        RowBox[{"2", "*", 
         RowBox[{"G", "[", 
          RowBox[{"n", "-", "1"}], "]"}]}], "-", 
        RowBox[{"6", "*", 
         RowBox[{"G", "[", 
          RowBox[{"n", "-", "2"}], "]"}]}]}]}], ",", "G", ",", "n"}], "]"}], 
    "]"}]}]}]}], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"Function", "[", 
  RowBox[{
   RowBox[{"{", "n", "}"}], ",", 
   RowBox[{
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"1", "-", 
        RowBox[{"\[ImaginaryI]", " ", 
         SqrtBox["5"]}]}], ")"}], "n"], " ", 
     RowBox[{"C", "[", "1", "]"}]}], "+", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"1", "+", 
        RowBox[{"\[ImaginaryI]", " ", 
         SqrtBox["5"]}]}], ")"}], "n"], " ", 
     RowBox[{"C", "[", "2", "]"}]}]}]}], "]"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The function ",
 StyleBox["G", "Input"],
 " is still able to compute values, but they will be in terms of the \
constants."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"G", "[", "5", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"1", "-", 
      RowBox[{"\[ImaginaryI]", " ", 
       SqrtBox["5"]}]}], ")"}], "5"], " ", 
   RowBox[{"C", "[", "1", "]"}]}], "+", 
  RowBox[{
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"1", "+", 
      RowBox[{"\[ImaginaryI]", " ", 
       SqrtBox["5"]}]}], ")"}], "5"], " ", 
   RowBox[{"C", "[", "2", "]"}]}]}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The capabilities of ",
 StyleBox[ButtonBox["RSolve",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/RSolve"], "FunctionLink"],
 ", like other ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " functions, are constantly being enhanced and extended. However, ",
 StyleBox[ButtonBox["RSolve",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/RSolve"], "FunctionLink"],
 " is not a panacea \[Dash]\[Dash] you can easily find recurrence relations \
that it is incapable of solving. When ",
 StyleBox[ButtonBox["RSolve",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/RSolve"], "FunctionLink"],
 " is unable to solve a recurrence relation, it simply returns unevaluated, \
as below."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"RSolve", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"u", "[", "n", "]"}], "\[Equal]", 
    RowBox[{
     RowBox[{
      RowBox[{"u", "[", 
       RowBox[{"n", "-", "1"}], "]"}], "^", "2"}], "-", 
     RowBox[{"Exp", "[", 
      RowBox[{"2", "*", 
       RowBox[{"e", "[", 
        RowBox[{"n", "-", "2"}], "]"}]}], "]"}]}]}], ",", "u", ",", "n"}], 
  "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"RSolve", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"u", "[", "n", "]"}], "\[Equal]", 
    RowBox[{
     RowBox[{"-", 
      SuperscriptBox["\[ExponentialE]", 
       RowBox[{"2", " ", 
        RowBox[{"e", "[", 
         RowBox[{
          RowBox[{"-", "2"}], "+", "n"}], "]"}]}]]}], "+", 
     SuperscriptBox[
      RowBox[{"u", "[", 
       RowBox[{
        RowBox[{"-", "1"}], "+", "n"}], "]"}], "2"]}]}], ",", "u", ",", "n"}],
   "]"}]], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
 "Problem Solving with ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " and Recurrence Relations"
}], "Subsection",
 CellTags->"Section2d"],

Cell[TextData[{
 "It is often the case that a problem, as presented, gives no clue that a \
solution may be found using recurrence. Let's see how we can use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to solve a problem that is not explicitly expressed as one requiring the \
use of recurrence for its solution."
}], "Text"],

Cell["\<\
Here is our problem: into how many regions is the plane divided by 1000 \
lines, assuming that no two of the lines are parallel, and no three are \
coincident? Such a situation may arise in an attempt to model fissures in the \
ocean floor. \
\>", "Text"],

Cell[TextData[{
 "To start, we might try to discover the answer for smaller numbers of lines. \
To generalize the problem, we may ask for the number of regions produced by \
",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " lines, where ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " is some positive integer. It is fairly obvious that a single line \
(corresponding to the case ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "=", "1"}], TraditionalForm]]],
 ") divides the plane into 2 regions. Two lines, if they are not parallel, \
can easily be seen to divide the plane into 4 regions.",
 " ",
 "(Two parallel lines produce only 3 regions.) If we call the number of \
regions produced by ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " lines, no two of which are parallel and no three of which are coincident, ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["R", "n"], TraditionalForm]]],
 ", then ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["R", "1"], "=", "2"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["R", "2"], "=", "4"}], TraditionalForm]]],
 ". "
}], "Text"],

Cell[BoxData[
 GraphicsBox[{
   {StrokeForm[Hue[0, 0, 0]], EdgeForm[GrayLevel[0]], EdgeForm[None], 
    LineBox[{{0.21250000000000005`, 0.9597222222222224}, {1.1569444444444448`,
      0.03194444444444433}}]}, 
   {StrokeForm[Hue[0, 0, 0]], EdgeForm[GrayLevel[0]], EdgeForm[None], 
    LineBox[{{1.1125000000000003`, 0.9458333333333335}, {0.04027777777777786, 
     0.1680555555555554}}]}, 
   {StrokeForm[Hue[0, 0, 0]], EdgeForm[GrayLevel[0]], EdgeForm[None], 
    LineBox[{{0.6847222222222223, 0.9513888888888891}, {0.9236111111111113, 
     0.04583333333333317}}]}},
  ContentSelectable->False,
  ImagePadding->{{0., 0.}, {0., 0.}},
  ImageSize->{327.096, 243.936},
  PlotLabel->"Figure 8.1: Three lines dividing the plane",
  PlotRange->{{0., 1.3333333333333335`}, {0., 1.}},
  PlotRangePadding->Automatic]], "Text",
 TextAlignment->Center],

Cell[TextData[{
 "What does the situation look like when ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "=", "3"}], TraditionalForm]]],
 "? Figure 8.1 is representative of this situation. In this case, the number \
of regions is 7, so ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["R", "3"], "=", "7"}], TraditionalForm]]],
 ". To find ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["R", "4"], TraditionalForm]]],
 " we must add a fourth line to the diagram. This suggests trying to compute ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["R", "4"], TraditionalForm]]],
 " in terms of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["R", "3"], TraditionalForm]]],
 " so that we begin to think of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["R", "n"], TraditionalForm]]],
 " as a recurrence relation. Figure 8.2 shows what the situation looks like \
when a fourth line is added to the three existing lines. From the assumptions \
that no two of the lines are parallel and no three pass through a single \
point, it follows that the new line must intersect each of the existing three \
lines in exactly one point. This means that the new line passes through \
exactly four of the regions formed by the original three lines. Each region \
that it passes through is divided into two regions, so the total number of \
new regions added by the fourth point is 4. Thus, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["R", "4"], "=", 
    RowBox[{
     SubscriptBox["R", "3"], "+", "4"}]}], TraditionalForm]]],
 ". Similar arguments for a general configuration of lines reveals that ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["R", "n"], TraditionalForm]]],
 " satisfies the recurrence relation ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["R", "n"], "=", 
    RowBox[{
     SubscriptBox["R", 
      RowBox[{"n", "-", "1"}]], "+", "n"}]}], TraditionalForm]]],
 "."
}], "Text"],

Cell[BoxData[
 GraphicsBox[{
   {StrokeForm[Hue[0, 0, 0]], EdgeForm[GrayLevel[0.]], EdgeForm[None], 
    LineBox[{{0.21250000000000005`, 0.9597222222222224}, {1.1569444444444448`,
      0.03194444444444433}}]}, 
   {StrokeForm[Hue[0, 0, 0]], EdgeForm[GrayLevel[0.]], EdgeForm[None], 
    LineBox[{{1.1125000000000003`, 0.9458333333333335}, {0.04027777777777786, 
     0.1680555555555554}}]}, 
   {StrokeForm[Hue[0, 0, 0]], EdgeForm[GrayLevel[0.]], EdgeForm[None], 
    LineBox[{{0.6847222222222223, 0.9513888888888891}, {0.9236111111111113, 
     0.04583333333333317}}]}, 
   {Dashing[{Medium, Medium}], StrokeForm[Hue[0, 0, 0]], EdgeForm[GrayLevel[
    0]], EdgeForm[None], 
    LineBox[{{0.04166666666666674, 0.43534482758620685`}, {
     1.3261494252873567`, 0.4497126436781609}}]}},
  ContentSelectable->False,
  ImagePadding->{{0., 0.}, {0., 0.}},
  ImageSize->{327.096, 243.936},
  PlotLabel->"Figure 8.2: Four lines dividing the plane",
  PlotRange->{{0., 1.3333333333333335`}, {0., 1.}},
  PlotRangePadding->Automatic]], "Text",
 TextAlignment->Center],

Cell[TextData[{
 "Furthermore, we have already computed the initial condition ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["R", "1"], "=", "2"}], TraditionalForm]]],
 ". This is enough to solve the recurrence. "
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"Clear", "[", "R", "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"R", "=", 
  RowBox[{"R", "/.", 
   RowBox[{"First", "[", 
    RowBox[{"RSolve", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"R", "[", "n", "]"}], "\[Equal]", 
         RowBox[{
          RowBox[{"R", "[", 
           RowBox[{"n", "-", "1"}], "]"}], "+", "n"}]}], ",", 
        RowBox[{
         RowBox[{"R", "[", "1", "]"}], "\[Equal]", "2"}]}], "}"}], ",", "R", 
      ",", "n"}], "]"}], "]"}]}]}]}], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"Function", "[", 
  RowBox[{
   RowBox[{"{", "n", "}"}], ",", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{"(", 
     RowBox[{"2", "+", "n", "+", 
      SuperscriptBox["n", "2"]}], ")"}]}]}], "]"}]], "Output"]
}, Open  ]],

Cell["\<\
To answer the question: how many regions is the plane divided by 1000 lines \
with no two parallel and no three coincident?\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"R", "[", "1000", "]"}]], "Input"],

Cell[BoxData["500501"], "Output"]
}, Open  ]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["8.3 Divide-and-Conquer Algorithms and Recurrence Relations", "Section",
 CellTags->"Section3"],

Cell["\<\
A very good example of divide and conquer relations is the one provided by \
the binary search algorithm. Here, we consider a practical application of \
this algorithm in an implementation of a binary search on a sorted list of \
integers. This is an implementation of the algorithm described in Algorithm 3 \
in Section 3.1 of the text and first presented in Section 3.1 of this manual. \
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"binarySearch", "[", 
   RowBox[{"x_Integer", ",", 
    RowBox[{"A", ":", 
     RowBox[{"{", "__Integer", "}"}]}]}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"n", ",", "i", ",", "j", ",", "m", ",", "location"}], "}"}], ",",
     "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"n", "=", 
      RowBox[{"Length", "[", "A", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"i", "=", "1"}], ";", "\[IndentingNewLine]", 
     RowBox[{"j", "=", "n"}], ";", "\[IndentingNewLine]", 
     RowBox[{"While", "[", 
      RowBox[{
       RowBox[{"i", "<", "j"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"m", "=", 
         RowBox[{"Floor", "[", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{"i", "+", "j"}], ")"}], "/", "2"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"x", ">", 
           RowBox[{"A", "[", 
            RowBox[{"[", "m", "]"}], "]"}]}], ",", "\[IndentingNewLine]", 
          RowBox[{"i", "=", 
           RowBox[{"m", "+", "1"}]}], ",", "\[IndentingNewLine]", 
          RowBox[{"j", "=", "m"}]}], "\[IndentingNewLine]", "]"}]}]}], 
      "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"x", "\[Equal]", 
        RowBox[{"A", "[", 
         RowBox[{"[", "i", "]"}], "]"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{"location", "=", "i"}], ",", "\[IndentingNewLine]", 
       RowBox[{"location", "=", "0"}]}], "\[IndentingNewLine]", "]"}], ";", 
     "\[IndentingNewLine]", "location"}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "The variable ",
 StyleBox["A", "Input"],
 " is the list of integers to search, which is assumed to be sorted in \
increasing order, and ",
 StyleBox["x", "Input"],
 " is the integer to search for. The local variables ",
 StyleBox["j", "Input"],
 " and ",
 StyleBox["i", "Input"],
 " are initialized to the number of elements in the list and 1, respectively. \
The ",
 StyleBox[ButtonBox["While",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/While"], "FunctionLink"],
 " loop continues as long as ",
 StyleBox["i", "Input"],
 " and ",
 StyleBox["j", "Input"],
 " are different from each other. Each step of the loop serves to narrow the \
difference between them by calculating the middle of the list, represented by \
",
 StyleBox["m", "Input"],
 ", and determining which half ",
 StyleBox["x", "Input"],
 " is in. Eventually, the search will focus in on one location in the list, \
which is either ",
 StyleBox["x", "Input"],
 " or, if not, the search has failed and the function returns 0."
}], "Text"],

Cell["\<\
Let us do the analysis of the algorithm to see how divide and conquer \
recurrence relations are generated. In general, a divide and conquer type \
recurrence relation has the form\
\>", "Text"],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"f", "(", "n", ")"}], "=", 
   RowBox[{
    RowBox[{"a", "\[CenterDot]", 
     RowBox[{"f", "(", 
      RowBox[{"n", "/", "b"}], ")"}]}], "+", 
    RowBox[{"g", "(", "n", ")"}]}]}], TraditionalForm]], "Equation"],

Cell[TextData[{
 "Each iteration of the ",
 StyleBox[ButtonBox["While",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/While"], "FunctionLink"],
 " loop of ",
 StyleBox["binarySearch", "Input"],
 " produces a single list half the size of the original. So ",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", "=", "1"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"b", "=", "2"}], TraditionalForm]]],
 ". The function ",
 Cell[BoxData[
  FormBox[
   RowBox[{"g", "(", "n", ")"}], TraditionalForm]]],
 ", which measures the comparisons added in implementing the reduction, is \
identically 2. This is because one comparison is added to see which half of \
the list the key is on, and one is added to see if the ",
 StyleBox[ButtonBox["While",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/While"], "FunctionLink"],
 " loop needs to continue. So, for the ",
 StyleBox["binarySearch", "Input"],
 " algorithm, the recurrence relation is"
}], "Text"],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"f", "(", "n", ")"}], "=", 
   RowBox[{
    RowBox[{"f", "(", 
     RowBox[{"n", "/", "2"}], ")"}], "+", "2"}]}], 
  TraditionalForm]], "Equation"],

Cell[TextData[{
 "Additionally, we can see that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"f", "(", "1", ")"}], "=", "2"}], TraditionalForm]]],
 ", because if the list is of length 1, then the algorithm will do one \
comparison to determine that the ",
 StyleBox[ButtonBox["While",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/While"], "FunctionLink"],
 " loop is unnecessary and one comparison to make sure that the element being \
searched for is the one element in the list. We can now use ",
 StyleBox[ButtonBox["RSolve",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/RSolve"], "FunctionLink"],
 " to solve this recurrence."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"RSolve", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"b", "[", "n", "]"}], "\[Equal]", 
      RowBox[{
       RowBox[{"b", "[", 
        RowBox[{"n", "/", "2"}], "]"}], "+", "2"}]}], ",", 
     RowBox[{
      RowBox[{"b", "[", "1", "]"}], "\[Equal]", "2"}]}], "}"}], ",", 
   RowBox[{"b", "[", "n", "]"}], ",", "n"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"b", "[", "n", "]"}], "\[Rule]", 
    RowBox[{"2", " ", 
     RowBox[{"(", 
      RowBox[{"1", "+", 
       FractionBox[
        RowBox[{"Log", "[", "n", "]"}], 
        RowBox[{"Log", "[", "2", "]"}]]}], ")"}]}]}], "}"}], "}"}]], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["8.4 Generating Functions", "Section",
 CellTags->"Section4"],

Cell[TextData[{
 "Generating functions are a powerful tool for manipulating sequences of \
numbers and for solving a variety of counting problems. In this section, we \
will see how ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " can be used to represent and manipulate generating functions."
}], "Text"],

Cell[TextData[{
 "The generating function ",
 Cell[BoxData[
  FormBox[
   RowBox[{"G", "(", "x", ")"}], TraditionalForm]]],
 " for a sequence ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    SubscriptBox["a", "k"], "}"}], TraditionalForm]]],
 " is the formal power series"
}], "Text"],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    UnderoverscriptBox["\[Sum]", 
     RowBox[{"k", "=", "0"}], "\[Infinity]",
     LimitsPositioning->False], " ", 
    RowBox[{
     SubscriptBox["a", "k"], 
     SuperscriptBox["x", "k"]}]}], "=", 
   RowBox[{
    SubscriptBox["a", "0"], "+", 
    RowBox[{
     SubscriptBox["a", "1"], "x"}], "+", 
    RowBox[{
     SubscriptBox["a", "2"], 
     SuperscriptBox["x", "2"]}], "+", 
    RowBox[{
     SubscriptBox["a", "3"], 
     SuperscriptBox["x", "3"]}], "+", "\[CenterEllipsis]", "+", 
    RowBox[{
     SubscriptBox["a", "k"], 
     SuperscriptBox["x", "k"]}], "+", "\[CenterEllipsis]"}]}], 
  TraditionalForm]], "Equation"],

Cell[TextData[{
 "It is called ",
 StyleBox["formal",
  FontSlant->"Italic"],
 " because we are not interested in evaluating it as a function of ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 ". Our focus is on finding a formula for its coefficients. In particular, \
this means that there are no convergence issues to be considered."
}], "Text"],

Cell[CellGroupData[{

Cell["Generating Functions Tools", "Subsection",
 CellTags->"Section4a"],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " provides extensive facilities for manipulating generating functions."
}], "Text"],

Cell[TextData[{
 "The first thing we need to do is to learn how to create a power series with \
",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", which is done with the function ",
 StyleBox[ButtonBox["GeneratingFunction",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/GeneratingFunction"], "FunctionLink"],
 ". This function requires three arguments. The last argument is the \
variable, such as ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 ", that the generating function will be written in terms of. The second \
argument is a variable, such as ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ", representing the index of summation. The first argument is an expression \
in terms of the index of summation that computes the coefficients of the \
series."
}], "Text"],

Cell[TextData[{
 "For example, to create the generating function for the sequence ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    SuperscriptBox["3", "k"], "}"}], TraditionalForm]]],
 ", we use the following code."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GeneratingFunction", "[", 
  RowBox[{
   RowBox[{"3", "^", "k"}], ",", "k", ",", "x"}], "]"}]], "Input"],

Cell[BoxData[
 FractionBox["1", 
  RowBox[{"1", "-", 
   RowBox[{"3", " ", "x"}]}]]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Observe that ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " has automatically found a rational expression for the generating function. \
While this is a useful and exact representation of the generating function, \
you often want to be able to view the generating function as a power series. \
"
}], "Text"],

Cell[TextData[{
 "To obtain a series representation of the generating function, we use the ",
 StyleBox[ButtonBox["Series",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Series"], "FunctionLink"],
 " function. ",
 StyleBox[ButtonBox["Series",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Series"], "FunctionLink"],
 " is used to find the Taylor series of a function about a point. If you have \
taken Calculus, you may remember Taylor series. If not, it is enough to know \
that the Taylor series of a generating function around ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "=", "0"}], TraditionalForm]]],
 " is the correct power series for our purposes."
}], "Text"],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["Series",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Series"], "FunctionLink"],
 " function requires two arguments. The first is the function, e.g. ",
 Cell[BoxData[
  FormBox[
   FractionBox["1", 
    RowBox[{"1", "-", 
     RowBox[{"3", "x"}]}]], TraditionalForm]]],
 ". The second is a list of length three: the variable ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 ", the number 0, and the largest order, or exponent, to be displayed. For \
example to display the first ten terms of the power series for ",
 Cell[BoxData[
  FormBox[
   FractionBox["1", 
    RowBox[{"1", "-", 
     RowBox[{"3", "x"}]}]], TraditionalForm]]],
 ", we enter the following."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"seriesEx", "=", 
  RowBox[{"Series", "[", 
   RowBox[{
    RowBox[{"1", "/", 
     RowBox[{"(", 
      RowBox[{"1", "-", 
       RowBox[{"3", "x"}]}], ")"}]}], ",", 
    RowBox[{"{", 
     RowBox[{"x", ",", "0", ",", "10"}], "}"}]}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"1", "+", 
   RowBox[{"3", " ", "x"}], "+", 
   RowBox[{"9", " ", 
    SuperscriptBox["x", "2"]}], "+", 
   RowBox[{"27", " ", 
    SuperscriptBox["x", "3"]}], "+", 
   RowBox[{"81", " ", 
    SuperscriptBox["x", "4"]}], "+", 
   RowBox[{"243", " ", 
    SuperscriptBox["x", "5"]}], "+", 
   RowBox[{"729", " ", 
    SuperscriptBox["x", "6"]}], "+", 
   RowBox[{"2187", " ", 
    SuperscriptBox["x", "7"]}], "+", 
   RowBox[{"6561", " ", 
    SuperscriptBox["x", "8"]}], "+", 
   RowBox[{"19683", " ", 
    SuperscriptBox["x", "9"]}], "+", 
   RowBox[{"59049", " ", 
    SuperscriptBox["x", "10"]}], "+", 
   InterpretationBox[
    SuperscriptBox[
     RowBox[{"O", "[", "x", "]"}], "11"],
    SeriesData[$CellContext`x, 0, {}, 0, 11, 1],
    Editable->False]}],
  SeriesData[$CellContext`x, 0, {1, 3, 9, 27, 81, 243, 729, 2187, 6561, 19683,
    59049}, 0, 11, 1],
  Editable->False]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The output ends with ",
 Cell[BoxData[
  StyleBox[
   SuperscriptBox[
    RowBox[{"O", "[", "x", "]"}], "11"], "Input"]],
  CellChangeTimes->{{3.5584452767270527`*^9, 3.558445289145015*^9}}],
 ", which indicates that the series continues with terms of degree 11 and \
higher."
}], "Text"],

Cell[TextData[{
 "If you wish to view the coefficients, without the generating function, you \
can apply ",
 StyleBox[ButtonBox["CoefficientList",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/CoefficientList"], "FunctionLink"],
 ", as shown below. The first argument is the series and the second is the \
variable."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CoefficientList", "[", 
  RowBox[{"seriesEx", ",", "x"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "1", ",", "3", ",", "9", ",", "27", ",", "81", ",", "243", ",", "729", ",", 
   "2187", ",", "6561", ",", "19683", ",", "59049"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "To create a power series from a recurrence relation, we need to combine the \
techniques of the last section for using ",
 StyleBox[ButtonBox["RSolve",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/RSolve"], "FunctionLink"],
 " to solve a recurrence relation with the ",
 StyleBox[ButtonBox["GeneratingFunction",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/GeneratingFunction"], "FunctionLink"],
 " command. For example, consider the recurrence relation given by ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["a", "n"], "=", 
    RowBox[{
     RowBox[{"2", 
      SubscriptBox["a", 
       RowBox[{"n", "-", "1"}]]}], "-", 
     SubscriptBox["a", 
      RowBox[{"n", "-", "2"}]], "+", "1"}]}], TraditionalForm]]],
 " with initial conditions ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["a", "0"], "=", "1"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["a", "1"], "=", "2"}], TraditionalForm]]],
 "."
}], "Text"],

Cell[TextData[{
 "First, we apply the ",
 StyleBox[ButtonBox["RSolve",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/RSolve"], "FunctionLink"],
 " function and extract an expression for the general term of the sequence. \
Note that because we are interested in an expression that we can use in ",
 StyleBox[ButtonBox["GeneratingFunction",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/GeneratingFunction"], "FunctionLink"],
 ", it makes sense to solve for ",
 StyleBox["a[n]", "Input"],
 " rather than solving for ",
 StyleBox["a", "Input"],
 " and obtaining a pure ",
 StyleBox[ButtonBox["Function",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Function"], "FunctionLink"],
 ". Either approach would work, however."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"recurrenceFormula", "=", 
  RowBox[{
   RowBox[{"a", "[", "n", "]"}], "/.", 
   RowBox[{"First", "[", "\[IndentingNewLine]", 
    RowBox[{"RSolve", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"a", "[", "n", "]"}], "\[Equal]", 
         RowBox[{
          RowBox[{"2", "*", 
           RowBox[{"a", "[", 
            RowBox[{"n", "-", "1"}], "]"}]}], "-", 
          RowBox[{"a", "[", 
           RowBox[{"n", "-", "2"}], "]"}], "+", "1"}]}], ",", 
        RowBox[{
         RowBox[{"a", "[", "0", "]"}], "\[Equal]", "1"}], ",", 
        RowBox[{
         RowBox[{"a", "[", "1", "]"}], "\[Equal]", "2"}]}], "}"}], ",", 
      RowBox[{"a", "[", "n", "]"}], ",", "n"}], "]"}], "\[IndentingNewLine]", 
    "]"}]}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  FractionBox["1", "2"], " ", 
  RowBox[{"(", 
   RowBox[{"2", "+", "n", "+", 
    SuperscriptBox["n", "2"]}], ")"}]}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Now we apply ",
 StyleBox[ButtonBox["GeneratingFunction",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/GeneratingFunction"], "FunctionLink"],
 " to this formula."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"recurrenceGFunction", "=", 
  RowBox[{"GeneratingFunction", "[", 
   RowBox[{"recurrenceFormula", ",", "n", ",", "x"}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 FractionBox[
  RowBox[{
   RowBox[{"-", "1"}], "+", "x", "-", 
   SuperscriptBox["x", "2"]}], 
  SuperscriptBox[
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", "1"}], "+", "x"}], ")"}], "3"]]], "Output"]
}, Open  ]],

Cell[TextData[{
 "As above, we can use ",
 StyleBox[ButtonBox["Series",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Series"], "FunctionLink"],
 " to view this as a formal power series."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"recurrenceSeries", "=", 
  RowBox[{"Series", "[", 
   RowBox[{"recurrenceGFunction", ",", 
    RowBox[{"{", 
     RowBox[{"x", ",", "0", ",", "5"}], "}"}]}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"1", "+", 
   RowBox[{"2", " ", "x"}], "+", 
   RowBox[{"4", " ", 
    SuperscriptBox["x", "2"]}], "+", 
   RowBox[{"7", " ", 
    SuperscriptBox["x", "3"]}], "+", 
   RowBox[{"11", " ", 
    SuperscriptBox["x", "4"]}], "+", 
   RowBox[{"16", " ", 
    SuperscriptBox["x", "5"]}], "+", 
   InterpretationBox[
    SuperscriptBox[
     RowBox[{"O", "[", "x", "]"}], "6"],
    SeriesData[$CellContext`x, 0, {}, 0, 6, 1],
    Editable->False]}],
  SeriesData[$CellContext`x, 0, {1, 2, 4, 7, 11, 16}, 0, 6, 1],
  Editable->False]], "Output"]
}, Open  ]],

Cell[TextData[{
 "We can also use the function ",
 StyleBox[ButtonBox["SeriesCoefficient",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/SeriesCoefficient"], "FunctionLink"],
 " to obtain specific coefficients, rather than the entire series. This \
function has two distinct forms that are useful. First, you can give the \
output of ",
 StyleBox[ButtonBox["Series",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Series"], "FunctionLink"],
 " as the first argument and an integer as the second, and ",
 StyleBox[ButtonBox["SeriesCoefficient",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/SeriesCoefficient"], "FunctionLink"],
 " will return the coefficient of that term. Note that for this to work, you \
must have already computed the series to the desired term. That is, the \
second argument of ",
 StyleBox[ButtonBox["SeriesCoefficient",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/SeriesCoefficient"], "FunctionLink"],
 " must be no greater than the order of the output from ",
 StyleBox[ButtonBox["Series",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Series"], "FunctionLink"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SeriesCoefficient", "[", 
  RowBox[{"recurrenceSeries", ",", "3"}], "]"}]], "Input"],

Cell[BoxData["7"], "Output"]
}, Open  ]],

Cell[TextData[{
 "The second, and even more useful, application of ",
 StyleBox[ButtonBox["SeriesCoefficient",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/SeriesCoefficient"], "FunctionLink"],
 " has the same arguments as ",
 StyleBox[ButtonBox["Series",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Series"], "FunctionLink"],
 ". Specifically, a generating function and a list consisting of the variable \
used in the function, the number 0, and a positive integer ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ". However, where ",
 StyleBox[ButtonBox["Series",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Series"], "FunctionLink"],
 " outputs the entire series out to the term of degree ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ", ",
 StyleBox[ButtonBox["SeriesCoefficient",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/SeriesCoefficient"], "FunctionLink"],
 " just returns the coefficient of ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["x", "n"], TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SeriesCoefficient", "[", 
  RowBox[{"recurrenceGFunction", ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "3"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData["7"], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Solving Problems with Generating Functions", "Subsection",
 CellTags->"Section4b"],

Cell[TextData[{
 "Generating functions are more than just a convenient way to represent \
numerical sequences. They are a powerful tool for solving recurrence \
relations, as well as other kinds of counting problems. This power stems from \
our ability to manipulate them like ordinary power series from Calculus and \
to interpret those manipulations. To illustrate ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "'s facilities for manipulating generating functions, consider Example 12 \
from Section 8.4 of the text:"
}], "Text"],

Cell[TextData[{
 "Use generating functions to determine the number of ways to insert tokens \
worth $1, $2, and $5 into a vending machine to pay for an item that costs ",
 Cell[BoxData[
  FormBox["7", TraditionalForm]]],
 " dollars in both the cases when the order in which the tokens are inserted \
does not matter and when the order does matter. "
}], "Item1Paragraph"],

Cell[TextData[{
 "Following the text, the solution to the problem when order does matter is \
the coefficient of ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["x", "r"], TraditionalForm]]],
 " in the generating function"
}], "Text"],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{"1", "+", "x", "+", 
     SuperscriptBox["x", "2"], "+", 
     SuperscriptBox["x", "3"], "+", "\[CenterEllipsis]"}], ")"}], 
   RowBox[{"(", 
    RowBox[{"1", "+", 
     SuperscriptBox["x", "2"], "+", 
     SuperscriptBox["x", "4"], "+", 
     SuperscriptBox["x", "6"], "+", "\[CenterEllipsis]"}], ")"}], 
   RowBox[{"(", 
    RowBox[{"1", "+", 
     SuperscriptBox["x", "5"], "+", 
     SuperscriptBox["x", "10"], "+", 
     SuperscriptBox["x", "15"], "+", "\[CenterEllipsis]"}], ")"}]}], 
  TraditionalForm]], "Equation"],

Cell[TextData[{
 "To solve the problem, we need to create the three power series and multiply \
them together. To create the first series, we can use the ",
 StyleBox[ButtonBox["GeneratingFunction",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/GeneratingFunction"], "FunctionLink"],
 " command demonstrated above. For example, the first series has every \
coefficient equal to 1, so its generating function can be found as shown \
below."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"token1D", "=", 
  RowBox[{"GeneratingFunction", "[", 
   RowBox[{"1", ",", "n", ",", "x"}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 FractionBox["1", 
  RowBox[{"1", "-", "x"}]]], "Output"]
}, Open  ]],

Cell[TextData[{
 "For the $2 tokens, we need to represent ",
 Cell[BoxData[
  FormBox[
   RowBox[{"1", "+", 
    SuperscriptBox["x", "2"], "+", 
    SuperscriptBox["x", "4"], "+", "\[CenterEllipsis]"}], TraditionalForm]]],
 ". Remember that ",
 StyleBox[ButtonBox["GeneratingFunction",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/GeneratingFunction"], "FunctionLink"],
 " requires an expression for the general coefficient as its first argument. \
In this case, it is easier to describe the series if we can include the \
entire term, not just the coefficients. That is, it is easier to say that the \
series is ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    UnderoverscriptBox["\[Sum]", 
     StyleBox[
      RowBox[{"k", "=", "0"}], "TI"], "\[Infinity]",
     LimitsPositioning->False], " ", 
    SuperscriptBox["x", 
     RowBox[{"2", "k"}]]}], TraditionalForm]]],
 " than to write a formula for the coefficients. "
}], "Text"],

Cell[TextData[{
 "Fortunately, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " allows us to do exactly that by applying the ",
 StyleBox[ButtonBox["Sum",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Sum"], "FunctionLink"],
 " function. Recall that the first argument to ",
 StyleBox[ButtonBox["Sum",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Sum"], "FunctionLink"],
 " is an expression for the generic term in terms of an index of summation, \
and the second argument defines the bounds of the summation. In this case, \
those bounds are 0 and ",
 StyleBox[ButtonBox["Infinity",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Infinity"], "FunctionLink"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"token2D", "=", 
  RowBox[{"Sum", "[", 
   RowBox[{
    RowBox[{"x", "^", 
     RowBox[{"(", 
      RowBox[{"2", "*", "k"}], ")"}]}], ",", 
    RowBox[{"{", 
     RowBox[{"k", ",", "0", ",", "Infinity"}], "}"}]}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 FractionBox["1", 
  RowBox[{"1", "-", 
   SuperscriptBox["x", "2"]}]]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Observe that ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " has automatically calculated the rational expression for this sum."
}], "Text"],

Cell[TextData[{
 "We could also use ",
 StyleBox[ButtonBox["Sum",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Sum"], "FunctionLink"],
 " for the $5 tokens. Or we could just refer to Table 1 in Section 8.4 of the \
text and  enter the rational form ",
 Cell[BoxData[
  FormBox[
   FractionBox["1", 
    RowBox[{"1", "-", 
     SuperscriptBox["x", "r"]}]], TraditionalForm]]],
 " directly. "
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"token5D", "=", 
  RowBox[{"1", "/", 
   RowBox[{"(", 
    RowBox[{"1", "-", 
     RowBox[{"x", "^", "5"}]}], ")"}]}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 FractionBox["1", 
  RowBox[{"1", "-", 
   SuperscriptBox["x", "5"]}]]], "Output"]
}, Open  ]],

Cell[TextData[{
 "We can use ",
 StyleBox[ButtonBox["Series",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Series"], "FunctionLink"],
 " to confirm that ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " does in fact recognize this as the generating function ",
 Cell[BoxData[
  FormBox[
   RowBox[{"1", "+", 
    SuperscriptBox["x", "5"], "+", 
    SuperscriptBox["x", "10"], "+", 
    SuperscriptBox["x", "15"], "+", "\[CenterEllipsis]"}], 
   TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Series", "[", 
  RowBox[{"token5D", ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "20"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"1", "+", 
   SuperscriptBox["x", "5"], "+", 
   SuperscriptBox["x", "10"], "+", 
   SuperscriptBox["x", "15"], "+", 
   SuperscriptBox["x", "20"], "+", 
   InterpretationBox[
    SuperscriptBox[
     RowBox[{"O", "[", "x", "]"}], "21"],
    SeriesData[$CellContext`x, 0, {}, 0, 21, 1],
    Editable->False]}],
  SeriesData[$CellContext`x, 0, {1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 
   1, 0, 0, 0, 0, 1}, 0, 21, 1],
  Editable->False]], "Output"]
}, Open  ]],

Cell[TextData[{
 "We can algebraically combine the series with the usual multiplication and \
",
 StyleBox[ButtonBox["Series",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Series"], "FunctionLink"],
 " will show us the power series expansion of the result."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"tokens", "=", 
  RowBox[{"token1D", "*", "token2D", "*", "token5D"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 FractionBox["1", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"1", "-", "x"}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{"1", "-", 
     SuperscriptBox["x", "2"]}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{"1", "-", 
     SuperscriptBox["x", "5"]}], ")"}]}]]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Series", "[", 
  RowBox[{"tokens", ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "7"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"1", "+", "x", "+", 
   RowBox[{"2", " ", 
    SuperscriptBox["x", "2"]}], "+", 
   RowBox[{"2", " ", 
    SuperscriptBox["x", "3"]}], "+", 
   RowBox[{"3", " ", 
    SuperscriptBox["x", "4"]}], "+", 
   RowBox[{"4", " ", 
    SuperscriptBox["x", "5"]}], "+", 
   RowBox[{"5", " ", 
    SuperscriptBox["x", "6"]}], "+", 
   RowBox[{"6", " ", 
    SuperscriptBox["x", "7"]}], "+", 
   InterpretationBox[
    SuperscriptBox[
     RowBox[{"O", "[", "x", "]"}], "8"],
    SeriesData[$CellContext`x, 0, {}, 0, 8, 1],
    Editable->False]}],
  SeriesData[$CellContext`x, 0, {1, 1, 2, 2, 3, 4, 5, 6}, 0, 8, 1],
  Editable->False]], "Output"]
}, Open  ]],

Cell[TextData[{
 "We can see from the above that there are 6 ways to pay for a $7 item (since \
the coefficient of ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["x", "7"], TraditionalForm]]],
 " is 6), just as was computed in the text. If we wanted to know the number \
of ways to pay for an item costing $234, all we would need to do is find the \
coefficient of ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["x", "234"], TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SeriesCoefficient", "[", 
  RowBox[{"tokens", ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "234"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData["2832"], "Output"]
}, Open  ]],

Cell["\<\
For the second part of the question, the case where the order does matter, \
the text explains that the generating function we need is\
\>", "Text"],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"1", "+", 
    RowBox[{"(", 
     RowBox[{"x", "+", 
      SuperscriptBox["x", "2"], "+", 
      SuperscriptBox["x", "5"]}], ")"}], "+", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"x", "+", 
       SuperscriptBox["x", "2"], "+", 
       SuperscriptBox["x", "5"]}], ")"}], "2"], "+", "\[CenterEllipsis]"}], 
   "=", 
   FractionBox["1", 
    RowBox[{"1", "-", 
     RowBox[{"(", 
      RowBox[{"x", "+", 
       SuperscriptBox["x", "2"], "+", 
       SuperscriptBox["x", "5"]}], ")"}]}]]}], TraditionalForm]], "Equation"],

Cell[TextData[{
 "Again, if we did not already know the rational expression for the \
generating function, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " could find it using ",
 StyleBox[ButtonBox["Sum",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Sum"], "FunctionLink"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"tokens2", "=", 
  RowBox[{"Sum", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"x", "+", 
       RowBox[{"x", "^", "2"}], "+", 
       RowBox[{"x", "^", "5"}]}], ")"}], "^", "n"}], ",", 
    RowBox[{"{", 
     RowBox[{"n", ",", "0", ",", "Infinity"}], "}"}]}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 FractionBox["1", 
  RowBox[{"1", "-", "x", "-", 
   SuperscriptBox["x", "2"], "-", 
   SuperscriptBox["x", "5"]}]]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SeriesCoefficient", "[", 
  RowBox[{"tokens2", ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "7"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData["26"], "Output"]
}, Open  ]],

Cell[TextData[{
 "Thus the coefficient of ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["x", "7"], TraditionalForm]]],
 " is 26, so there are 26 ways to pay for a $7 item when order does matter."
}], "Text"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["8.5 Inclusion-Exclusion", "Section",
 CellTags->"Section5"],

Cell[TextData[{
 "In this section we will apply the principle of inclusion and exclusion. We \
will see how to use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to solve problems with this technique."
}], "Text"],

Cell["\<\
At the heart of the principle of inclusion and exclusion is the formula\
\>", "Text"],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"|", 
    RowBox[{"A", "\[Union]", "B"}], "|"}], "=", 
   RowBox[{"|", "A", "|", 
    RowBox[{"+", 
     RowBox[{"|", "B", "|", 
      RowBox[{"-", 
       RowBox[{"|", 
        RowBox[{"A", "\[Intersection]", "B"}], "|"}]}]}]}]}]}], 
  TraditionalForm]], "Equation"],

Cell[TextData[{
 "which says that, for two finite sets ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 ", the number of elements in the union ",
 Cell[BoxData[
  FormBox[
   RowBox[{"A", "\[Union]", "B"}], TraditionalForm]]],
 " of the two sets may be found by adding the sizes of ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 " and then subtracting the number of elements common to both ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]]],
 ", which would otherwise be counted twice. This formula can be generalized \
to count the number of elements in the union of any finite number of finite \
sets."
}], "Text"],

Cell[TextData[{
 "Recall that in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", sets are represented as lists, but that the set operations put elements \
in a canonical order and remove duplicates. "
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"A", "=", 
  RowBox[{"{", 
   RowBox[{"1", ",", "2", ",", "3"}], "}"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "2", ",", "3"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "To find the cardinality of a set, you can use ",
 StyleBox[ButtonBox["Length",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Length"], "FunctionLink"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Length", "[", "A", "]"}]], "Input"],

Cell[BoxData["3"], "Output"]
}, Open  ]],

Cell[TextData[{
 "The set operations ",
 StyleBox[ButtonBox["Union",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Union"], "FunctionLink"],
 " and ",
 StyleBox[ButtonBox["Intersection",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Intersection"], "FunctionLink"],
 " can be applied in functional form, with each set an argument. You can also \
access operator forms with the aliases \[EscapeKey]un\[EscapeKey] and \
\[EscapeKey]inter\[EscapeKey]. "
}], "Text"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"X", "=", 
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "3", ",", "4", ",", "5"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Y", "=", 
   RowBox[{"{", 
    RowBox[{"4", ",", "5", ",", "6", ",", "7", ",", "8"}], "}"}]}], 
  ";"}]}], "Input",
 InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Union", "[", 
  RowBox[{"X", ",", "Y"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "1", ",", "2", ",", "3", ",", "4", ",", "5", ",", "6", ",", "7", ",", "8"}],
   "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"X", "\[Union]", "Y"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "1", ",", "2", ",", "3", ",", "4", ",", "5", ",", "6", ",", "7", ",", "8"}],
   "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Intersection", "[", 
  RowBox[{"X", ",", "Y"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"4", ",", "5"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Also, the set theoretic difference is computed by the ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " function ",
 StyleBox[ButtonBox["Complement",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Complement"], "FunctionLink"],
 ". The following computes ",
 Cell[BoxData[
  FormBox[
   RowBox[{"X", "-", "Y"}], TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Complement", "[", 
  RowBox[{"X", ",", "Y"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "2", ",", "3"}], "}"}]], "Output"]
}, Open  ]],

Cell["\<\
Let's use the operations to illustrate the principle of inclusion and \
exclusion with a particular example.\
\>", "Text"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"flintstones", "=", 
   RowBox[{"{", 
    RowBox[{"\"\<Fred\>\"", ",", "\"\<Wilma\>\"", ",", "\"\<Pebbles\>\""}], 
    "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"rubbles", "=", 
   RowBox[{"{", 
    RowBox[{"\"\<Barney\>\"", ",", "\"\<Betty\>\"", ",", "\"\<Bam Bam\>\""}], 
    "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"husbands", "=", 
   RowBox[{"{", 
    RowBox[{"\"\<Fred\>\"", ",", "\"\<Barney\>\""}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"wives", "=", 
   RowBox[{"{", 
    RowBox[{"\"\<Wilma\>\"", ",", "\"\<Betty\>\""}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"kids", "=", 
   RowBox[{"{", 
    RowBox[{"\"\<Pebbles\>\"", ",", "\"\<Bam Bam\>\""}], "}"}]}], 
  ";"}]}], "Input",
 InitializationCell->True],

Cell["\<\
If this were a complete census, then the number of children living in Bedrock \
would be\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Length", "[", "kids", "]"}]], "Input"],

Cell[BoxData["2"], "Output"]
}, Open  ]],

Cell["\<\
The number of Bedrock inhabitants who are either Flintstones or children is\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Length", "[", 
  RowBox[{"Union", "[", 
   RowBox[{"flintstones", ",", "kids"}], "]"}], "]"}]], "Input"],

Cell[BoxData["4"], "Output"]
}, Open  ]],

Cell["\<\
According to the principle of inclusion and exclusion, this number should be \
the same as \
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Length", "[", "flintstones", "]"}], "+", 
  RowBox[{"Length", "[", "kids", "]"}], "-", 
  RowBox[{"Length", "[", 
   RowBox[{"Intersection", "[", 
    RowBox[{"flintstones", ",", "kids"}], "]"}], "]"}]}]], "Input"],

Cell[BoxData["4"], "Output"]
}, Open  ]],

Cell["which, of course, it is.", "Text"],

Cell["\<\
As another example, consider the problem of determining the number of \
positive integers less than or equal to 100 that are not divisible by either \
2 or 11. First we generate the set of positive integers less than or equal to \
100. \
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"hundred", "=", 
  RowBox[{"Range", "[", "100", "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "1", ",", "2", ",", "3", ",", "4", ",", "5", ",", "6", ",", "7", ",", "8", 
   ",", "9", ",", "10", ",", "11", ",", "12", ",", "13", ",", "14", ",", "15",
    ",", "16", ",", "17", ",", "18", ",", "19", ",", "20", ",", "21", ",", 
   "22", ",", "23", ",", "24", ",", "25", ",", "26", ",", "27", ",", "28", 
   ",", "29", ",", "30", ",", "31", ",", "32", ",", "33", ",", "34", ",", 
   "35", ",", "36", ",", "37", ",", "38", ",", "39", ",", "40", ",", "41", 
   ",", "42", ",", "43", ",", "44", ",", "45", ",", "46", ",", "47", ",", 
   "48", ",", "49", ",", "50", ",", "51", ",", "52", ",", "53", ",", "54", 
   ",", "55", ",", "56", ",", "57", ",", "58", ",", "59", ",", "60", ",", 
   "61", ",", "62", ",", "63", ",", "64", ",", "65", ",", "66", ",", "67", 
   ",", "68", ",", "69", ",", "70", ",", "71", ",", "72", ",", "73", ",", 
   "74", ",", "75", ",", "76", ",", "77", ",", "78", ",", "79", ",", "80", 
   ",", "81", ",", "82", ",", "83", ",", "84", ",", "85", ",", "86", ",", 
   "87", ",", "88", ",", "89", ",", "90", ",", "91", ",", "92", ",", "93", 
   ",", "94", ",", "95", ",", "96", ",", "97", ",", "98", ",", "99", ",", 
   "100"}], "}"}]], "Output"]
}, Open  ]],

Cell["Next, we remove those elements that are divisible by 2: ", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"divBy2", "=", 
  RowBox[{"Complement", "[", 
   RowBox[{"hundred", ",", 
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{"2", "*", "i"}], ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", "50"}], "}"}]}], "]"}]}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "1", ",", "3", ",", "5", ",", "7", ",", "9", ",", "11", ",", "13", ",", 
   "15", ",", "17", ",", "19", ",", "21", ",", "23", ",", "25", ",", "27", 
   ",", "29", ",", "31", ",", "33", ",", "35", ",", "37", ",", "39", ",", 
   "41", ",", "43", ",", "45", ",", "47", ",", "49", ",", "51", ",", "53", 
   ",", "55", ",", "57", ",", "59", ",", "61", ",", "63", ",", "65", ",", 
   "67", ",", "69", ",", "71", ",", "73", ",", "75", ",", "77", ",", "79", 
   ",", "81", ",", "83", ",", "85", ",", "87", ",", "89", ",", "91", ",", 
   "93", ",", "95", ",", "97", ",", "99"}], "}"}]], "Output"]
}, Open  ]],

Cell["and those that are divisible by 11: ", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"divBy11", "=", 
  RowBox[{"Complement", "[", 
   RowBox[{"hundred", ",", 
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{"11", "*", "i"}], ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", "9"}], "}"}]}], "]"}]}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "1", ",", "2", ",", "3", ",", "4", ",", "5", ",", "6", ",", "7", ",", "8", 
   ",", "9", ",", "10", ",", "12", ",", "13", ",", "14", ",", "15", ",", "16",
    ",", "17", ",", "18", ",", "19", ",", "20", ",", "21", ",", "23", ",", 
   "24", ",", "25", ",", "26", ",", "27", ",", "28", ",", "29", ",", "30", 
   ",", "31", ",", "32", ",", "34", ",", "35", ",", "36", ",", "37", ",", 
   "38", ",", "39", ",", "40", ",", "41", ",", "42", ",", "43", ",", "45", 
   ",", "46", ",", "47", ",", "48", ",", "49", ",", "50", ",", "51", ",", 
   "52", ",", "53", ",", "54", ",", "56", ",", "57", ",", "58", ",", "59", 
   ",", "60", ",", "61", ",", "62", ",", "63", ",", "64", ",", "65", ",", 
   "67", ",", "68", ",", "69", ",", "70", ",", "71", ",", "72", ",", "73", 
   ",", "74", ",", "75", ",", "76", ",", "78", ",", "79", ",", "80", ",", 
   "81", ",", "82", ",", "83", ",", "84", ",", "85", ",", "86", ",", "87", 
   ",", "89", ",", "90", ",", "91", ",", "92", ",", "93", ",", "94", ",", 
   "95", ",", "96", ",", "97", ",", "98", ",", "100"}], "}"}]], "Output"]
}, Open  ]],

Cell["\<\
We are looking for integers that belong to either or both of those sets, that \
is, to their union, so we calculate:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Length", "[", 
  RowBox[{"Union", "[", 
   RowBox[{"divBy2", ",", "divBy11"}], "]"}], "]"}]], "Input"],

Cell[BoxData["96"], "Output"]
}, Open  ]],

Cell["\<\
According to the principle of inclusion and exclusion, this value could also \
be computed as\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Length", "[", "divBy2", "]"}], "+", 
  RowBox[{"Length", "[", "divBy11", "]"}], "-", 
  RowBox[{"Length", "[", 
   RowBox[{"Intersection", "[", 
    RowBox[{"divBy2", ",", "divBy11"}], "]"}], "]"}]}]], "Input"],

Cell[BoxData["96"], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["8.6 Applications of Inclusion-Exclusion", "Section",
 CellTags->"Section6"],

Cell["\<\
In this section we will explore the following problem: Three sets of twins, \
Ashley and Amanda Abel, Brandon and Benjamin Bernoulli, and Christopher and \
Courtney Cartan (none of whom bear any relation to the mathematicians with \
the same surname), are to be seated in a row. List the ways in which they can \
be seated so that no person sits next to his or her twin. \
\>", "Text"],

Cell["\<\
The principle of inclusion-exclusion gives us insight into how we might \
accomplish this task. Rather than attempting to generate only those seating \
arrangements in which no person sits next to his or her twin, it will be \
easier to consider all the possible arrangements of the twins and then \
exclude those that do not satisfy the condition. To begin, we'll define lists \
to store the names of the twins. \
\>", "Text"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"abels", "=", 
   RowBox[{"{", 
    RowBox[{"\"\<Ashley\>\"", ",", "\"\<Amanda\>\""}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"bernoullis", "=", 
   RowBox[{"{", 
    RowBox[{"\"\<Brandon\>\"", ",", "\"\<Benjamin\>\""}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"cartans", "=", 
   RowBox[{"{", 
    RowBox[{"\"\<Christopher\>\"", ",", "\"\<Courtney\>\""}], "}"}]}], 
  ";"}]}], "Input",
 InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"twins", "=", 
  RowBox[{"{", 
   RowBox[{"abels", ",", "bernoullis", ",", "cartans"}], "}"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"\<\"Ashley\"\>", ",", "\<\"Amanda\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"Brandon\"\>", ",", "\<\"Benjamin\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"Christopher\"\>", ",", "\<\"Courtney\"\>"}], "}"}]}], 
  "}"}]], "Output"]
}, Open  ]],

Cell["\<\
Next we create a function to test whether an arrangement satisfies the \
condition of having no twins seated next to each other. We will use this \
function to determine which seatings to accept.\
\>", "Text"],

Cell["\<\
To test whether a given arrangement has a pair of twins seated next to one \
another, we will consider the five pairs of seats, 1 and 2, 2 and 3, ..., 5 \
and 6, and check to see if the people seated in those positions are twins.\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"testSeating", "[", "seating_List", "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"i", ",", "twinpair"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"Catch", "[", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"For", "[", 
       RowBox[{
        RowBox[{"i", "=", "1"}], ",", 
        RowBox[{"i", "\[LessEqual]", "5"}], ",", 
        RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
        RowBox[{"Do", "[", 
         RowBox[{
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{
             RowBox[{"MemberQ", "[", 
              RowBox[{"twinpair", ",", 
               RowBox[{"seating", "[", 
                RowBox[{"[", "i", "]"}], "]"}]}], "]"}], "&&", 
             RowBox[{"MemberQ", "[", 
              RowBox[{"twinpair", ",", 
               RowBox[{"seating", "[", 
                RowBox[{"[", 
                 RowBox[{"i", "+", "1"}], "]"}], "]"}]}], "]"}]}], ",", 
            RowBox[{"Throw", "[", "False", "]"}]}], "]"}], 
          "\[IndentingNewLine]", ",", 
          RowBox[{"{", 
           RowBox[{"twinpair", ",", "twins"}], "}"}]}], "]"}]}], 
       "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"Throw", "[", "True", "]"}]}], "\[IndentingNewLine]", "]"}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "The function is passed a list of the six people's names, representing a \
seating, so that the person in the third seat is ",
 StyleBox["seating[[3]]", "Input"],
 ". The ",
 StyleBox[ButtonBox["For",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/For"], "FunctionLink"],
 " loop indexed by ",
 StyleBox["i", "Input"],
 " goes through the five pairs of seats. The inner ",
 StyleBox[ButtonBox["Do",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Do"], "FunctionLink"],
 " loop avoids having to duplicate the ",
 StyleBox[ButtonBox["If",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/If"], "FunctionLink"],
 " statement. We could have written one ",
 StyleBox[ButtonBox["If",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/If"], "FunctionLink"],
 " statement checking to see if the people in seats ",
 Cell[BoxData[
  FormBox["i", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"i", "+", "1"}], TraditionalForm]]],
 " are both Abels, and then a second ",
 StyleBox[ButtonBox["If",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/If"], "FunctionLink"],
 " statement to see if they are both Bernoullis, and then a third to see if \
they are both Cartans. Instead, the loop sets the ",
 StyleBox["twinpair", "Input"],
 " variable to each of the lists in twins in turn. So ",
 StyleBox["twinpair", "Input"],
 " represents, at each step in the loop, one of the families. And then the ",
 StyleBox[ButtonBox["If",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/If"], "FunctionLink"],
 " statement checks to see whether the people in the seats ",
 Cell[BoxData[
  FormBox["i", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"i", "+", "1"}], TraditionalForm]]],
 " are members of that family, using the ",
 StyleBox[ButtonBox["MemberQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/MemberQ"], "FunctionLink"],
 " test. Recall that ",
 StyleBox[ButtonBox["MemberQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/MemberQ"], "FunctionLink"],
 " returns true if the second argument is a member of the first. If any of \
these ",
 StyleBox[ButtonBox["If",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/If"], "FunctionLink"],
 " statements are true, that the people in the pair of consecutive seats are \
from the same family, then false is thrown to the enclosing ",
 StyleBox[ButtonBox["Catch",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Catch"], "FunctionLink"],
 ", indicating that the seating is not acceptable. If the seating survives \
all of the ",
 StyleBox[ButtonBox["If",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/If"], "FunctionLink"],
 " statements, then the function returns true."
}], "Text"],

Cell[TextData[{
 "To check the ",
 StyleBox["testSeating", "Input"],
 " function, consider the following potential seatings."
}], "Text"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"seating1", "=", 
   RowBox[{"{", 
    RowBox[{
    "\"\<Ashley\>\"", ",", "\"\<Amanda\>\"", ",", "\"\<Brandon\>\"", ",", 
     "\"\<Benjamin\>\"", ",", "\"\<Christopher\>\"", ",", 
     "\"\<Courtnet\>\""}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"seating2", "=", 
   RowBox[{"{", 
    RowBox[{
    "\"\<Ashley\>\"", ",", "\"\<Brandon\>\"", ",", "\"\<Christopher\>\"", 
     ",", "\"\<Amanda\>\"", ",", "\"\<Benjamin\>\"", ",", 
     "\"\<Courtney\>\""}], "}"}]}], ";"}]}], "Input",
 InitializationCell->True],

Cell["\<\
We see that the first seating fails but the second passes, as they should. \
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"testSeating", "[", "seating1", "]"}]], "Input"],

Cell[BoxData["False"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"testSeating", "[", "seating2", "]"}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]],

Cell[TextData[{
 "We now have a function to test a potential seating for the condition of not \
having twins seated next to each other. To generate a list of all of such \
seatings, we'll use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "'s ",
 StyleBox[ButtonBox["Permutations",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Permutations"], "FunctionLink"],
 " function to generate all the possible permutations of the people and then \
test to see which are valid and which should be discarded. The ",
 StyleBox[ButtonBox["Permutations",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Permutations"], "FunctionLink"],
 " command takes a list and returns all the possible permutations of the \
objects. (Refer to Chapter 6 of this manual for more information about this \
function.)"
}], "Text"],

Cell[TextData[{
 "Note that the ",
 StyleBox[ButtonBox["Permutations",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Permutations"], "FunctionLink"],
 " function expects a list of objects, so we will need a list of all the \
people. We can use the ",
 StyleBox[ButtonBox["Flatten",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Flatten"], "FunctionLink"],
 " function to turn our ",
 StyleBox["twins", "Input"],
 " list into a list of all the names. ",
 StyleBox[ButtonBox["Flatten",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Flatten"], "FunctionLink"],
 " takes a list and removes any nesting of lists so that the result is a list \
of the objects."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Flatten", "[", "twins", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\<\"Ashley\"\>", ",", "\<\"Amanda\"\>", ",", "\<\"Brandon\"\>", 
   ",", "\<\"Benjamin\"\>", ",", "\<\"Christopher\"\>", 
   ",", "\<\"Courtney\"\>"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "To create the list of only the valid seatings, we will apply the ",
 StyleBox[ButtonBox["Select",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Select"], "FunctionLink"],
 " function. ",
 StyleBox[ButtonBox["Select",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Select"], "FunctionLink"],
 " requires two arguments. The first is a list of objects, and the second is \
a function of one argument that returns ",
 ButtonBox["true",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/True"],
 " for those elements of the list that satisfy the desired criterion. The \
result is the list of the elements that passed the test."
}], "Text"],

Cell[TextData[{
 "We apply ",
 StyleBox[ButtonBox["Select",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Select"], "FunctionLink"],
 " to all of the permutations of the flattened list of twins with criterion \
function ",
 StyleBox["testSeating", "Input"],
 " and store its output in the variable ",
 StyleBox["twinSeatings", "Input"],
 " but suppress the output. Then we'll use ",
 StyleBox[ButtonBox["Length",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Length"], "FunctionLink"],
 " to check how many possible seatings there are. (It is generally a good \
idea to suppress the output of a function that is listing what may be a very \
large number of possibilities until you know how many there are, as the \
output may take some time to display.)"
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"twinSeatings", "=", 
   RowBox[{"Select", "[", 
    RowBox[{
     RowBox[{"Permutations", "[", 
      RowBox[{"Flatten", "[", "twins", "]"}], "]"}], ",", "testSeating"}], 
    "]"}]}], ";"}]], "Input",
 InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Length", "[", "twinSeatings", "]"}]], "Input"],

Cell[BoxData["240"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"twinSeatings", "[", 
  RowBox[{"[", "123", "]"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\<\"Benjamin\"\>", ",", "\<\"Ashley\"\>", 
   ",", "\<\"Christopher\"\>", ",", "\<\"Amanda\"\>", ",", "\<\"Brandon\"\>", 
   ",", "\<\"Courtney\"\>"}], "}"}]], "Output"]
}, Open  ]],

Cell["\<\
We see that there are 240 possible seatings and have displayed the 123rd \
seating. \
\>", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
Solutions to Computer Projects and Computations and Explorations\
\>", "Section",
 PageBreakWithin->False,
 CellTags->"Solutions"],

Cell[CellGroupData[{

Cell["Computer Projects 12", "Subsection",
 PageBreakAbove->False,
 PageBreakWithin->False,
 CellTags->"SolutionsCP12"],

Cell[TextData[{
 "Given positive integers ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ", find the number of onto functions from a set with ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 " elements to a set with ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " elements."
}], "Problem",
 PageBreakAbove->False,
 PageBreakWithin->False],

Cell[TextData[{
 StyleBox["Solution:",
  FontSlant->"Italic"],
 " We have a very convenient formula:"
}], "Text"],

Cell[BoxData[
 FormBox[
  RowBox[{
   UnderoverscriptBox["\[Sum]", 
    RowBox[{"k", "=", "0"}], 
    RowBox[{"n", "-", "1"}],
    LimitsPositioning->False], " ", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}], "k"], 
    RowBox[{"C", "(", 
     RowBox[{"n", ",", "k"}], ")"}], 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"n", "-", "k"}], ")"}], "m"]}]}], TraditionalForm]], "Equation"],

Cell[TextData[{
 "This is the number of onto functions from a set of ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 " elements to a set of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " elements, assuming ",
 Cell[BoxData[
  FormBox[
   RowBox[{"m", "\[GreaterEqual]", "n"}], TraditionalForm]]],
 ". This formula is derived in the textbook from the principle of \
inclusion-exclusion (see Theorem 1 of Section 8.6). The only input required \
in this formula are the integer parameters ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ", which represent the sizes of the domain and codomain, respectively. ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "\[CloseCurlyQuote]s ",
 StyleBox[ButtonBox["Sum",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Sum"], "FunctionLink"],
 " function will compute summations such as the one above. The first argument \
is an expression in terms of an index of summation, e.g., ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 ", and the second is of the form ",
 StyleBox["{k,a,b}", "Input"],
 " indicating the bounds of the summation. For example, to compute"
}], "Text"],

Cell[BoxData[
 FormBox[
  RowBox[{
   UnderoverscriptBox["\[Sum]", 
    RowBox[{"k", "=", "3"}], "8",
    LimitsPositioning->False], " ", 
   FractionBox["1", "k"]}], TraditionalForm]], "Equation"],

Cell["you would enter the following expressions.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Sum", "[", 
  RowBox[{
   RowBox[{"1", "/", "k"}], ",", 
   RowBox[{"{", 
    RowBox[{"k", ",", "3", ",", "8"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 FractionBox["341", "280"]], "Output"]
}, Open  ]],

Cell["We now create a function encapsulating the formula above.", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"ontoFunctions", "[", 
    RowBox[{"m_Integer", ",", "n_Integer"}], "]"}], "/;", 
   RowBox[{
    RowBox[{"m", ">", "0"}], "&&", 
    RowBox[{"n", ">", "0"}]}]}], ":=", 
  RowBox[{"If", "[", 
   RowBox[{
    RowBox[{"m", "\[GreaterEqual]", "n"}], ",", 
    RowBox[{"Sum", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{"-", "1"}], ")"}], "^", "k"}], " ", "*", " ", 
       RowBox[{"Binomial", "[", 
        RowBox[{"n", ",", "k"}], "]"}], "*", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"n", "-", "k"}], ")"}], "^", "m"}]}], ",", 
      RowBox[{"{", 
       RowBox[{"k", ",", "0", ",", 
        RowBox[{"n", "-", "1"}]}], "}"}]}], "]"}], ",", "0"}], 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["If",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/If"], "FunctionLink"],
 " statement captures the fact that there are no onto functions from a set to \
a set that is larger and ensures that the result is 0 in that case. The ",
 StyleBox[ButtonBox["Condition",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Condition"], "FunctionLink"],
 " (",
 StyleBox["/;", "Input"],
 ") ensures that the input to the function is positive and will return no \
output if not, since it is meaningless to ask about functions between sets of \
non-positive cardinalities. For example,"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ontoFunctions", "[", 
  RowBox[{"4", ",", "9"}], "]"}]], "Input"],

Cell[BoxData["0"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ontoFunctions", "[", 
  RowBox[{
   RowBox[{"-", "3"}], ",", "0"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"ontoFunctions", "[", 
  RowBox[{
   RowBox[{"-", "3"}], ",", "0"}], "]"}]], "Output"]
}, Open  ]],

Cell["\<\
As an example, we can use our function to compute the number of onto \
functions from a set with 100 elements to a set with 20 elements. \
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ontoFunctions", "[", 
  RowBox[{"100", ",", "20"}], "]"}]], "Input"],

Cell[BoxData[\
"11238195910319657928539447038143170285517894975095769496294319007413091913959\
828334936464196298192508890182316163261067934269440000"], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Computations and Explorations 2", "Subsection",
 CellTags->"SolutionsCE2"],

Cell["\<\
Find the smallest Fibonacci number greater than 1,000,000, greater than \
1,000,000,000, and greater than 1,000,000,000,000. \
\>", "Problem"],

Cell[TextData[{
 StyleBox["Solution:",
  FontSlant->"Italic"],
 " We can solve this quite easily with ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " using a simple ",
 StyleBox[ButtonBox["While",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/While"], "FunctionLink"],
 " loop. In this chapter we've seen several ways to compute Fibonacci \
numbers, including the ",
 StyleBox["fibonacci", "Input"],
 " function we created in section 8.1 and the formula ",
 StyleBox["fibonacci2", "Input"],
 " in section 8.2. For this exercise, we'll use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "'s built-in function ",
 StyleBox[ButtonBox["Fibonacci",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Fibonacci"], "FunctionLink"],
 "."
}], "Text"],

Cell[TextData[{
 "The idea is to compute Fibonacci numbers until the value exceeds the \
target. The ",
 StyleBox[ButtonBox["While",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/While"], "FunctionLink"],
 " loop is well-suited to this sort of problem. We will create a function \
that takes the target value as input and prints out the desired Fibonacci \
number and its index."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"findFib", "[", "target_Integer", "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"n", "=", "1"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"While", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Fibonacci", "[", "n", "]"}], "<", "target"}], ",", 
       RowBox[{"n", "++"}]}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"Print", "[", 
      RowBox[{
      "\"\<The \>\"", ",", "n", ",", "\"\<th Fibonacci number is \>\"", ",", 
       RowBox[{"Fibonacci", "[", "n", "]"}]}], "]"}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "As long as the ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 "th Fibonacci number is smaller than the target value, the index ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " is increased. Once the target has been exceeded, the ",
 StyleBox[ButtonBox["Print",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Print"], "FunctionLink"],
 " statement displays the index and the value of the Fibonacci number."
}], "Text"],

Cell["The numbers called for by the question are:", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"findFib", "[", "1000000", "]"}]], "Input"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"The \"\>", "\[InvisibleSpace]", "31", 
   "\[InvisibleSpace]", "\<\"th Fibonacci number is \"\>", 
   "\[InvisibleSpace]", "1346269"}],
  SequenceForm["The ", 31, "th Fibonacci number is ", 1346269],
  Editable->False]], "Print"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"findFib", "[", "1000000000", "]"}]], "Input"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"The \"\>", "\[InvisibleSpace]", "45", 
   "\[InvisibleSpace]", "\<\"th Fibonacci number is \"\>", 
   "\[InvisibleSpace]", "1134903170"}],
  SequenceForm["The ", 45, "th Fibonacci number is ", 1134903170],
  Editable->False]], "Print"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"findFib", "[", "1000000000000", "]"}]], "Input"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"The \"\>", "\[InvisibleSpace]", "60", 
   "\[InvisibleSpace]", "\<\"th Fibonacci number is \"\>", 
   "\[InvisibleSpace]", "1548008755920"}],
  SequenceForm["The ", 60, "th Fibonacci number is ", 1548008755920],
  Editable->False]], "Print"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Computations and Explorations 3", "Subsection",
 CellTags->"SolutionsCE3"],

Cell["\<\
Find as many prime Fibonacci numbers as you can. It is unknown whether there \
are infinitely many of these. \
\>", "Problem"],

Cell[TextData[{
 StyleBox["Solution:",
  FontSlant->"Italic"],
 " Using ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", this sort of problem becomes fairly straightforward. We can simply use \
the ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " function ",
 StyleBox[ButtonBox["Fibonacci",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Fibonacci"], "FunctionLink"],
 " to generate Fibonacci numbers and use the ",
 StyleBox[ButtonBox["PrimeQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/PrimeQ"], "FunctionLink"],
 " function to test each for primality. We will wrap this in a function that \
takes a number of seconds as an argument and uses ",
 StyleBox[ButtonBox["TimeConstrained",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/TimeConstrained"], "FunctionLink"],
 " to control the length of the evaluation. Note the use of ",
 StyleBox[ButtonBox["Sow",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Sow"], "FunctionLink"],
 " within the ",
 StyleBox[ButtonBox["TimeConstrained",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/TimeConstrained"], "FunctionLink"],
 " block and ",
 StyleBox[ButtonBox["Reap",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Reap"], "FunctionLink"],
 " surrounding it. This allows the termination of the ",
 StyleBox[ButtonBox["TimeConstrained",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/TimeConstrained"], "FunctionLink"],
 " portion of the function to trigger the ",
 StyleBox[ButtonBox["Reap",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Reap"], "FunctionLink"],
 "."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"primeFib", "[", "time_", "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"i", "=", "0"}], ",", "temp", ",", 
      RowBox[{"primes", "=", 
       RowBox[{"{", "}"}]}]}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"Reap", "[", "\[IndentingNewLine]", 
      RowBox[{"TimeConstrained", "[", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"While", "[", 
         RowBox[{"True", ",", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"i", "++"}], ";", "\[IndentingNewLine]", 
           RowBox[{"temp", "=", 
            RowBox[{"Fibonacci", "[", "i", "]"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{"PrimeQ", "[", "temp", "]"}], ",", 
             RowBox[{"Sow", "[", "temp", "]"}]}], "]"}]}]}], 
         "\[IndentingNewLine]", "]"}], ",", "time"}], "]"}], 
      "\[IndentingNewLine]", "]"}], "[", 
     RowBox[{"[", 
      RowBox[{"2", ",", "1"}], "]"}], "]"}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell["\<\
We can obtain several examples even in only a hundredth of a second.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"primeFib", "[", "0.01", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "2", ",", "3", ",", "5", ",", "13", ",", "89", ",", "233", ",", "1597", ",",
    "28657", ",", "514229", ",", "433494437", ",", "2971215073", ",", 
   "99194853094755497", ",", "1066340417491710595814572169", ",", 
   "19134702400093278081449423917"}], "}"}]], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Computations and Explorations 11", "Subsection",
 CellTags->"SolutionsCE11"],

Cell[TextData[{
 "Compute the probability that a permutation of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " objects is a derangement for all positive integers not exceeding 20 and \
determine how quickly these probabilities approach the number ",
 Cell[BoxData[
  FormBox[
   RowBox[{"1", "/", "e"}], TraditionalForm]]],
 "."
}], "Problem"],

Cell[TextData[{
 StyleBox["Solution:",
  FontSlant->"Italic"],
 " To solve this problem, we will make use of the formula which gives the \
number of derangements of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " objects, namely,"
}], "Text"],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["D", "n"], "=", 
   RowBox[{
    RowBox[{"n", "!"}], " ", "[", 
    RowBox[{"1", "-", 
     FractionBox["1", 
      RowBox[{"1", "!"}]], "+", 
     FractionBox["1", 
      RowBox[{"2", "!"}]], "-", 
     FractionBox["1", 
      RowBox[{"3", "!"}]], "+", "\[Ellipsis]", "+", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"-", "1"}], ")"}], "n"], 
      FractionBox["1", 
       RowBox[{"n", "!"}]]}]}], "]"}]}], TraditionalForm]], "Equation"],

Cell[TextData[{
 "The total number of permutations of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " objects is, of course, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "!"}], TraditionalForm]]],
 ", so the probability that one of them is a derangement is the ratio ",
 Cell[BoxData[
  FormBox[
   FractionBox[
    SubscriptBox["D", "n"], 
    RowBox[{"n", "!"}]], TraditionalForm]]],
 ", which is given by the expression"
}], "Text"],

Cell[BoxData[
 FormBox[
  RowBox[{"1", "-", 
   FractionBox["1", 
    RowBox[{"1", "!"}]], "+", 
   FractionBox["1", 
    RowBox[{"2", "!"}]], "-", 
   FractionBox["1", 
    RowBox[{"3", "!"}]], "+", "\[Ellipsis]", "+", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}], "n"], 
    FractionBox["1", 
     RowBox[{"n", "!"}]]}]}], TraditionalForm]], "Equation"],

Cell[TextData[{
 "A very simple ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " function will compute these values for us."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"derangementP", "[", "n_Integer", "]"}], "/;", 
   RowBox[{"n", ">", "0"}]}], ":=", 
  RowBox[{"Sum", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "^", "k"}], "*", 
     RowBox[{"1", "/", 
      RowBox[{"k", "!"}]}]}], ",", 
    RowBox[{"{", 
     RowBox[{"k", ",", "0", ",", "n"}], "}"}]}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "The probabilities that a permutation of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " objects is a derangement for ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "\[LessEqual]", "20"}], TraditionalForm]]],
 " are:"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Table", "[", 
  RowBox[{
   RowBox[{"derangementP", "[", "n", "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"n", ",", "20"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", 
   FractionBox["1", "2"], ",", 
   FractionBox["1", "3"], ",", 
   FractionBox["3", "8"], ",", 
   FractionBox["11", "30"], ",", 
   FractionBox["53", "144"], ",", 
   FractionBox["103", "280"], ",", 
   FractionBox["2119", "5760"], ",", 
   FractionBox["16687", "45360"], ",", 
   FractionBox["16481", "44800"], ",", 
   FractionBox["1468457", "3991680"], ",", 
   FractionBox["16019531", "43545600"], ",", 
   FractionBox["63633137", "172972800"], ",", 
   FractionBox["2467007773", "6706022400"], ",", 
   FractionBox["34361893981", "93405312000"], ",", 
   FractionBox["15549624751", "42268262400"], ",", 
   FractionBox["8178130767479", "22230464256000"], ",", 
   FractionBox["138547156531409", "376610217984000"], ",", 
   FractionBox["92079694567171", "250298560512000"], ",", 
   FractionBox["4282366656425369", "11640679464960000"]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "To see how these probabilities differ from ",
 Cell[BoxData[
  FormBox[
   RowBox[{"1", "/", "e"}], TraditionalForm]]],
 ", we'll multiply them by ",
 Cell[BoxData[
  FormBox["e", TraditionalForm]]],
 " and subtract 1. To represent the number ",
 Cell[BoxData[
  FormBox["e", TraditionalForm]]],
 " in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", we use the symbol ",
 StyleBox[ButtonBox["E",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/E"], "FunctionLink"],
 ". We also apply ",
 StyleBox[ButtonBox["N",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/N"], "FunctionLink"],
 " with second argument 25 in order to obtain numerical approximations with \
10 digits of precision."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Table", "[", 
  RowBox[{
   RowBox[{"N", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"E", "*", 
       RowBox[{"derangementP", "[", "n", "]"}]}], "-", "1"}], ",", "10"}], 
    "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"n", ",", "20"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"-", "1.`10."}], ",", 
   "0.35914091422952261768014373567633124888`10.", ",", 
   RowBox[{"-", "0.09390605718031825487990417621577916742`10."}], ",", 
   "0.01935568567214196326010780175724843666`10.", ",", 
   RowBox[{"-", "0.00329666289835008036789459383735708416`10."}], ",", 
   "0.00047872853006526023677247209507716931`10.", ",", 
   RowBox[{"-", "0.00006061310256550270675139446669915261`10."}], ",", 
   "6.80460151334266118908885352288763`10.*^-6", ",", 
   RowBox[{"-", "6.8625449541793524874262650178351`10.*^-7"}], ",", 
   "6.283110545812439504052150068361`10.*^-8", ",", 
   RowBox[{"-", "5.26758553060829984885559044977`10.*^-9"}], ",", 
   "4.07305385119424725259167144679592204313783315575`10.*^-10", ",", 
   RowBox[{"-", "2.9224685321169472749660362585625618269569400012`10.*^-11"}],
    ",", "1.956033996015827108113030790461369057812936816`10.*^-12", ",", 
   RowBox[{"-", "1.22680625129859549071862101277763430679218973`10.*^-13"}], 
   ",", "7.239038691745867002193704455932349851540764`10.*^-15", ",", 
   RowBox[{"-", "4.0329447423092217863310764604975488556275`10.*^-16"}], ",", 
   "2.1279590545566109190604137393695377609667`10.*^-17", ",", 
   RowBox[{"-", "1.066412863722748063275430155959899399407`10.*^-18"}], ",", 
   "5.0887306741694799418548221522864451047`10.*^-20"}], "}"}]], "Output"]
}, Open  ]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Exercises", "Section",
 CellTags->"Exercises"],

Cell[CellGroupData[{

Cell["\<\
Implement a function to find the optimal schedule that maximizes total \
attendance.\
\>", "Item1Exercise"],

Cell["\<\
Implement a dynamic programming algorithm for finding the maximum sum of \
consecutive terms of a sequence of real numbers. (See problem 56 in Section \
8.1.) \
\>", "Item1Exercise"],

Cell["\<\
Implement a dynamic programming algorithm for optimally computing \
matrix-chain multiplication. (See problem 57 in Section 8.1.) \
\>", "Item1Exercise"],

Cell[TextData[{
 "Use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to solve the following recurrence relations."
}], "Item1Exercise"],

Cell[CellGroupData[{

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["a", "n"], "=", 
    RowBox[{
     SubscriptBox["a", 
      RowBox[{"n", "-", "1"}]], "-", 
     SubscriptBox["a", 
      RowBox[{"n", "-", "2"}]]}]}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["a", "1"], "=", "1"}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["a", "2"], "=", "1"}], TraditionalForm]]],
 ";"
}], "Item2Exercise"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["a", "n"], "=", 
    RowBox[{
     RowBox[{"15", 
      SubscriptBox["a", 
       RowBox[{"n", "-", "1"}]]}], "+", 
     RowBox[{
      FractionBox["1", "2"], 
      SubscriptBox["a", 
       RowBox[{"n", "-", "2"}]]}]}]}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["a", "1"], "=", 
    FractionBox["23", "22"]}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["a", "2"], "=", 
    FractionBox["7", "2"]}], TraditionalForm]]]
}], "Item2Exercise"]
}, Open  ]],

Cell[TextData[{
 "Use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to solve each of the recurrence relations in Exercise 1 in Section 8.2 of \
the textbook. (Solve even those that are not linear homogeneous recurrence \
relations with constant coefficients.)"
}], "Item1Exercise"],

Cell[TextData[{
 "Write a general solver in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " for linear homogeneous recurrence relations with constant coefficients of \
degree 3 with distinct roots. Your solver should check that the roots are in \
fact distinct and, if they are not, should return ",
 StyleBox["$Failed", "Input"],
 ", which is a standard return value for a ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " function when it cannot complete a computation for some reason."
}], "Item1Exercise"],

Cell[TextData[{
 "Use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to investigate the behavior of the limit"
}], "Item1Exercise"],

Cell[BoxData[
 FormBox[
  RowBox[{
   UnderscriptBox["lim", 
    RowBox[{"n", "\[Rule]", "\[Infinity]"}]], 
   FractionBox[
    SubscriptBox["\[CurlyPhi]", "n"], 
    SubscriptBox["\[Psi]", "n"]]}], TraditionalForm]], "ExerciseEquation"],

Cell[TextData[{
 "where ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[CurlyPhi]", "n"], TraditionalForm]]],
 " is defined to be the number of prime Fibonacci numbers less than or equal \
to ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[Psi]", "n"], TraditionalForm]]],
 " is defined to be the number of Fermat numbers less than or equal to ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 "."
}], "ExerciseContinuation"],

Cell[TextData[{
 "Use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to find the number of square-free integers less than 100,000,000."
}], "Item1Exercise"],

Cell[TextData[{
 "Use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to find the number of onto functions from a set with 1,000,000 elements to \
a set with 1,000 elements."
}], "Item1Exercise"],

Cell[TextData[{
 "It is probably obvious that the number of onto functions from one set to \
another increases with the sizes of either the domain or the range. Using ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to experiment, explore whether an increase in the size of the domain or \
the size of the range has the greater impact on the number of onto \
functions."
}], "Item1Exercise"],

Cell[TextData[{
 "To generate the ",
 StyleBox["lucky numbers",
  FontSlant->"Italic"],
 ", start with the positive integers and delete every second integer in the \
list, starting the count with 1 (e.g., delete 2, 4, 6, etc., leaving 1, 3, 5, \
7, ...). Other than 1, the smallest integer left is 3. Continue by deleting \
every third integer from those that remain, starting the counting with 1 \
(since 1, 3, 5, 7, 9, ... remain, 1 is the first number left, 3 is the second \
one left, 5 is the third left and gets deleted, and so on). Continue the \
process where at each stage, every ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 "th integer is deleted, where ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " is the smallest integer left, other than the previous values of ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 ". The integers that remain are the lucky numbers. Develop a ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " function that generates the lucky numbers up to ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 "."
}], "Item1Exercise"],

Cell["\<\
Can you make any conjectures about lucky numbers by looking at a list of the \
first 1000 of them? For example, what sort of conjectures can you make about \
twin lucky numbers? What evidence do you have for your conjectures?\
\>", "Item1Exercise"],

Cell[TextData[{
 "Generalize the ",
 StyleBox["listSeatings", "Input"],
 " function to accept one argument, a list of lists (the same structure as \
the twins list), and determines the arrangements such that no two from the \
same sublist are seated next to one another."
}], "Item1Exercise"],

Cell[TextData[{
 "Further generalize the ",
 StyleBox["listSeatings", "Input"],
 " function so that it takes two arguments: a list of lists as before and a \
number ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ". The function should determine the arrangements of the people such that no \
",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " from the same sublist are seated together."
}], "Item1Exercise"]
}, Open  ]]
}, Closed]]
}, Open  ]]
},
WindowSize->{725, 850},
WindowMargins->{{4, Automatic}, {Automatic, 4}},
DockedCells->Cell[
  BoxData[
   TagBox[
    GridBox[{{"\"Exploring Discrete Mathematics with Mathematica\"", 
       ActionMenuBox[
       "\"Navigation\"", {
        "\"CloseAll\"" :> Module[{$CellContext`DJSection$}, SelectionMove[
             SelectedNotebook[], All, Notebook]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; 
           Do[NotebookLocate[$CellContext`DJSection$]; FrontEndExecute[
               FrontEndToken["OpenCloseGroup"]], {$CellContext`DJSection$, {
              "Section1a", "Section1b", "Section2a", "Section2b", "Section2c",
                "Section2d", "Section4a", "Section4b", "SolutionsCE11", 
               "SolutionsCE2", "SolutionsCE3", "SolutionsCP12", "Section1", 
               "Section2", "Section3", "Section4", "Section5", "Section6", 
               "Solutions", "Exercises"}}]; SelectionMove[
             SelectedNotebook[], Before, Notebook]], "\"Section1\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Section5", 
                "Section6", "Solutions", "Exercises"}}] 
            NotebookLocate["Section1"]; FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Section2\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Section5", 
                "Section6", "Solutions", "Exercises"}}] 
            NotebookLocate["Section2"]; FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Section3\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Section5", 
                "Section6", "Solutions", "Exercises"}}] 
            NotebookLocate["Section3"]; FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Section4\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Section5", 
                "Section6", "Solutions", "Exercises"}}] 
            NotebookLocate["Section4"]; FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Section5\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Section5", 
                "Section6", "Solutions", "Exercises"}}] 
            NotebookLocate["Section5"]; FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Section6\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Section5", 
                "Section6", "Solutions", "Exercises"}}] 
            NotebookLocate["Section6"]; FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Solutions\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Section5", 
                "Section6", "Solutions", "Exercises"}}] 
            NotebookLocate["Solutions"]; FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Exercises\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Section5", 
                "Section6", "Solutions", "Exercises"}}] 
            NotebookLocate["Exercises"]; FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Open All\"" :> 
         Module[{$CellContext`DJSection$}, 
           Do[NotebookLocate[$CellContext`DJSection$]; FrontEndExecute[
               FrontEndToken[
               "SelectionOpenAllGroups"]], {$CellContext`DJSection$, {
              "Exercises", "Section1", "Section1a", "Section1b", "Section2", 
               "Section2a", "Section2b", "Section2c", "Section2d", "Section3",
                "Section4", "Section4a", "Section4b", "Section5", "Section6", 
               "Solutions", "SolutionsCE11", "SolutionsCE2", "SolutionsCE3", 
               "SolutionsCP12"}}] SelectionMove[
             SelectedNotebook[], Before, Notebook]]}]}}, 
     GridBoxAlignment -> {
      "Columns" -> {Left, Baseline}, "Rows" -> {Right, Baseline}}, AutoDelete -> 
     False, GridBoxItemSize -> {"Columns" -> {
         Scaled[0.8], 
         Scaled[0.2]}}], "Grid"]], "DockedCell", FontFamily -> "Times"],
PrintingCopies->1,
PrintingPageRange->{1, Automatic},
Magnification->1,
FrontEndVersion->"9.0 for Mac OS X x86 (32-bit, 64-bit Kernel) (January 25, \
2013)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> 
     FrontEnd`FileName[{"Book"}, "Textbook.nb", CharacterEncoding -> 
       "UTF-8"]]], 
   Cell[
    StyleData[All, "Printout"], PageWidth -> PaperWidth, 
    CellHorizontalScrolling -> True, 
    CellLabelMargins -> {{2, Inherited}, {Inherited, Inherited}}, 
    ShowAutoStyles -> False, 
    AutoStyleOptions -> {"HighlightMisspelledWords" -> False}, ScriptMinSize -> 
    9, MenuSortingValue -> 1500, 
    PrivateFontOptions -> {"FontType" -> "Outline"}, Magnification -> 1], 
   Cell[
    StyleData["Title"], Editable -> False, ShowCellBracket -> Automatic, 
    CellGroupingRules -> {"TitleGrouping", 0}, PageBreakBelow -> False, 
    DefaultNewInlineCellStyle -> {}, LanguageCategory -> "NaturalLanguage", 
    FontFamily -> "Helvetica", FontSize -> 25, FontWeight -> "Bold", 
    FontColor -> GrayLevel[0.137255]], 
   Cell[
    StyleData["BookChapterTitle", StyleDefinitions -> StyleData["Title"]], 
    CellFrame -> {{0, 0}, {1, 1}}, CellDingbat -> Cell[
      BoxData[
       GraphicsBox[{{
          RGBColor[0.09375, 0.570313, 0.816406], 
          RectangleBox[{0, 0}]}, {
          GrayLevel[1], FontSize -> 35, FontWeight -> Bold, FontFamily -> 
          "Helvetica", 
          InsetBox[
           CounterBox["ChapterNumber"], {0.5, 0.45}, {Center, Center}]}}, 
        ImageSize -> {70, 70}]], "Output"], CellMargins -> {{75, 0}, {20, 5}},
     CellFrameColor -> GrayLevel[1]], 
   Cell[
    StyleData["DockedCell"], CellMargins -> {{0, 12}, {0, 0}}, 
    CellGroupingRules -> "TitleGrouping", 
    ConversionRules :> {"TeX" -> {"\n\\chapter{", "}"}}, 
    TaggingRules :> {"CounterSettings" -> {"CounterTarget" -> "CellGroup"}}, 
    PageBreakAbove -> False, PageBreakBelow -> False, LineSpacing -> {1, 2}, 
    MenuSortingValue -> 1105, FontFamily -> "Helvetica", FontSize -> 20, 
    FontWeight -> "Bold", Background -> 
    RGBColor[0.09375, 0.570313, 0.816406]], 
   Cell[
    StyleData["FunctionLink"], ConversionRules :> {"TeX" -> {{"", 
         $CellContext`ExportAsHyperRef[], ""}, {"", Automatic, ""}}}, 
    MenuSortingValue -> 1105, ButtonStyleMenuListing -> None, FontFamily -> 
    "Courier", FontVariations -> {"Underline" -> True}, FontColor -> 
    RGBColor[0, 0, 1], 
    ButtonBoxOptions -> {Active -> True, ButtonFunction :> (FrontEndExecute[{
         FrontEnd`NotebookLocate[#2]}]& ), ButtonNote -> ButtonData}], 
   Cell[
    StyleData["Problem"], CellMargins -> {{40, 12}, {15, 15}}, 
    CellGroupingRules -> {"GroupTogetherNestedGrouping", 15010}, 
    ConversionRules :> {
     "TeX" -> {{"", Automatic, ""}, {"\n\n", Automatic, ""}}}, 
    TaggingRules :> {
     "ExtraConversionRules" -> {"General" -> {"RestartList" -> False}}}], 
   Cell[
    CellGroupData[{
      Cell[
       StyleData["Item1Exercise"], CellDingbat -> Cell[
         TextData[{
           CounterBox["Item1Exercise"], "."}], CellSize -> {30, Inherited}, 
         TextAlignment -> Right, FontWeight -> "Bold"], 
       CellMargins -> {{50, 12}, {3, 5}}, 
       CellGroupingRules -> {"GroupTogetherNestedGrouping", 17000}, 
       CellFrameLabels -> {{None, None}, {None, None}}, CellFrameLabelMargins -> 
       6, CounterIncrements -> "Item1Exercise", 
       CounterAssignments -> {{"Item2Exercise", 0}}, MenuSortingValue -> 
       4100], 
      Cell[
       StyleData["Item1Exercise", "Printout"], CellDingbat -> Cell[
         TextData[{
           CounterBox["Item1Exercise"], "."}], CellSize -> {20, Inherited}, 
         TextAlignment -> Right, FontWeight -> "Bold"], 
       CellMargins -> {{35, 12}, {3, 5}}, 
       CellGroupingRules -> {"GroupTogetherNestedGrouping", 17000}, 
       CellFrameLabels -> {{None, None}, {None, None}}, CellFrameLabelMargins -> 
       6]}, Open]], 
   Cell[
    CellGroupData[{
      Cell[
       StyleData["ExerciseContinuation"], CellMargins -> {{50, 12}, {1, 3}}, 
       CellGroupingRules -> {"GroupTogetherNestedGrouping", 17000}, 
       CellFrameLabels -> {{None, None}, {None, None}}, CellFrameLabelMargins -> 
       6, MenuSortingValue -> 4101], 
      Cell[
       StyleData["ExerciseContinuation", "Printout"], 
       CellMargins -> {{35, 12}, {1, 3}}, 
       CellFrameLabels -> {{None, None}, {None, None}}, CellFrameLabelMargins -> 
       6]}, Open]], 
   Cell[
    CellGroupData[{
      Cell[
       StyleData["Item2Exercise"], CellDingbat -> Cell[
         TextData[{
           CounterBox["Item2Exercise"], "."}], CellSize -> {32, Inherited}, 
         TextAlignment -> Right, FontWeight -> "Bold"], 
       CellMargins -> {{75, 12}, {1, 2}}, 
       CellGroupingRules -> {"GroupTogetherNestedGrouping", 17100}, 
       CellFrameLabels -> {{None, None}, {None, None}}, CellFrameLabelMargins -> 
       6, CounterIncrements -> "Item2Exercise", MenuSortingValue -> 4200, 
       CounterBoxOptions -> {CounterFunction :> (Part[
           CharacterRange["a", "z"], #]& )}], 
      Cell[
       StyleData["Item2Exercise", "Printout"], CellDingbat -> Cell[
         TextData[{
           CounterBox["Item2Exercise"], "."}], CellSize -> {20, Inherited}, 
         TextAlignment -> Right, FontWeight -> "Bold"], 
       CellMargins -> {{50, 12}, {1, 2}}, 
       CellFrameLabels -> {{None, None}, {None, None}}, CellFrameLabelMargins -> 
       6, CounterBoxOptions -> {CounterFunction :> (Part[
           CharacterRange["a", "z"], #]& )}]}, Open]], 
   Cell[
    StyleData["ExerciseEquation"], CellMargins -> {{36, 12}, {7, 7}}, 
    CellGroupingRules -> {"GroupTogetherNestedGrouping", 17200}, 
    CellHorizontalScrolling -> True, 
    CellFrameLabels -> {{None, None}, {None, None}}, DefaultFormatType -> 
    TraditionalForm, InputAutoReplacements -> {}, TextAlignment -> Center, 
    MenuSortingValue -> 4300, 
    ButtonBoxOptions -> {Appearance -> {Automatic, None}}], 
   Cell[
    StyleData["myCode"], CellMargins -> {{66, 10}, {5, 10}}, 
    StyleKeyMapping -> {"=" -> "WolframAlphaShort"}, Evaluatable -> False, 
    CellGroupingRules -> "InputGrouping", TextClipboardType -> "InputText", 
    PageBreakWithin -> False, GroupPageBreakWithin -> False, 
    DefaultFormatType -> DefaultInputFormatType, ContextMenu -> 
    FEPrivate`FrontEndResource["ContextMenus", "myCode"], ShowAutoStyles -> 
    True, "TwoByteSyntaxCharacterAutoReplacement" -> True, 
    HyphenationOptions -> {"HyphenationCharacter" -> "\[Continuation]"}, 
    AutoItalicWords -> {}, LanguageCategory -> "Mathematica", FormatType -> 
    InputForm, NumberMarks -> True, 
    LinebreakAdjustments -> {0.85, 2, 10, 0, 1}, CounterIncrements -> "Code", 
    MenuSortingValue -> 1495, MenuCommandKey -> "8", FontWeight -> 
    "Bold", $CellContext`StripStyleOnPaste -> 
    True, $CellContext`AutoQuoteCharacters -> {}, \
$CellContext`PasteAutoQuoteCharacters -> {}], 
   Cell[
    StyleData["CodeVar"], DefaultFormatType -> DefaultInputFormatType, 
    LineBreakWithin -> False, MenuSortingValue -> 10000, FontWeight -> "Bold",
     FontSlant -> "Italic", FontColor -> GrayLevel[0.4]], 
   Cell[
    CellGroupData[{
      Cell[
       StyleData["Section"], CellFrame -> {{6, 0}, {0, 1}}, ShowCellBracket -> 
       Automatic, CellMargins -> {{12, 12}, {3, 24}}, 
       CellGroupingRules -> {"SectionGrouping", 30}, 
       ConversionRules :> {
        "TeX" -> {"\n\\section{", "}"}, 
         "HTML" -> {"<h3 class=\"Section\">", "</h3>"}}, 
       TaggingRules :> {
        "CounterSettings" -> {
          "CounterFormat" -> {"BookChapterNumber", ".", "Section"}, 
           "CounterTarget" -> "Cell", "XRefPrefix" -> "Section"}}, 
       PageBreakBelow -> False, CellFrameMargins -> {{8, 8}, {4, 6}}, 
       CellFrameColor -> RGBColor[0.09375, 0.570313, 0.816406], TabSpacings -> 
       2.1, CounterIncrements -> {
        "Section", "SectionHeader", "SubsectionHeader", "SubsubsectionHeader",
          "SubsubsubsectionHeader"}, FontFamily -> "Helvetica", FontSize -> 
       16, FontWeight -> "Bold", Background -> GrayLevel[0.999985]], 
      Cell[
       StyleData["Section", "Printout"], CellFrame -> {{4, 0}, {0, 0.5}}, 
       CellMargins -> {{2, 2}, {3, 21}}, PageBreakBelow -> False, 
       CellFrameMargins -> {{6, 6}, {3, 5}}, CellFrameColor -> 
       RGBColor[0.09375, 0.570313, 0.816406]], 
      Cell[
       StyleData["Section", "DoubleSpacedPrintout"], 
       CellFrame -> {{4, 0}, {0, 0.5}}, CellMargins -> {{2, 2}, {3, 21}}, 
       PageBreakBelow -> False, CellFrameMargins -> {{6, 6}, {3, 5}}, 
       CellFrameColor -> RGBColor[0.09375, 0.570313, 0.816406], FontSize -> 
       14]}, Open]], 
   Cell[
    CellGroupData[{
      Cell[
       StyleData["Subsubsection"], CellDingbat -> None, 
       CellMargins -> {{12, 12}, {0, 6}}, 
       CellElementSpacings -> {"ClosedGroupTopMargin" -> 12}, 
       CellGroupingRules -> {"SectionGrouping", 50}, 
       ConversionRules :> {
        "TeX" -> {"\n\\subsubsection{", "}"}, 
         "HTML" -> {"<h5 class=\"Subsubsection\">", "</h5>"}}, PageBreakBelow -> 
       False, CellFrameLabelMargins -> 4, 
       CounterIncrements -> {
        "Subsubsection", "SubsubsectionHeader", "SubsubsubsectionHeader"}, 
       FontFamily -> "Helvetica", FontSize -> 11, FontWeight -> "Bold", 
       FontSlant -> "Italic"], 
      Cell[
       StyleData["Subsubsection", "Printout"], PageBreakBelow -> False, 
       FontFamily -> "Helvetica", FontSize -> 11, FontWeight -> "Bold", 
       FontSlant -> "Italic"]}, Open]], 
   Cell[
    StyleData["Equation"], CellMargins -> {{26, 10}, {7, 7}}, 
    ConversionRules :> {
     "HTML" -> {{"", ""}, {"<p class=\"Equation\">", "</p>"}}}, 
    CellHorizontalScrolling -> True, DefaultFormatType -> TraditionalForm, 
    InputAutoReplacements -> {}, TextAlignment -> Center, CounterIncrements -> 
    "Equation", ButtonBoxOptions -> {Appearance -> {Automatic, None}}], 
   Cell[
    StyleData["Subsubsubsection"], CellDingbat -> None, 
    CellMargins -> {{12, 12}, {0, 6}}, 
    CellGroupingRules -> {"SectionGrouping", 60}, 
    ConversionRules :> {
     "TeX" -> {"\n\\subsubsubsection{", "}"}, 
      "HTML" -> {"<h6 class=\"Subsubsubsection\">", "</h6>"}}, PageBreakBelow -> 
    False, CellFrameLabelMargins -> 4, 
    CounterIncrements -> {"Subsubsubsection", "SubsubsubsectionHeader"}, 
    FontFamily -> "Helvetica", FontSize -> 11, FontSlant -> "Italic"]}, 
  Visible -> False, FrontEndVersion -> 
  "9.0 for Mac OS X x86 (32-bit, 64-bit Kernel) (January 25, 2013)", 
  StyleDefinitions -> "PrivateStylesheetFormatting.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "Section1"->{
  Cell[1543, 50, 66, 1, 55, "Section",
   CellTags->"Section1"]},
 "Section1a"->{
  Cell[16826, 557, 68, 1, 32, "Subsection",
   CellTags->"Section1a"]},
 "Section1b"->{
  Cell[25573, 825, 65, 1, 26, "Subsection",
   CellTags->"Section1b"]},
 "Section2"->{
  Cell[39107, 1262, 81, 1, 35, "Section",
   CellTags->"Section2"]},
 "Section2a"->{
  Cell[40145, 1295, 120, 3, 32, "Subsection",
   CellTags->"Section2a"]},
 "Section2b"->{
  Cell[89336, 3139, 81, 1, 26, "Subsection",
   CellTags->"Section2b"]},
 "Section2c"->{
  Cell[101157, 3594, 147, 5, 26, "Subsection",
   CellTags->"Section2c"]},
 "Section2d"->{
  Cell[117952, 4207, 161, 6, 26, "Subsection",
   CellTags->"Section2d"]},
 "Section3"->{
  Cell[125115, 4426, 100, 1, 35, "Section",
   CellTags->"Section3"]},
 "Section4"->{
  Cell[131448, 4620, 66, 1, 35, "Section",
   CellTags->"Section4"]},
 "Section4a"->{
  Cell[133178, 4683, 72, 1, 32, "Subsection",
   CellTags->"Section4a"]},
 "Section4b"->{
  Cell[145713, 5106, 88, 1, 26, "Subsection",
   CellTags->"Section4b"]},
 "Section5"->{
  Cell[156721, 5501, 65, 1, 35, "Section",
   CellTags->"Section5"]},
 "Section6"->{
  Cell[168061, 5896, 81, 1, 35, "Section",
   CellTags->"Section6"]},
 "Solutions"->{
  Cell[179739, 6250, 140, 4, 35, "Section",
   PageBreakWithin->False,
   CellTags->"Solutions"]},
 "SolutionsCP12"->{
  Cell[179904, 6258, 119, 3, 32, "Subsection",
   PageBreakAbove->False,
   PageBreakWithin->False,
   CellTags->"SolutionsCP12"]},
 "SolutionsCE2"->{
  Cell[185133, 6450, 80, 1, 26, "Subsection",
   CellTags->"SolutionsCE2"]},
 "SolutionsCE3"->{
  Cell[188912, 6575, 80, 1, 26, "Subsection",
   CellTags->"SolutionsCE3"]},
 "SolutionsCE11"->{
  Cell[192365, 6682, 82, 1, 26, "Subsection",
   CellTags->"SolutionsCE11"]},
 "Exercises"->{
  Cell[198858, 6897, 52, 1, 35, "Section",
   CellTags->"Exercises"]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"Section1", 222581, 7447},
 {"Section1a", 222663, 7450},
 {"Section1b", 222751, 7453},
 {"Section2", 222838, 7456},
 {"Section2a", 222923, 7459},
 {"Section2b", 223013, 7462},
 {"Section2c", 223102, 7465},
 {"Section2d", 223193, 7468},
 {"Section3", 223283, 7471},
 {"Section4", 223369, 7474},
 {"Section4a", 223455, 7477},
 {"Section4b", 223545, 7480},
 {"Section5", 223634, 7483},
 {"Section6", 223719, 7486},
 {"Solutions", 223805, 7489},
 {"SolutionsCP12", 223924, 7493},
 {"SolutionsCE2", 224075, 7498},
 {"SolutionsCE3", 224171, 7501},
 {"SolutionsCE11", 224268, 7504},
 {"Exercises", 224362, 7507}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[579, 22, 101, 1, 96, "BookChapterTitle",
 CounterAssignments->{{"ChapterNumber", 8}}],
Cell[CellGroupData[{
Cell[705, 27, 31, 0, 55, "Section"],
Cell[739, 29, 767, 16, 80, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1543, 50, 66, 1, 55, "Section",
 CellTags->"Section1"],
Cell[1612, 53, 320, 9, 35, "Text"],
Cell[1935, 64, 228, 8, 26, "Equation"],
Cell[2166, 74, 443, 17, 20, "Text"],
Cell[2612, 93, 615, 14, 65, "Text"],
Cell[3230, 109, 797, 27, 50, "Text"],
Cell[4030, 138, 393, 6, 50, "Text"],
Cell[4426, 146, 577, 19, 60, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[5028, 169, 178, 5, 28, "Input"],
Cell[5209, 176, 294, 6, 28, "Output"]
}, Open  ]],
Cell[5518, 185, 383, 8, 50, "Text"],
Cell[5904, 195, 1757, 53, 116, "Text"],
Cell[7664, 250, 111, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[7800, 257, 556, 17, 28, "Input"],
Cell[8359, 276, 294, 6, 28, "Output"]
}, Open  ]],
Cell[8668, 285, 385, 10, 36, "Text"],
Cell[CellGroupData[{
Cell[9078, 299, 63, 1, 28, "Input"],
Cell[9144, 302, 30, 0, 28, "Output"]
}, Open  ]],
Cell[9189, 305, 1355, 44, 71, "Text"],
Cell[CellGroupData[{
Cell[10569, 353, 668, 21, 28, "Input"],
Cell[11240, 376, 71, 0, 28, "Output"]
}, Open  ]],
Cell[11326, 379, 816, 19, 53, "Text"],
Cell[12145, 400, 455, 16, 26, "Equation"],
Cell[12603, 418, 1177, 32, 66, "Text"],
Cell[13783, 452, 1039, 35, 36, "Text"],
Cell[CellGroupData[{
Cell[14847, 491, 757, 23, 44, "Input"],
Cell[15607, 516, 450, 11, 44, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[16094, 532, 242, 7, 28, "Input"],
Cell[16339, 541, 450, 11, 44, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[16826, 557, 68, 1, 32, "Subsection",
 CellTags->"Section1a"],
Cell[16897, 560, 209, 4, 32, "Text"],
Cell[17109, 566, 291, 10, 22, "Equation"],
Cell[17403, 578, 290, 9, 32, "Text"],
Cell[17696, 589, 154, 5, 22, "Equation"],
Cell[17853, 596, 138, 5, 18, "Text"],
Cell[17994, 603, 569, 16, 60, "Text"],
Cell[18566, 621, 168, 4, 21, "Text"],
Cell[18737, 627, 305, 8, 45, "Input",
 InitializationCell->True],
Cell[19045, 637, 234, 6, 18, "Text"],
Cell[19282, 645, 994, 24, 98, "Text"],
Cell[20279, 671, 885, 21, 124, "Input",
 InitializationCell->True],
Cell[21167, 694, 183, 4, 21, "Text"],
Cell[21353, 700, 292, 8, 29, "Input",
 InitializationCell->True],
Cell[21648, 710, 165, 5, 21, "Text"],
Cell[CellGroupData[{
Cell[21838, 719, 58, 1, 28, "Input"],
Cell[CellGroupData[{
Cell[21921, 724, 323, 6, 22, "Print"],
Cell[22247, 732, 323, 6, 22, "Print"],
Cell[22573, 740, 323, 6, 22, "Print"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[22945, 752, 58, 1, 28, "Input"],
Cell[CellGroupData[{
Cell[23028, 757, 323, 6, 22, "Print"],
Cell[23354, 765, 323, 6, 22, "Print"],
Cell[23680, 773, 323, 6, 22, "Print"],
Cell[24006, 781, 323, 6, 22, "Print"],
Cell[24332, 789, 323, 6, 22, "Print"],
Cell[24658, 797, 323, 6, 22, "Print"],
Cell[24984, 805, 323, 6, 22, "Print"]
}, Open  ]]
}, Open  ]],
Cell[25334, 815, 202, 5, 32, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[25573, 825, 65, 1, 26, "Subsection",
 CellTags->"Section1b"],
Cell[25641, 828, 248, 4, 35, "Text"],
Cell[25892, 834, 476, 7, 65, "Text"],
Cell[26371, 843, 67, 0, 20, "Text"],
Cell[26441, 845, 1323, 48, 167, "Text"],
Cell[27767, 895, 86, 2, 20, "Text"],
Cell[CellGroupData[{
Cell[27878, 901, 709, 20, 44, "Input",
 InitializationCell->True],
Cell[28590, 923, 645, 18, 44, "Output"]
}, Open  ]],
Cell[29250, 944, 135, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[29410, 951, 73, 0, 16, "Item1Numbered"],
Cell[29486, 953, 442, 18, 16, "Item1Numbered"],
Cell[29931, 973, 742, 28, 35, "Item1Numbered"],
Cell[30676, 1003, 252, 9, 16, "Item1Numbered"]
}, Open  ]],
Cell[30943, 1015, 1084, 28, 83, "Text"],
Cell[32030, 1045, 178, 6, 28, "myCode"],
Cell[32211, 1053, 640, 19, 51, "Text"],
Cell[32854, 1074, 1038, 26, 81, "Text"],
Cell[33895, 1102, 1821, 46, 232, "Input",
 InitializationCell->True],
Cell[35719, 1150, 867, 27, 56, "Text"],
Cell[36589, 1179, 1170, 32, 123, "Input",
 InitializationCell->True],
Cell[37762, 1213, 105, 3, 20, "Text"],
Cell[37870, 1218, 951, 26, 109, "Input",
 InitializationCell->True],
Cell[38824, 1246, 91, 2, 20, "Text"],
Cell[CellGroupData[{
Cell[38940, 1252, 74, 1, 28, "Input"],
Cell[39017, 1255, 29, 0, 28, "Output"]
}, Open  ]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[39107, 1262, 81, 1, 35, "Section",
 CellTags->"Section2"],
Cell[39191, 1265, 498, 13, 51, "Text"],
Cell[39692, 1280, 428, 11, 35, "Text"],
Cell[CellGroupData[{
Cell[40145, 1295, 120, 3, 32, "Subsection",
 CellTags->"Section2a"],
Cell[40268, 1300, 305, 11, 35, "Text"],
Cell[40576, 1313, 455, 16, 26, "Equation"],
Cell[41034, 1331, 594, 20, 35, "Text"],
Cell[41631, 1353, 144, 3, 20, "Text"],
Cell[41778, 1358, 444, 15, 29, "Equation"],
Cell[42225, 1375, 292, 7, 35, "Text"],
Cell[CellGroupData[{
Cell[42542, 1386, 40, 0, 18, "Subsubsection"],
Cell[42585, 1388, 119, 3, 20, "Text"],
Cell[42707, 1393, 272, 10, 26, "Equation"],
Cell[42982, 1405, 277, 12, 20, "Text"],
Cell[43262, 1419, 51, 0, 20, "Text"],
Cell[43316, 1421, 161, 5, 29, "Equation"],
Cell[43480, 1428, 324, 10, 36, "Text"],
Cell[CellGroupData[{
Cell[43829, 1442, 197, 7, 28, "Input"],
Cell[44029, 1451, 204, 7, 28, "Output"]
}, Open  ]],
Cell[44248, 1461, 417, 11, 36, "Text"],
Cell[44668, 1474, 381, 14, 35, "Text"],
Cell[45052, 1490, 317, 10, 26, "Equation"],
Cell[45372, 1502, 95, 2, 20, "Text"],
Cell[45470, 1506, 361, 13, 20, "Text"],
Cell[45834, 1521, 331, 12, 45, "Equation"],
Cell[46168, 1535, 275, 9, 21, "Text"],
Cell[CellGroupData[{
Cell[46468, 1548, 413, 14, 29, "Input"],
Cell[46884, 1564, 236, 8, 46, "Output"]
}, Open  ]],
Cell[47135, 1575, 227, 6, 32, "Text"],
Cell[47365, 1583, 143, 3, 18, "Text"],
Cell[47511, 1588, 342, 13, 41, "Equation"],
Cell[47856, 1603, 326, 10, 35, "Text"],
Cell[48185, 1615, 386, 14, 29, "Input"],
Cell[CellGroupData[{
Cell[48596, 1633, 177, 5, 28, "Input"],
Cell[48776, 1640, 179, 4, 28, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[49004, 1650, 41, 0, 17, "Subsubsection"],
Cell[49048, 1652, 88, 2, 18, "Text"],
Cell[49139, 1656, 342, 13, 41, "Equation"],
Cell[49484, 1671, 290, 13, 27, "Text"],
Cell[49777, 1686, 422, 9, 50, "Text"],
Cell[CellGroupData[{
Cell[50224, 1699, 381, 13, 29, "Input",
 InitializationCell->True],
Cell[50608, 1714, 228, 8, 47, "Output"]
}, Open  ]],
Cell[50851, 1725, 323, 9, 36, "Text"],
Cell[CellGroupData[{
Cell[51199, 1738, 120, 3, 29, "Input",
 InitializationCell->True],
Cell[51322, 1743, 114, 4, 47, "Output"]
}, Open  ]],
Cell[51451, 1750, 291, 7, 32, "Text"],
Cell[51745, 1759, 161, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[51931, 1766, 841, 26, 45, "Input"],
Cell[52775, 1794, 217, 7, 47, "Output"]
}, Open  ]],
Cell[53007, 1804, 76, 0, 18, "Text"],
Cell[53086, 1806, 376, 14, 41, "Equation"]
}, Open  ]],
Cell[CellGroupData[{
Cell[53499, 1825, 47, 0, 17, "Subsubsection"],
Cell[53549, 1827, 165, 3, 32, "Text"],
Cell[53717, 1832, 119, 4, 24, "Equation"],
Cell[53839, 1838, 65, 0, 20, "Text"],
Cell[CellGroupData[{
Cell[53929, 1842, 270, 9, 29, "Input",
 InitializationCell->True],
Cell[54202, 1853, 301, 12, 46, "Output"]
}, Open  ]],
Cell[54518, 1868, 143, 5, 20, "Text"],
Cell[CellGroupData[{
Cell[54686, 1877, 337, 11, 29, "Input",
 InitializationCell->True],
Cell[55026, 1890, 451, 17, 46, "Output"]
}, Open  ]],
Cell[55492, 1910, 119, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[55636, 1917, 868, 27, 61, "Input",
 InitializationCell->True],
Cell[56507, 1946, 693, 26, 59, "Output"]
}, Open  ]],
Cell[57215, 1975, 426, 11, 36, "Text"],
Cell[CellGroupData[{
Cell[57666, 1990, 108, 3, 29, "Input"],
Cell[57777, 1995, 822, 33, 57, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[58636, 2033, 145, 4, 29, "Input"],
Cell[58784, 2039, 564, 22, 62, "Output"]
}, Open  ]],
Cell[59363, 2064, 287, 5, 35, "Text"],
Cell[CellGroupData[{
Cell[59675, 2073, 198, 6, 29, "Input",
 InitializationCell->True],
Cell[59876, 2081, 822, 33, 57, "Output"]
}, Open  ]],
Cell[60713, 2117, 491, 15, 32, "Text"],
Cell[61207, 2134, 146, 5, 20, "Text"],
Cell[CellGroupData[{
Cell[61378, 2143, 63, 1, 28, "Input"],
Cell[61444, 2146, 508, 21, 56, "Output"]
}, Open  ]],
Cell[61967, 2170, 225, 6, 21, "Text"],
Cell[CellGroupData[{
Cell[62217, 2180, 94, 2, 29, "Input"],
Cell[62314, 2184, 28, 0, 28, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[62391, 2190, 33, 0, 17, "Subsubsection"],
Cell[62427, 2192, 414, 9, 60, "Text"],
Cell[62844, 2203, 304, 10, 22, "Equation"],
Cell[63151, 2215, 361, 13, 32, "Text"],
Cell[63515, 2230, 508, 11, 53, "Text"],
Cell[64026, 2243, 2723, 75, 189, "Input",
 InitializationCell->True],
Cell[66752, 2320, 927, 27, 74, "Text"],
Cell[67682, 2349, 234, 4, 35, "Text"],
Cell[CellGroupData[{
Cell[67941, 2357, 191, 5, 29, "Input",
 InitializationCell->True],
Cell[68135, 2364, 822, 33, 57, "Output"]
}, Open  ]],
Cell[68972, 2400, 149, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[69146, 2407, 207, 6, 29, "Input"],
Cell[69356, 2415, 164, 4, 28, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[69569, 2425, 57, 0, 17, "Subsubsection"],
Cell[69629, 2427, 238, 4, 46, "Text"],
Cell[69870, 2433, 272, 10, 22, "Equation"],
Cell[70145, 2445, 43, 0, 20, "Text"],
Cell[CellGroupData[{
Cell[70213, 2449, 196, 6, 29, "Input",
 InitializationCell->True],
Cell[70412, 2457, 139, 4, 30, "Output"]
}, Open  ]],
Cell[70566, 2464, 30, 0, 20, "Text"],
Cell[CellGroupData[{
Cell[70621, 2468, 170, 5, 29, "Input",
 InitializationCell->True],
Cell[70794, 2475, 73, 2, 28, "Output"]
}, Open  ]],
Cell[70882, 2480, 207, 5, 20, "Text"],
Cell[CellGroupData[{
Cell[71114, 2489, 172, 5, 28, "Input"],
Cell[71289, 2496, 31, 0, 28, "Output"]
}, Open  ]],
Cell[71335, 2499, 209, 6, 18, "Text"],
Cell[71547, 2507, 304, 8, 23, "Equation"],
Cell[71854, 2517, 428, 15, 32, "Text"],
Cell[72285, 2534, 585, 17, 44, "Equation"],
Cell[72873, 2553, 81, 2, 20, "Text"],
Cell[CellGroupData[{
Cell[72979, 2559, 948, 29, 61, "Input",
 InitializationCell->True],
Cell[73930, 2590, 163, 5, 46, "Output"]
}, Open  ]],
Cell[74108, 2598, 399, 12, 22, "Text"],
Cell[CellGroupData[{
Cell[74532, 2614, 325, 11, 29, "Input"],
Cell[74860, 2627, 118, 4, 30, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[75027, 2637, 57, 0, 17, "Subsubsection"],
Cell[75087, 2639, 371, 6, 60, "Text"],
Cell[75461, 2647, 304, 10, 22, "Equation"],
Cell[75768, 2659, 267, 12, 20, "Text"],
Cell[76038, 2673, 5050, 132, 300, "Input",
 InitializationCell->True],
Cell[81091, 2807, 248, 6, 36, "Text"],
Cell[CellGroupData[{
Cell[81364, 2817, 106, 2, 28, "Input"],
Cell[81473, 2821, 822, 33, 57, "Output"]
}, Open  ]],
Cell[82310, 2857, 55, 0, 20, "Text"],
Cell[CellGroupData[{
Cell[82390, 2861, 125, 3, 28, "Input"],
Cell[82518, 2866, 118, 4, 30, "Output"]
}, Open  ]],
Cell[82651, 2873, 254, 4, 46, "Text"],
Cell[82908, 2879, 451, 16, 26, "Equation"],
Cell[CellGroupData[{
Cell[83384, 2899, 206, 6, 29, "Input",
 InitializationCell->True],
Cell[83593, 2907, 163, 6, 49, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[83793, 2918, 207, 6, 29, "Input"],
Cell[84003, 2926, 175, 4, 28, "Output"]
}, Open  ]],
Cell[84193, 2933, 86, 2, 18, "Text"],
Cell[84282, 2937, 250, 9, 22, "Equation"],
Cell[84535, 2948, 267, 12, 20, "Text"],
Cell[CellGroupData[{
Cell[84827, 2964, 226, 7, 29, "Input",
 InitializationCell->True],
Cell[85056, 2973, 1086, 41, 55, "Output"]
}, Open  ]],
Cell[86157, 3017, 213, 4, 35, "Text"],
Cell[CellGroupData[{
Cell[86395, 3025, 207, 6, 29, "Input"],
Cell[86605, 3033, 210, 5, 28, "Output"]
}, Open  ]],
Cell[86830, 3041, 185, 5, 20, "Text"],
Cell[CellGroupData[{
Cell[87040, 3050, 238, 7, 29, "Input",
 InitializationCell->True],
Cell[87281, 3059, 1149, 43, 55, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[88467, 3107, 207, 6, 29, "Input"],
Cell[88677, 3115, 376, 11, 28, "Output"]
}, Open  ]],
Cell[89068, 3129, 219, 4, 32, "Text"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[89336, 3139, 81, 1, 26, "Subsection",
 CellTags->"Section2b"],
Cell[89420, 3142, 380, 8, 50, "Text"],
Cell[89803, 3152, 495, 17, 26, "Equation"],
Cell[90301, 3171, 748, 22, 50, "Text"],
Cell[91052, 3195, 48, 0, 20, "Text"],
Cell[91103, 3197, 345, 12, 26, "Equation"],
Cell[91451, 3211, 59, 0, 20, "Text"],
Cell[91513, 3213, 115, 3, 20, "Text"],
Cell[91631, 3218, 272, 10, 26, "Equation"],
Cell[91906, 3230, 194, 5, 21, "Text"],
Cell[CellGroupData[{
Cell[92125, 3239, 227, 6, 29, "Input",
 InitializationCell->True],
Cell[92355, 3247, 422, 15, 47, "Output"]
}, Open  ]],
Cell[92792, 3265, 688, 15, 66, "Text"],
Cell[93483, 3282, 218, 7, 29, "Equation"],
Cell[93704, 3291, 91, 2, 20, "Text"],
Cell[CellGroupData[{
Cell[93820, 3297, 265, 9, 29, "Input",
 InitializationCell->True],
Cell[94088, 3308, 180, 6, 31, "Output"]
}, Open  ]],
Cell[94283, 3317, 539, 19, 36, "Text"],
Cell[CellGroupData[{
Cell[94847, 3340, 401, 13, 29, "Input",
 InitializationCell->True],
Cell[95251, 3355, 964, 36, 31, "Output"]
}, Open  ]],
Cell[96230, 3394, 195, 6, 21, "Text"],
Cell[CellGroupData[{
Cell[96450, 3404, 117, 3, 29, "Input",
 InitializationCell->True],
Cell[96570, 3409, 372, 13, 30, "Output"]
}, Open  ]],
Cell[96957, 3425, 659, 26, 35, "Text"],
Cell[97619, 3453, 426, 15, 26, "Equation"],
Cell[98048, 3470, 632, 17, 52, "Text"],
Cell[CellGroupData[{
Cell[98705, 3491, 251, 8, 29, "Input",
 InitializationCell->True],
Cell[98959, 3501, 207, 7, 47, "Output"]
}, Open  ]],
Cell[99181, 3511, 49, 0, 20, "Text"],
Cell[CellGroupData[{
Cell[99255, 3515, 138, 4, 29, "Input"],
Cell[99396, 3521, 201, 7, 47, "Output"]
}, Open  ]],
Cell[99612, 3531, 548, 18, 20, "Text"],
Cell[CellGroupData[{
Cell[100185, 3553, 227, 7, 29, "Input"],
Cell[100415, 3562, 612, 22, 47, "Output"]
}, Open  ]],
Cell[101042, 3587, 78, 2, 18, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[101157, 3594, 147, 5, 26, "Subsection",
 CellTags->"Section2c"],
Cell[101307, 3601, 317, 9, 35, "Text"],
Cell[101627, 3612, 1670, 45, 132, "Text"],
Cell[103300, 3659, 552, 20, 20, "Text"],
Cell[CellGroupData[{
Cell[103877, 3683, 578, 17, 29, "Input"],
Cell[104458, 3702, 373, 13, 47, "Output"]
}, Open  ]],
Cell[104846, 3718, 320, 7, 36, "Text"],
Cell[105169, 3727, 471, 15, 37, "Text"],
Cell[CellGroupData[{
Cell[105665, 3746, 120, 3, 29, "Input"],
Cell[105788, 3751, 241, 9, 47, "Output"]
}, Open  ]],
Cell[106044, 3763, 349, 8, 36, "Text"],
Cell[106396, 3773, 402, 14, 47, "myCode"],
Cell[106801, 3789, 684, 24, 47, "Text"],
Cell[107488, 3815, 471, 15, 37, "Text"],
Cell[CellGroupData[{
Cell[107984, 3834, 581, 18, 29, "Input",
 InitializationCell->True],
Cell[108568, 3854, 454, 16, 47, "Output"]
}, Open  ]],
Cell[109037, 3873, 512, 12, 52, "Text"],
Cell[CellGroupData[{
Cell[109574, 3889, 144, 4, 29, "Input",
 InitializationCell->True],
Cell[109721, 3895, 340, 12, 47, "Output"]
}, Open  ]],
Cell[110076, 3910, 96, 4, 21, "Text"],
Cell[CellGroupData[{
Cell[110197, 3918, 55, 1, 28, "Input"],
Cell[110255, 3921, 40, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[110332, 3926, 170, 5, 28, "Input"],
Cell[110505, 3933, 168, 4, 28, "Output"]
}, Open  ]],
Cell[110688, 3940, 214, 6, 21, "Text"],
Cell[CellGroupData[{
Cell[110927, 3950, 570, 17, 28, "Input"],
Cell[111500, 3969, 168, 4, 28, "Output"]
}, Open  ]],
Cell[111683, 3976, 323, 8, 32, "Text"],
Cell[112009, 3986, 216, 7, 22, "Equation"],
Cell[112228, 3995, 158, 7, 20, "Text"],
Cell[CellGroupData[{
Cell[112411, 4006, 618, 19, 44, "Input",
 InitializationCell->True],
Cell[113032, 4027, 179, 6, 33, "Output"]
}, Open  ]],
Cell[113226, 4036, 744, 20, 60, "Text"],
Cell[113973, 4058, 523, 14, 52, "Text"],
Cell[CellGroupData[{
Cell[114521, 4076, 567, 18, 44, "Input",
 InitializationCell->True],
Cell[115091, 4096, 529, 18, 38, "Output"]
}, Open  ]],
Cell[115635, 4117, 153, 5, 21, "Text"],
Cell[CellGroupData[{
Cell[115813, 4126, 54, 1, 28, "Input"],
Cell[115870, 4129, 418, 15, 35, "Output"]
}, Open  ]],
Cell[116303, 4147, 709, 19, 52, "Text"],
Cell[CellGroupData[{
Cell[117037, 4170, 389, 13, 29, "Input"],
Cell[117429, 4185, 474, 16, 33, "Output"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[117952, 4207, 161, 6, 26, "Subsection",
 CellTags->"Section2d"],
Cell[118116, 4215, 335, 7, 35, "Text"],
Cell[118454, 4224, 265, 5, 35, "Text"],
Cell[118722, 4231, 1146, 36, 80, "Text"],
Cell[119871, 4269, 844, 17, 252, "Text"],
Cell[120718, 4288, 1896, 54, 125, "Text"],
Cell[122617, 4344, 1060, 21, 252, "Text"],
Cell[123680, 4367, 236, 7, 20, "Text"],
Cell[CellGroupData[{
Cell[123941, 4378, 585, 18, 44, "Input",
 InitializationCell->True],
Cell[124529, 4398, 245, 8, 46, "Output"]
}, Open  ]],
Cell[124789, 4409, 147, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[124961, 4416, 57, 1, 28, "Input"],
Cell[125021, 4419, 33, 0, 28, "Output"]
}, Open  ]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[125115, 4426, 100, 1, 35, "Section",
 CellTags->"Section3"],
Cell[125218, 4429, 412, 6, 50, "Text"],
Cell[125633, 4437, 1728, 44, 277, "Input",
 InitializationCell->True],
Cell[127364, 4483, 1024, 29, 84, "Text"],
Cell[128391, 4514, 204, 4, 35, "Text"],
Cell[128598, 4520, 258, 8, 26, "Equation"],
Cell[128859, 4530, 961, 28, 68, "Text"],
Cell[129823, 4560, 193, 7, 26, "Equation"],
Cell[130019, 4569, 654, 17, 52, "Text"],
Cell[CellGroupData[{
Cell[130698, 4590, 389, 12, 28, "Input"],
Cell[131090, 4604, 309, 10, 49, "Output"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[131448, 4620, 66, 1, 35, "Section",
 CellTags->"Section4"],
Cell[131517, 4623, 311, 7, 35, "Text"],
Cell[131831, 4632, 285, 11, 20, "Text"],
Cell[132119, 4645, 678, 23, 52, "Equation"],
Cell[132800, 4670, 353, 9, 35, "Text"],
Cell[CellGroupData[{
Cell[133178, 4683, 72, 1, 32, "Subsection",
 CellTags->"Section4a"],
Cell[133253, 4686, 149, 4, 20, "Text"],
Cell[133405, 4692, 791, 20, 66, "Text"],
Cell[134199, 4714, 228, 7, 26, "Text"],
Cell[CellGroupData[{
Cell[134452, 4725, 128, 3, 29, "Input"],
Cell[134583, 4730, 95, 3, 47, "Output"]
}, Open  ]],
Cell[134693, 4736, 333, 8, 35, "Text"],
Cell[135029, 4746, 670, 16, 51, "Text"],
Cell[135702, 4764, 727, 22, 83, "Text"],
Cell[CellGroupData[{
Cell[136454, 4790, 303, 10, 28, "Input",
 InitializationCell->True],
Cell[136760, 4802, 939, 29, 31, "Output"]
}, Open  ]],
Cell[137714, 4834, 306, 9, 24, "Text"],
Cell[138023, 4845, 329, 8, 36, "Text"],
Cell[CellGroupData[{
Cell[138377, 4857, 98, 2, 28, "Input"],
Cell[138478, 4861, 190, 4, 28, "Output"]
}, Open  ]],
Cell[138683, 4868, 1000, 32, 52, "Text"],
Cell[139686, 4902, 732, 19, 53, "Text"],
Cell[CellGroupData[{
Cell[140443, 4925, 819, 23, 59, "Input",
 InitializationCell->True],
Cell[141265, 4950, 154, 5, 46, "Output"]
}, Open  ]],
Cell[141434, 4958, 194, 6, 21, "Text"],
Cell[CellGroupData[{
Cell[141653, 4968, 189, 4, 29, "Input",
 InitializationCell->True],
Cell[141845, 4974, 219, 8, 53, "Output"]
}, Open  ]],
Cell[142079, 4985, 200, 6, 21, "Text"],
Cell[CellGroupData[{
Cell[142304, 4995, 227, 6, 28, "Input",
 InitializationCell->True],
Cell[142534, 5003, 580, 18, 31, "Output"]
}, Open  ]],
Cell[143129, 5024, 1110, 26, 85, "Text"],
Cell[CellGroupData[{
Cell[144264, 5054, 108, 2, 28, "Input"],
Cell[144375, 5058, 28, 0, 28, "Output"]
}, Open  ]],
Cell[144418, 5061, 1020, 29, 52, "Text"],
Cell[CellGroupData[{
Cell[145463, 5094, 170, 4, 28, "Input"],
Cell[145636, 5100, 28, 0, 28, "Output"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[145713, 5106, 88, 1, 26, "Subsection",
 CellTags->"Section4b"],
Cell[145804, 5109, 538, 10, 65, "Text"],
Cell[146345, 5121, 371, 7, 41, "Item1Paragraph"],
Cell[146719, 5130, 233, 7, 20, "Text"],
Cell[146955, 5139, 587, 17, 33, "Equation"],
Cell[147545, 5158, 452, 9, 51, "Text"],
Cell[CellGroupData[{
Cell[148022, 5171, 161, 4, 29, "Input",
 InitializationCell->True],
Cell[148186, 5177, 71, 2, 46, "Output"]
}, Open  ]],
Cell[148272, 5182, 931, 25, 71, "Text"],
Cell[149206, 5209, 685, 19, 52, "Text"],
Cell[CellGroupData[{
Cell[149916, 5232, 282, 9, 29, "Input",
 InitializationCell->True],
Cell[150201, 5243, 96, 3, 47, "Output"]
}, Open  ]],
Cell[150312, 5249, 165, 5, 20, "Text"],
Cell[150480, 5256, 404, 13, 46, "Text"],
Cell[CellGroupData[{
Cell[150909, 5273, 180, 6, 28, "Input",
 InitializationCell->True],
Cell[151092, 5281, 96, 3, 47, "Output"]
}, Open  ]],
Cell[151203, 5287, 488, 17, 24, "Text"],
Cell[CellGroupData[{
Cell[151716, 5308, 148, 4, 28, "Input"],
Cell[151867, 5314, 497, 14, 31, "Output"]
}, Open  ]],
Cell[152379, 5331, 271, 7, 36, "Text"],
Cell[CellGroupData[{
Cell[152675, 5342, 132, 3, 28, "Input",
 InitializationCell->True],
Cell[152810, 5347, 282, 10, 52, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[153129, 5362, 146, 4, 28, "Input"],
Cell[153278, 5368, 678, 21, 31, "Output"]
}, Open  ]],
Cell[153971, 5392, 461, 13, 41, "Text"],
Cell[CellGroupData[{
Cell[154457, 5409, 159, 4, 28, "Input"],
Cell[154619, 5415, 31, 0, 28, "Output"]
}, Open  ]],
Cell[154665, 5418, 158, 3, 20, "Text"],
Cell[154826, 5423, 577, 19, 52, "Equation"],
Cell[155406, 5444, 299, 10, 21, "Text"],
Cell[CellGroupData[{
Cell[155730, 5458, 348, 11, 29, "Input",
 InitializationCell->True],
Cell[156081, 5471, 141, 4, 47, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[156259, 5480, 158, 4, 28, "Input"],
Cell[156420, 5486, 29, 0, 28, "Output"]
}, Open  ]],
Cell[156464, 5489, 208, 6, 23, "Text"]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[156721, 5501, 65, 1, 35, "Section",
 CellTags->"Section5"],
Cell[156789, 5504, 221, 6, 35, "Text"],
Cell[157013, 5512, 95, 2, 20, "Text"],
Cell[157111, 5516, 313, 11, 30, "Equation"],
Cell[157427, 5529, 809, 26, 50, "Text"],
Cell[158239, 5557, 220, 6, 35, "Text"],
Cell[CellGroupData[{
Cell[158484, 5567, 133, 4, 28, "Input",
 InitializationCell->True],
Cell[158620, 5573, 83, 2, 28, "Output"]
}, Open  ]],
Cell[158718, 5578, 187, 6, 21, "Text"],
Cell[CellGroupData[{
Cell[158930, 5588, 59, 1, 29, "Input"],
Cell[158992, 5591, 28, 0, 28, "Output"]
}, Open  ]],
Cell[159035, 5594, 464, 12, 36, "Text"],
Cell[159502, 5608, 328, 11, 44, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[159855, 5623, 81, 2, 28, "Input"],
Cell[159939, 5627, 139, 4, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[160115, 5636, 56, 1, 28, "Input"],
Cell[160174, 5639, 139, 4, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[160350, 5648, 88, 2, 28, "Input"],
Cell[160441, 5652, 73, 2, 28, "Output"]
}, Open  ]],
Cell[160529, 5657, 371, 13, 21, "Text"],
Cell[CellGroupData[{
Cell[160925, 5674, 86, 2, 29, "Input"],
Cell[161014, 5678, 83, 2, 28, "Output"]
}, Open  ]],
Cell[161112, 5683, 132, 3, 20, "Text"],
Cell[161247, 5688, 834, 26, 93, "Input",
 InitializationCell->True],
Cell[162084, 5716, 112, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[162221, 5723, 62, 1, 29, "Input"],
Cell[162286, 5726, 28, 0, 28, "Output"]
}, Open  ]],
Cell[162329, 5729, 99, 2, 20, "Text"],
Cell[CellGroupData[{
Cell[162453, 5735, 128, 3, 29, "Input"],
Cell[162584, 5740, 28, 0, 28, "Output"]
}, Open  ]],
Cell[162627, 5743, 115, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[162767, 5750, 249, 6, 29, "Input"],
Cell[163019, 5758, 28, 0, 28, "Output"]
}, Open  ]],
Cell[163062, 5761, 40, 0, 20, "Text"],
Cell[163105, 5763, 260, 5, 35, "Text"],
Cell[CellGroupData[{
Cell[163390, 5772, 116, 3, 29, "Input",
 InitializationCell->True],
Cell[163509, 5777, 1208, 18, 89, "Output"]
}, Open  ]],
Cell[164732, 5798, 72, 0, 20, "Text"],
Cell[CellGroupData[{
Cell[164829, 5802, 291, 9, 29, "Input",
 InitializationCell->True],
Cell[165123, 5813, 629, 10, 59, "Output"]
}, Open  ]],
Cell[165767, 5826, 52, 0, 20, "Text"],
Cell[CellGroupData[{
Cell[165844, 5830, 292, 9, 29, "Input",
 InitializationCell->True],
Cell[166139, 5841, 1101, 16, 74, "Output"]
}, Open  ]],
Cell[167255, 5860, 140, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[167420, 5867, 126, 3, 29, "Input"],
Cell[167549, 5872, 29, 0, 28, "Output"]
}, Open  ]],
Cell[167593, 5875, 117, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[167735, 5882, 245, 6, 29, "Input"],
Cell[167983, 5890, 29, 0, 28, "Output"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[168061, 5896, 81, 1, 35, "Section",
 CellTags->"Section6"],
Cell[168145, 5899, 395, 6, 50, "Text"],
Cell[168543, 5907, 437, 7, 50, "Text"],
Cell[168983, 5916, 495, 16, 61, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[169503, 5936, 156, 4, 28, "Input",
 InitializationCell->True],
Cell[169662, 5942, 321, 9, 29, "Output"]
}, Open  ]],
Cell[169998, 5954, 219, 4, 35, "Text"],
Cell[170220, 5960, 253, 4, 35, "Text"],
Cell[170476, 5966, 1391, 35, 169, "Input",
 InitializationCell->True],
Cell[171870, 6003, 2662, 73, 149, "Text"],
Cell[174535, 6078, 137, 4, 21, "Text"],
Cell[174675, 6084, 566, 15, 45, "Input",
 InitializationCell->True],
Cell[175244, 6101, 99, 2, 20, "Text"],
Cell[CellGroupData[{
Cell[175368, 6107, 71, 1, 29, "Input"],
Cell[175442, 6110, 32, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[175511, 6115, 71, 1, 29, "Input"],
Cell[175585, 6118, 31, 0, 28, "Output"]
}, Open  ]],
Cell[175631, 6121, 805, 18, 67, "Text"],
Cell[176439, 6141, 669, 18, 52, "Text"],
Cell[CellGroupData[{
Cell[177133, 6163, 64, 1, 28, "Input"],
Cell[177200, 6166, 209, 4, 29, "Output"]
}, Open  ]],
Cell[177424, 6173, 651, 16, 51, "Text"],
Cell[178078, 6191, 768, 18, 67, "Text"],
Cell[178849, 6211, 263, 8, 29, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[179137, 6223, 70, 1, 29, "Input"],
Cell[179210, 6226, 30, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[179277, 6231, 90, 2, 29, "Input"],
Cell[179370, 6235, 209, 4, 29, "Output"]
}, Open  ]],
Cell[179594, 6242, 108, 3, 20, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[179739, 6250, 140, 4, 35, "Section",
 PageBreakWithin->False,
 CellTags->"Solutions"],
Cell[CellGroupData[{
Cell[179904, 6258, 119, 3, 32, "Subsection",
 PageBreakAbove->False,
 PageBreakWithin->False,
 CellTags->"SolutionsCP12"],
Cell[180026, 6263, 418, 16, 42, "Problem",
 PageBreakAbove->False,
 PageBreakWithin->False],
Cell[180447, 6281, 113, 4, 20, "Text"],
Cell[180563, 6287, 433, 15, 54, "Equation"],
Cell[180999, 6304, 1199, 33, 82, "Text"],
Cell[182201, 6339, 197, 6, 54, "Equation"],
Cell[182401, 6347, 58, 0, 20, "Text"],
Cell[CellGroupData[{
Cell[182484, 6351, 162, 5, 28, "Input"],
Cell[182649, 6358, 52, 1, 47, "Output"]
}, Open  ]],
Cell[182716, 6362, 73, 0, 20, "Text"],
Cell[182792, 6364, 801, 26, 45, "Input",
 InitializationCell->True],
Cell[183596, 6392, 628, 15, 52, "Text"],
Cell[CellGroupData[{
Cell[184249, 6411, 89, 2, 28, "Input"],
Cell[184341, 6415, 28, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[184406, 6420, 108, 3, 28, "Input"],
Cell[184517, 6425, 109, 3, 28, "Output"]
}, Open  ]],
Cell[184641, 6431, 161, 3, 35, "Text"],
Cell[CellGroupData[{
Cell[184827, 6438, 92, 2, 28, "Input"],
Cell[184922, 6442, 162, 2, 44, "Output"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[185133, 6450, 80, 1, 26, "Subsection",
 CellTags->"SolutionsCE2"],
Cell[185216, 6453, 152, 3, 42, "Problem"],
Cell[185371, 6458, 751, 23, 53, "Text"],
Cell[186125, 6483, 392, 9, 36, "Text"],
Cell[186520, 6494, 662, 18, 76, "Input",
 InitializationCell->True],
Cell[187185, 6514, 449, 12, 36, "Text"],
Cell[187637, 6528, 59, 0, 20, "Text"],
Cell[CellGroupData[{
Cell[187721, 6532, 66, 1, 28, "Input"],
Cell[187790, 6535, 278, 6, 21, "Print"]
}, Open  ]],
Cell[CellGroupData[{
Cell[188105, 6546, 69, 1, 28, "Input"],
Cell[188177, 6549, 284, 6, 21, "Print"]
}, Open  ]],
Cell[CellGroupData[{
Cell[188498, 6560, 72, 1, 28, "Input"],
Cell[188573, 6563, 290, 6, 21, "Print"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[188912, 6575, 80, 1, 26, "Subsection",
 CellTags->"SolutionsCE3"],
Cell[188995, 6578, 136, 3, 42, "Problem"],
Cell[189134, 6583, 1544, 44, 84, "Text"],
Cell[190681, 6629, 1136, 30, 169, "Input",
 InitializationCell->True],
Cell[191820, 6661, 92, 2, 20, "Text"],
Cell[CellGroupData[{
Cell[191937, 6667, 64, 1, 29, "Input"],
Cell[192004, 6670, 312, 6, 59, "Output"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[192365, 6682, 82, 1, 26, "Subsection",
 CellTags->"SolutionsCE11"],
Cell[192450, 6685, 352, 10, 55, "Problem"],
Cell[192805, 6697, 250, 8, 20, "Text"],
Cell[193058, 6707, 520, 18, 47, "Equation"],
Cell[193581, 6727, 443, 15, 45, "Text"],
Cell[194027, 6744, 394, 14, 47, "Equation"],
Cell[194424, 6760, 142, 5, 20, "Text"],
Cell[194569, 6767, 437, 15, 29, "Input",
 InitializationCell->True],
Cell[195009, 6784, 252, 9, 20, "Text"],
Cell[CellGroupData[{
Cell[195286, 6797, 171, 5, 29, "Input"],
Cell[195460, 6804, 912, 21, 121, "Output"]
}, Open  ]],
Cell[196387, 6828, 720, 24, 36, "Text"],
Cell[CellGroupData[{
Cell[197132, 6856, 288, 10, 29, "Input"],
Cell[197423, 6868, 1374, 22, 107, "Output"]
}, Open  ]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[198858, 6897, 52, 1, 35, "Section",
 CellTags->"Exercises"],
Cell[CellGroupData[{
Cell[198935, 6902, 117, 3, 20, "Item1Exercise"],
Cell[199055, 6907, 192, 4, 33, "Item1Exercise"],
Cell[199250, 6913, 163, 3, 33, "Item1Exercise"],
Cell[199416, 6918, 143, 5, 20, "Item1Exercise"],
Cell[CellGroupData[{
Cell[199584, 6927, 478, 21, 15, "Item2Exercise"],
Cell[200065, 6950, 592, 25, 26, "Item2Exercise"]
}, Open  ]],
Cell[200672, 6978, 290, 7, 33, "Item1Exercise"],
Cell[200965, 6987, 521, 12, 47, "Item1Exercise"],
Cell[201489, 7001, 139, 5, 20, "Item1Exercise"],
Cell[201631, 7008, 237, 7, 47, "ExerciseEquation"],
Cell[201871, 7017, 490, 17, 29, "ExerciseContinuation"],
Cell[202364, 7036, 164, 5, 20, "Item1Exercise"],
Cell[202531, 7043, 202, 6, 20, "Item1Exercise"],
Cell[202736, 7051, 397, 8, 46, "Item1Exercise"],
Cell[203136, 7061, 1089, 26, 85, "Item1Exercise"],
Cell[204228, 7089, 258, 4, 33, "Item1Exercise"],
Cell[204489, 7095, 292, 6, 34, "Item1Exercise"],
Cell[204784, 7103, 418, 12, 34, "Item1Exercise"]
}, Open  ]]
}, Closed]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
